@@ -12,25 +12,22 @@
 // GNU General Public License for more details.
 //
 
-#include <stdio.h>
+#include "SDL.h"
+
+#include <stdint.h>
 #include <stdlib.h>
-#include <string.h>
-
-#include "doomdef.h"
-
-#include "textscreen.h"
 
 #include "d_iwad.h"
-#include "m_misc2.h"
-#include "d_englsh.h"
-
-#include "multiplayer.h"
+#include "doomdef.h"
+#include "doomkeys.h"
 #include "execute.h"
-
+#include "m_misc2.h"
+#include "multiplayer.h"
+#include "net_defs.h"
 #include "net_io.h"
 #include "net_query.h"
 
-#include "net_petname.h"
+#include "textscreen.h"
 
 #define MULTI_START_HELP_URL "https://www.chocolate-doom.org/setup-multi-start"
 #define MULTI_JOIN_HELP_URL "https://www.chocolate-doom.org/setup-multi-join"
@@ -49,7 +46,7 @@ typedef enum
 // Fallback IWADs to use if no IWADs are detected.
 
 static const iwad_t fallback_iwads[] = {
-    { "doom.wad",    doom,       retail,     "Doom" }
+    { "doom.wad", doom, retail, vanilla, "Doom" }
 };
 
 // Array of IWADs found to be installed
@@ -247,6 +244,8 @@ static void StartGame(int multiplayer)
 
     ExecuteDoom(exec);
 
+    SDL_Quit();
+
     exit(0);
 }
 
