@@ -16,32 +16,34 @@
 //     to the IWAD type.
 //
 
+#include "SDL.h"
+
 #include <stdlib.h>
+#include <string.h>
 
-#include "SDL_filesystem.h" // [FG] SDL_GetBasePath()
-
-#include "i_system.h"
-#include "m_io.h"
 #include "d_iwad.h"
+#include "i_system.h"
 #include "m_argv.h"
+#include "m_io.h"
 #include "m_misc2.h"
 
 static const iwad_t iwads[] =
 {
-    { "doom2.wad",     doom2,      commercial,   "Doom II" },
-    { "plutonia.wad",  pack_plut,  commercial,   "Final Doom: Plutonia Experiment" },
-    { "tnt.wad",       pack_tnt,   commercial,   "Final Doom: TNT: Evilution" },
+    { "doom2.wad",     doom2,      commercial,   vanilla, "Doom II" },
+    { "plutonia.wad",  pack_plut,  commercial,   vanilla, "Final Doom: Plutonia Experiment" },
+    { "tnt.wad",       pack_tnt,   commercial,   vanilla, "Final Doom: TNT: Evilution" },
     // "doom.wad" may be retail or registered
-    { "doom.wad",      doom,       indetermined, "Doom" },
-    { "doom1.wad",     doom,       indetermined, "Doom Shareware" },
-    { "doom2f.wad",    doom2,      commercial,   "Doom II: L'Enfer sur Terre" },
-    { "chex.wad",      pack_chex,  retail,       "Chex Quest" },
-    { "hacx.wad",      pack_hacx,  commercial,   "Hacx" },
-    { "freedoom2.wad", doom2,      commercial,   "Freedoom: Phase 2" },
-    { "freedoom1.wad", doom,       retail,       "Freedoom: Phase 1" },
-    { "freedm.wad",    doom2,      commercial,   "FreeDM" },
-    { "rekkrsa.wad",   pack_rekkr, retail,       "REKKR" },
-    { "rekkrsl.wad",   pack_rekkr, retail,       "REKKR: Sunken Land" },
+    { "doom.wad",      doom,       indetermined, vanilla,  "Doom" },
+    { "doom1.wad",     doom,       indetermined, vanilla,  "Doom Shareware" },
+    { "doom2f.wad",    doom2,      commercial,   vanilla,  "Doom II: L'Enfer sur Terre" },
+    { "chex.wad",      pack_chex,  retail,       vanilla,  "Chex Quest" },
+    { "hacx.wad",      pack_hacx,  commercial,   vanilla,  "Hacx" },
+    { "freedoom2.wad", doom2,      commercial,   freedoom, "Freedoom: Phase 2" },
+    { "freedoom1.wad", doom,       retail,       freedoom, "Freedoom: Phase 1" },
+    { "freedm.wad",    doom2,      commercial,   freedoom, "FreeDM" },
+    { "rekkrsa.wad",   pack_rekkr, retail,       vanilla,  "REKKR" },
+    { "rekkrsl.wad",   pack_rekkr, retail,       vanilla,  "REKKR: Sunken Land" },
+    { "miniwad.wad",   doom2,      commercial,   miniwad,  "miniwad"}
 };
 
 // "128 IWAD search directories should be enough for anybody".
@@ -459,7 +461,7 @@ static void AddIWADPath(const char *path, const char *suffix)
     free(dup_path);
 }
 
-#ifndef _WIN32
+#if !defined(_WIN32)
 // Add standard directories where IWADs are located on Unix systems.
 // To respect the freedesktop.org specification we support overriding
 // using standard environment variables. See the XDG Base Directory
@@ -521,7 +523,7 @@ static void AddXdgDirs(void)
     AddIWADPath(env, "/games/doom3bfg/base/wads");
 }
 
-#ifndef __MACOSX__
+#if !defined(__MACOSX__)
 // Steam on Linux allows installing some select Windows games,
 // including the classic Doom series (running DOSBox via Wine).  We
 // could parse *.vdf files to more accurately detect installation
@@ -589,7 +591,7 @@ void BuildIWADDirList(void)
         AddIWADDir(env);
     }
 
-#ifdef WOOFDATADIR
+#if defined(WOOFDATADIR)
     // [FG] Add a build-time configurable data directory
     AddIWADDir(WOOFDATADIR);
 #endif
@@ -605,7 +607,7 @@ void BuildIWADDirList(void)
 
 #else
     AddXdgDirs();
-#ifndef __MACOSX__
+#if !defined(__MACOSX__)
     AddSteamDirs();
 #endif
 #endif
@@ -695,7 +697,7 @@ char *D_TryFindWADByName(const char *filename)
 // D_FindIWADFile
 //
 
-char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission)
+char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission, GameVariant_t *variant)
 {
     char *result;
 
@@ -751,6 +753,7 @@ char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission)
             {
                 *mode = iwads[i].mode;
                 *mission = iwads[i].mission;
+                *variant = iwads[i].variant;
                 break;
             }
         }
