@@ -26,13 +26,14 @@ typedef struct
     const char *name;
     GameMission_t mission;
     GameMode_t mode;
+    GameVariant_t variant;
     const char *description;
 } iwad_t;
 
 char *D_DoomExeDir(void);       // killough 2/16/98: path to executable's dir
 char *D_FindWADByName(const char *filename);
 char *D_TryFindWADByName(const char *filename);
-char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission);
+char *D_FindIWADFile(GameMode_t *mode, GameMission_t *mission, GameVariant_t *variant);
 boolean D_IsIWADName(const char *name);
 const iwad_t **D_GetIwads(void);
 
