@@ -16,29 +16,26 @@
 //     Main loop code.
 //
 
-#include <stdlib.h>
 #include <string.h>
 
-#include "doomstat.h"
 #include "d_event.h"
 #include "d_loop.h"
 #include "d_ticcmd.h"
-
+#include "doomdef.h"
+#include "doomstat.h"
 #include "i_printf.h"
 #include "i_system.h"
+#include "i_timer.h"
 #include "i_video.h"
-
 #include "m_argv.h"
 #include "m_fixed.h"
-
 #include "net_client.h"
 #include "net_gui.h"
 #include "net_io.h"
-#include "net_query.h"
-#include "net_server.h"
-#include "net_sdl.h"
 #include "net_loop.h"
-
+#include "net_query.h"
+#include "net_sdl.h"
+#include "net_server.h"
 #include "s_sound.h"
 
 // The complete set of data for a particular tic.
@@ -558,6 +555,7 @@ void D_QuitNetGame (void)
 {
     NET_SV_Shutdown();
     NET_CL_Disconnect();
+    net_sdl_module.Shutdown();
 }
 
 static int GetLowTic(void)
@@ -728,7 +726,7 @@ void TryRunTics (void)
 
     // [AM] If we've uncapped the framerate and there are no tics
     //      to run, return early instead of waiting around.
-    #define return_early (uncapped && counts == 0 && leveltime > oldleveltime)
+    #define return_early (uncapped && counts == 0)
 
     // get real tics
     entertic = I_GetTime() / ticdup;
@@ -755,20 +753,7 @@ void TryRunTics (void)
 
     if (new_sync)
     {
-        if (uncapped)
-        {
-            // decide how many tics to run
-            if (realtics < availabletics-1)
-                counts = realtics+1;
-            else if (realtics < availabletics)
-                counts = realtics;
-            else
-                counts = availabletics;
-        }
-        else
-        {
-            counts = availabletics;
-        }
+        counts = availabletics;
 
         // [AM] If we've uncapped the framerate and there are no tics
         //      to run, return early instead of waiting around.
