@@ -20,58 +20,72 @@
 //
 //-----------------------------------------------------------------------------
 
-#include "m_io.h" // haleyjd
-#include "SDL_filesystem.h" // [FG] SDL_GetPrefPath()
-#include <stdlib.h> // [FG] qsort()
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#include "../miniz/miniz.h"
+#include <ctype.h>
+#include <limits.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
+#include "am_map.h"
+#include "config.h"
+#include "d_deh.h"  // Ty 04/08/98 - Externalizations
+#include "d_event.h"
+#include "d_iwad.h"
+#include "d_loop.h"
+#include "d_main.h"
+#include "d_player.h"
+#include "d_quit.h"
+#include "d_ticcmd.h"
 #include "doomdef.h"
 #include "doomstat.h"
+#include "dsdhacked.h"
 #include "dstrings.h"
-#include "sounds.h"
-#include "z_zone.h"
-#include "w_wad.h"
-#include "s_sound.h"
-#include "v_video.h"
 #include "f_finale.h"
 #include "f_wipe.h"
-#include "m_argv.h"
-#include "m_misc.h"
-#include "m_misc2.h" // [FG] M_StringDuplicate()
-#include "m_menu.h"
-#include "m_swap.h"
-#include "i_printf.h"
-#include "i_system.h"
-#include "i_sound.h"
-#include "i_video.h"
 #include "g_game.h"
 #include "hu_stuff.h"
-#include "wi_stuff.h"
-#include "st_stuff.h"
-#include "am_map.h"
-#include "p_setup.h"
-#include "r_draw.h"
-#include "r_main.h"
-#include "d_main.h"
-#include "d_iwad.h" // [FG] D_FindWADByName()
-#include "d_deh.h"  // Ty 04/08/98 - Externalizations
-#include "statdump.h" // [FG] StatDump()
-#include "u_mapinfo.h" // U_ParseMapInfo()
-#include "i_glob.h" // [FG] I_StartMultiGlob()
-#include "p_map.h" // MELEERANGE
 #include "i_endoom.h"
-#include "d_quit.h"
-#include "r_bmaps.h"
-
-#include "dsdhacked.h"
-
+#include "i_glob.h"
+#include "i_input.h"
+#include "i_printf.h"
+#include "i_sound.h"
+#include "i_system.h"
+#include "i_timer.h"
+#include "i_video.h"
+#include "info.h"
+#include "m_argv.h"
+#include "m_array.h"
+#include "m_fixed.h"
+#include "m_input.h"
+#include "m_io.h"
+#include "m_menu.h"
+#include "m_misc.h"
+#include "m_misc2.h"
+#include "m_swap.h"
 #include "net_client.h"
 #include "net_dedicated.h"
+#include "p_inter.h" // maxhealthbonus
+#include "p_map.h"   // MELEERANGE
+#include "p_mobj.h"
+#include "p_setup.h"
+#include "r_bmaps.h"
+#include "r_defs.h"
+#include "r_draw.h"
+#include "r_main.h"
+#include "r_state.h"
+#include "r_voxel.h"
+#include "s_sound.h"
+#include "sounds.h"
+#include "st_stuff.h"
+#include "statdump.h"
+#include "u_mapinfo.h"
+#include "v_video.h"
+#include "w_wad.h"
+#include "wi_stuff.h"
+#include "z_zone.h"
+
+#include "miniz.h"
 
 // DEHacked support - Ty 03/09/97
 // killough 10/98:
@@ -150,8 +164,7 @@ char    *basedefault = NULL;   // default file
 char    *basesavegame = NULL;  // killough 2/16/98: savegame directory
 char    *screenshotdir = NULL; // [FG] screenshot directory
 
-// If true, the main game loop has started.
-boolean main_loop_started = false;
+boolean organize_savefiles;
 
 boolean coop_spawns = false;
 
@@ -179,8 +192,19 @@ int eventhead, eventtail;
 //
 void D_PostEvent(event_t *ev)
 {
-  events[eventhead++] = *ev;
-  eventhead &= MAXEVENTS-1;
+  switch (ev->type)
+  {
+    case ev_mouse:
+    case ev_joystick:
+      G_MovementResponder(ev);
+      G_PrepTiccmd();
+      break;
+
+    default:
+      events[eventhead++] = *ev;
+      eventhead &= MAXEVENTS-1;
+      break;
+  }
 }
 
 //
@@ -200,6 +224,34 @@ void D_ProcessEvents (void)
     }
 }
 
+static boolean input_ready;
+
+void D_UpdateDeltaTics(void)
+{
+  if (uncapped && raw_input)
+  {
+    static uint64_t last_time;
+    const uint64_t current_time = I_GetTimeUS();
+
+    if (input_ready)
+    {
+      const uint64_t delta_time = current_time - last_time;
+      deltatics = (double)delta_time * TICRATE / 1000000.0;
+      deltatics = BETWEEN(0.0, 1.0, deltatics);
+    }
+    else
+    {
+      deltatics = 0.0;
+    }
+
+    last_time = current_time;
+  }
+  else
+  {
+    deltatics = 1.0;
+  }
+}
+
 //
 // D_Display
 //  draw current display, possibly wiping it from the previous
@@ -233,8 +285,41 @@ void D_Display (void)
   if (nodrawers)                    // for comparative timing / profiling
     return;
 
+  input_ready = (!menuactive && gamestate == GS_LEVEL && !paused);
+
+  if (uncapped)
+  {
+    // [AM] Figure out how far into the current tic we're in as a fixed_t.
+    fractionaltic = I_GetFracTime();
+
+    if (input_ready && raw_input)
+    {
+      I_StartDisplay();
+    }
+  }
+
   redrawsbar = false;
 
+  wipe = false;
+
+  // save the current screen if about to wipe
+  if (gamestate != wipegamestate && (strictmode || screen_melt))
+    {
+      wipe = true;
+      wipe_StartScreen(0, 0, video.unscaledw, SCREENHEIGHT);
+    }
+
+  if (!wipe)
+    {
+      if (resetneeded)
+        I_ResetScreen();
+      else if (gamestate == GS_LEVEL)
+        I_DynamicResolution();
+    }
+
+  if (setsmoothlight)
+    R_SmoothLight();
+
   if (setsizeneeded)                // change the view size if needed
     {
       R_ExecuteSetViewSize();
@@ -242,10 +327,6 @@ void D_Display (void)
       borderdrawcount = 3;
     }
 
-  // save the current screen if about to wipe
-  if ((wipe = gamestate != wipegamestate) && NOTSTRICTMODE(screen_melt))
-    wipe_StartScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);
-
   if (gamestate == GS_LEVEL && gametic)
     HU_Erase();
 
@@ -296,13 +377,14 @@ void D_Display (void)
     }
 
   // see if the border needs to be updated to the screen
-  if (gamestate == GS_LEVEL && automap_off && scaledviewwidth != SCREENWIDTH)
+  if (gamestate == GS_LEVEL && automap_off && scaledviewwidth != video.unscaledw)
     {
       if (menuactive || menuactivestate || !viewactivestate)
         borderdrawcount = 3;
       if (borderdrawcount)
         {
           R_DrawViewBorder ();    // erase old menu stuff
+          HU_Drawer ();
           borderdrawcount--;
         }
     }
@@ -326,14 +408,16 @@ void D_Display (void)
   // draw pause pic
   if (paused)
     {
+      int x = scaledviewx;
       int y = 4;
-      int x = (viewwindowx>>hires);
       patch_t *patch = W_CacheLumpName("M_PAUSE", PU_CACHE);
 
+      x += (scaledviewwidth - SHORT(patch->width)) / 2 - video.deltaw;
+
       if (!automapactive)
-        y += (viewwindowy>>hires);
-      V_DrawPatchDirect(x + (scaledviewwidth - SHORT(patch->width)) / 2 - WIDESCREENDELTA,
-                        y, 0, patch);
+        y += scaledviewy;
+
+      V_DrawPatch(x, y, patch);
     }
 
   // menus go directly to the screen
@@ -344,14 +428,14 @@ void D_Display (void)
     HU_DemoProgressBar(true);
 
   // normal update
-  if (!wipe || STRICTMODE(!screen_melt))
+  if (!wipe)
     {
       I_FinishUpdate ();              // page flip or blit buffer
       return;
     }
 
   // wipe update
-  wipe_EndScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);
+  wipe_EndScreen(0, 0, video.unscaledw, SCREENHEIGHT);
 
   wipestart = I_GetTime () - 1;
 
@@ -365,11 +449,13 @@ void D_Display (void)
         }
       while (!tics);
       wipestart = nowtime;
-      done = wipe_ScreenWipe(wipe_Melt,0,0,SCREENWIDTH,SCREENHEIGHT,tics);
+      done = wipe_ScreenWipe(wipe_Melt, 0, 0, video.unscaledw, SCREENHEIGHT, tics);
       M_Drawer();                   // menu is drawn even on top of wipes
       I_FinishUpdate();             // page flip or blit buffer
     }
   while (!done);
+
+  drs_skip_frame = true; // skip DRS after wipe
 }
 
 //
@@ -410,12 +496,12 @@ void D_PageDrawer(void)
       unsigned c = 0;
       while (s--)
 	c = c*3 + t[s];
-      V_DrawPatchFullScreen(0, (patch_t *) t);
+      V_DrawPatchFullScreen((patch_t *) t);
       if (c==2119826587u || c==2391756584u)
         // [FG] removed the embedded DOGOVRLY title pic overlay graphic lump
         if (W_CheckNumForName("DOGOVRLY") > 0)
         {
-	V_DrawPatch(0, 0, 0, W_CacheLumpName("DOGOVRLY", PU_CACHE));
+	V_DrawPatch(0, 0, W_CacheLumpName("DOGOVRLY", PU_CACHE));
         }
     }
   else
@@ -579,6 +665,30 @@ void D_StartTitle (void)
   D_AdvanceDemo();
 }
 
+static boolean CheckExtensions(const char *filename, const char *ext, ...)
+{
+    boolean result = false;
+    va_list args;
+
+    if (M_StringCaseEndsWith(filename, ext))
+    {
+        return true;
+    }
+
+    va_start(args, ext);
+    while (true)
+    {
+        const char *arg = va_arg(args, const char *);
+        if (arg == NULL || (result = M_StringCaseEndsWith(filename, arg)))
+        {
+            break;
+        }
+    }
+    va_end(args);
+
+    return result;
+}
+
 char **tempdirs = NULL;
 
 static void AutoLoadWADs(const char *path);
@@ -588,9 +698,8 @@ static boolean D_AddZipFile(const char *file)
   int i;
   mz_zip_archive zip_archive;
   char *str, *tempdir, counter[8];
-  static int idx = 0;
 
-  if (!M_StringCaseEndsWith(file, ".zip"))
+  if (!CheckExtensions(file, ".zip", ".pk3", NULL))
   {
     return false;
   }
@@ -599,10 +708,9 @@ static boolean D_AddZipFile(const char *file)
   if (!mz_zip_reader_init_file(&zip_archive, file, MZ_ZIP_FLAG_DO_NOT_SORT_CENTRAL_DIRECTORY))
   {
     I_Error("D_AddZipFile: Failed to open %s", file);
-    return false;
   }
 
-  M_snprintf(counter, sizeof(counter), "%04d", idx);
+  M_snprintf(counter, sizeof(counter), "%04d", array_size(tempdirs));
   str = M_StringJoin("_", counter, "_", PROJECT_SHORTNAME, "_", M_BaseName(file), NULL);
   tempdir = M_TempFile(str);
   free(str);
@@ -624,9 +732,8 @@ static boolean D_AddZipFile(const char *file)
     if (name[0] == '.')
       continue;
 
-    if (M_StringCaseEndsWith(name, ".wad") || M_StringCaseEndsWith(name, ".lmp") ||
-        M_StringCaseEndsWith(name, ".ogg") || M_StringCaseEndsWith(name, ".flac") ||
-        M_StringCaseEndsWith(name, ".mp3"))
+    if (CheckExtensions(name, ".wad", ".pk3", ".lmp", ".ogg", ".flac",
+                        ".mp3", ".kvx", NULL))
     {
       char *dest = M_StringJoin(tempdir, DIR_SEPARATOR_S, name, NULL);
 
@@ -644,9 +751,7 @@ static boolean D_AddZipFile(const char *file)
 
   AutoLoadWADs(tempdir);
 
-  tempdirs = I_Realloc(tempdirs, (idx + 2) * sizeof(*tempdirs));
-  tempdirs[idx++] = tempdir;
-  tempdirs[idx] = NULL;
+  array_push(tempdirs, tempdir);
 
   return true;
 }
@@ -661,21 +766,19 @@ static boolean D_AddZipFile(const char *file)
 
 void D_AddFile(const char *file)
 {
-  static int numwadfiles, numwadfiles_alloc;
-
+  // [FG] search for PWADs by their filename
   char *path = D_TryFindWADByName(file);
 
+  if (M_StringCaseEndsWith(path, ".kvx"))
+  {
+    array_push(vxfiles, path);
+    return;
+  }
+
   if (D_AddZipFile(path))
     return;
 
-  if (numwadfiles == numwadfiles_alloc - 1 || !numwadfiles_alloc)
-  {
-    numwadfiles_alloc = (numwadfiles_alloc ? numwadfiles_alloc * 2 : 8);
-    wadfiles = I_Realloc(wadfiles, numwadfiles_alloc * sizeof(*wadfiles));
-  }
-  // [FG] search for PWADs by their filename
-  wadfiles[numwadfiles++] = path;
-  wadfiles[numwadfiles] = NULL;
+  array_push(wadfiles, path);
 }
 
 // killough 10/98: return the name of the program the exe was invoked as
@@ -740,7 +843,7 @@ static struct {
     char *(*func)(void);
     boolean createdir;
 } autoload_basedirs[] = {
-#ifndef WIN32
+#if !defined(WIN32)
     {"../share/" PROJECT_SHORTNAME, D_DoomExeDir, false},
 #endif
     {NULL, D_DoomPrefDir, true},
@@ -839,12 +942,13 @@ static void CheckIWAD(const char *iwadname)
     {
         fclose(file);
         I_Error("CheckIWAD: failed to read header %s", iwadname);
-        return;
     }
 
-    if (strncmp(header.identification, "IWAD", 4))
+    if (strncmp(header.identification, "IWAD", 4) &&
+        strncmp(header.identification, "PWAD", 4))
     {
-        I_Printf(VB_WARNING, "CheckIWAD: IWAD tag not present %s", iwadname);
+        fclose(file);
+        I_Error("Wad file %s doesn't have IWAD or PWAD id\n", iwadname);
     }
 
     // read IWAD directory
@@ -858,7 +962,6 @@ static void CheckIWAD(const char *iwadname)
         free(fileinfo);
         fclose(file);
         I_Error("CheckIWAD: failed to read directory %s", iwadname);
-        return;
     }
 
     for (i = 0; i < header.numlumps; ++i)
@@ -905,6 +1008,69 @@ static void CheckIWAD(const char *iwadname)
     }
 }
 
+static boolean FileContainsMaps(const char *filename)
+{
+    int i;
+    FILE *file = NULL;
+    wadinfo_t header;
+    filelump_t *fileinfo = NULL;
+    boolean ret = false;
+
+    while (ret == false)
+    {
+        if (filename == NULL || M_StringCaseEndsWith(filename, ".wad") == false)
+        {
+            break;
+        }
+
+        file = M_fopen(filename, "rb");
+
+        if (file == NULL)
+        {
+            break;
+        }
+
+        if (fread(&header, sizeof(header), 1, file) != 1)
+        {
+            break;
+        }
+
+        if (strncmp(header.identification, "IWAD", 4) &&
+            strncmp(header.identification, "PWAD", 4))
+        {
+            break;
+        }
+
+        header.numlumps = LONG(header.numlumps);
+        header.infotableofs = LONG(header.infotableofs);
+        fileinfo = malloc(header.numlumps * sizeof(filelump_t));
+
+        if (fseek(file, header.infotableofs, SEEK_SET) ||
+            fread(fileinfo, sizeof(filelump_t), header.numlumps, file) != header.numlumps)
+        {
+            break;
+        }
+
+        for (i = 0; i < header.numlumps; i++)
+        {
+            if (StartsWithMapIdentifier(fileinfo[i].name))
+            {
+                ret = true;
+                break;
+            }
+        }
+
+        break;
+    }
+
+    if (fileinfo)
+        free(fileinfo);
+    if (file)
+        fclose(file);
+
+    return ret;
+}
+
 //
 // IdentifyVersion
 //
@@ -983,7 +1149,7 @@ void IdentifyVersion (void)
 
   // locate the IWAD and determine game mode from it
 
-  iwad = D_FindIWADFile(&gamemode, &gamemission);
+  iwad = D_FindIWADFile(&gamemode, &gamemission, &gamevariant);
 
   if (iwad && *iwad)
     {
@@ -1544,7 +1710,8 @@ static void AutoLoadWADs(const char *path)
     const char *filename;
 
     glob = I_StartMultiGlob(path, GLOB_FLAG_NOCASE|GLOB_FLAG_SORTED,
-                            "*.wad", "*.lmp", "*.zip", "*.ogg", "*.flac", "*.mp3", NULL);
+                            "*.wad", "*.lmp", "*.kvx", "*.zip", "*.pk3",
+                            "*.ogg", "*.flac", "*.mp3", NULL);
     for (;;)
     {
         filename = I_NextGlob(glob);
@@ -1552,6 +1719,7 @@ static void AutoLoadWADs(const char *path)
         {
             break;
         }
+
         D_AddFile(filename);
     }
 
@@ -1567,7 +1735,9 @@ static void D_AutoloadIWadDir()
     char *autoload_dir;
 
     // common auto-loaded files for all Doom flavors
-    if (gamemission < pack_chex)
+    if (gamemission < pack_chex &&
+        gamevariant != freedoom &&
+        gamevariant != miniwad)
     {
       autoload_dir = GetAutoloadDir(*base, "doom-all", true);
       AutoLoadWADs(autoload_dir);
@@ -1585,7 +1755,7 @@ static void D_AutoloadPWadDir()
 {
   int i;
 
-  for (i = 1; wadfiles[i]; ++i)
+  for (i = 1; i < array_size(wadfiles); ++i)
   {
     char **base;
 
@@ -1632,7 +1802,9 @@ static void D_AutoloadDehDir()
     char *autoload_dir;
 
     // common auto-loaded files for all Doom flavors
-    if (gamemission < pack_chex)
+    if (gamemission < pack_chex &&
+        gamevariant != freedoom &&
+        gamevariant != miniwad)
     {
       autoload_dir = GetAutoloadDir(*base, "doom-all", true);
       AutoLoadPatches(autoload_dir);
@@ -1650,7 +1822,7 @@ static void D_AutoloadPWadDehDir()
 {
   int i;
 
-  for (i = 1; wadfiles[i]; ++i)
+  for (i = 1; i < array_size(wadfiles); ++i)
   {
     char **base;
 
@@ -1769,6 +1941,23 @@ static void D_InitTables(void)
     states[i].flags |= STATEF_SKILL5FAST;
 }
 
+void D_SetMaxHealth(void)
+{
+  extern boolean deh_set_maxhealth;
+  extern int deh_maxhealth;
+
+  if (demo_compatibility)
+  {
+    maxhealth = 100;
+    maxhealthbonus = deh_set_maxhealth ? deh_maxhealth : 200;
+  }
+  else
+  {
+    maxhealth = deh_set_maxhealth ? deh_maxhealth : 100;
+    maxhealthbonus = maxhealth * 2;
+  }
+}
+
 void D_SetBloodColor(void)
 {
   extern boolean deh_set_blood_color;
@@ -1790,49 +1979,6 @@ void D_SetBloodColor(void)
   }
 }
 
-static const int predefined_translucency[] = {
-    // MBF
-    MT_FIRE,      MT_SMOKE,     MT_FATSHOT,  MT_BRUISERSHOT,
-    MT_SPAWNFIRE, MT_TROOPSHOT, MT_HEADSHOT, MT_PLASMA,
-    MT_BFG,       MT_ARACHPLAZ, MT_PUFF,     MT_TFOG,
-    MT_IFOG,      MT_MISC12,    MT_INV,      MT_INS,
-    MT_MEGA,
-    // [Woof!]
-    MT_PLASMA1,   MT_PLASMA2
-};
-
-static boolean deh_set_translucency[arrlen(predefined_translucency)] = {false};
-
-void D_DehChangePredefinedTranslucency(int index)
-{
-  int i;
-
-  for (i = 0; i < arrlen(predefined_translucency); ++i)
-  {
-    if (predefined_translucency[i] == index)
-    {
-      deh_set_translucency[i] = true;
-      break;
-    }
-  }
-}
-
-void D_SetPredefinedTranslucency(void)
-{
-  int i;
-
-  for (i = 0; i < arrlen(predefined_translucency); ++i)
-  {
-    if (deh_set_translucency[i])
-      continue;
-
-    if (STRICTMODE_VANILLA(translucency))
-      mobjinfo[predefined_translucency[i]].flags |= MF_TRANSLUCENT;
-    else
-      mobjinfo[predefined_translucency[i]].flags &= ~MF_TRANSLUCENT;
-  }
-}
-
 // killough 2/22/98: Add support for ENDBOOM, which is PC-specific
 // killough 8/1/98: change back to ENDOOM
 
@@ -1856,12 +2002,10 @@ static void D_ShowEndDoom(void)
 
 static void D_EndDoom(void)
 {
-  if (!main_loop_started || !D_CheckEndDoom())
+  if (D_CheckEndDoom())
   {
-    return;
+    D_ShowEndDoom();
   }
-
-  D_ShowEndDoom();
 }
 
 // [FG] fast-forward demo to the desired map
@@ -1907,6 +2051,7 @@ static boolean CheckHaveSSG (void)
 void D_DoomMain(void)
 {
   int p;
+  int mainwadfile = 0;
 
   setbuf(stdout,NULL);
 
@@ -2082,8 +2227,6 @@ void D_DoomMain(void)
   if ((p=M_CheckParm ("-turbo")))
     {
       int scale = 200;
-      extern int forwardmove[2];
-      extern int sidemove[2];
 
       if (p < myargc - 1 && myargv[p + 1][0] != '-')
         scale = M_ParmArgToInt(p);
@@ -2096,6 +2239,12 @@ void D_DoomMain(void)
       forwardmove[1] = forwardmove[1]*scale/100;
       sidemove[0] = sidemove[0]*scale/100;
       sidemove[1] = sidemove[1]*scale/100;
+
+      // Prevent ticcmd overflow.
+      forwardmove[0] = MIN(forwardmove[0], SCHAR_MAX);
+      forwardmove[1] = MIN(forwardmove[1], SCHAR_MAX);
+      sidemove[0] = MIN(sidemove[0], SCHAR_MAX);
+      sidemove[1] = MIN(sidemove[1], SCHAR_MAX);
     }
 
   if (beta_emulation)
@@ -2135,6 +2284,7 @@ void D_DoomMain(void)
       // killough 11/98: allow multiple -file parameters
 
       boolean file = modifiedgame = true;            // homebrew levels
+      mainwadfile = array_size(wadfiles);
       while (++p < myargc)
         if (*myargv[p] == '-')
           file = !strcasecmp(myargv[p],"-file");
@@ -2380,16 +2530,61 @@ void D_DoomMain(void)
 
   PrintVersion();
 
+  if (!M_CheckParm("-save"))
+  {
+    if (organize_savefiles == -1)
+    {
+      // [FG] check for at least one savegame in the old location
+      glob_t *glob = I_StartMultiGlob(basesavegame,
+                                      GLOB_FLAG_NOCASE|GLOB_FLAG_SORTED,
+                                      "*.dsg", NULL);
+
+      organize_savefiles = (I_NextGlob(glob) == NULL);
+
+      I_EndGlob(glob);
+    }
+
+    if (organize_savefiles)
+    {
+      int i;
+      char *wadname = wadfiles[0], *oldsavegame = basesavegame;
+
+      for (i = mainwadfile; i < array_size(wadfiles); i++)
+      {
+        if (FileContainsMaps(wadfiles[i]))
+        {
+          wadname = wadfiles[i];
+          break;
+        }
+      }
+
+      basesavegame = M_StringJoin(oldsavegame, DIR_SEPARATOR_S,
+                                  "savegames", NULL);
+      free(oldsavegame);
+
+      NormalizeSlashes(basesavegame);
+      M_MakeDirectory(basesavegame);
+
+      oldsavegame = basesavegame;
+      basesavegame = M_StringJoin(oldsavegame, DIR_SEPARATOR_S,
+                                  M_BaseName(wadname), NULL);
+      free(oldsavegame);
+
+      NormalizeSlashes(basesavegame);
+      M_MakeDirectory(basesavegame);
+    }
+  }
+
+  I_Printf(VB_INFO, "Savegame directory: %s\n", basesavegame);
+
   bodyquesize = default_bodyquesize; // killough 10/98
 
   // 1/18/98 killough: Z_Init call moved to i_main.c
 
   // init subsystems
-  I_Printf(VB_INFO, "V_Init: allocate screens.");    // killough 11/98: moved down to here
-  V_Init();
 
   I_Printf(VB_INFO, "W_Init: Init WADfiles.");
-  W_InitMultipleFiles(wadfiles);
+  W_InitMultipleFiles();
 
   // Check for wolf levels
   haswolflevels = (W_CheckNumForName("map31") >= 0);
@@ -2524,10 +2719,9 @@ void D_DoomMain(void)
 
   I_Printf(VB_INFO, "I_Init: Setting up machine state.");
   I_InitTimer();
-  I_InitJoystick();
+  I_InitController();
   I_InitSound();
   I_InitMusic();
-  M_GetMidiDevices();
 
   I_Printf(VB_INFO, "NET_Init: Init network subsystem.");
   NET_Init();
@@ -2538,6 +2732,9 @@ void D_DoomMain(void)
   I_Printf(VB_INFO, "D_CheckNetGame: Checking network game status.");
   D_CheckNetGame();
 
+  G_UpdateSideMove();
+  G_UpdateCarryAngle();
+
   M_ResetTimeScale();
 
   I_Printf(VB_INFO, "S_Init: Setting up sound.");
@@ -2549,6 +2746,11 @@ void D_DoomMain(void)
 
   I_Printf(VB_INFO, "ST_Init: Init status bar.");
   ST_Init();
+  ST_Warnings();
+
+  // andrewj: voxel support
+  I_Printf(VB_INFO, "VX_Init: ");
+  VX_Init();
 
   I_PutChar(VB_INFO, '\n');
 
@@ -2696,9 +2898,11 @@ void D_DoomMain(void)
     I_SetFastdemoTimer(true);
   }
 
-  // [FG] init graphics (WIDESCREENDELTA) before HUD widgets
+  // [FG] init graphics (video.widedelta) before HUD widgets
   I_InitGraphics();
 
+  M_InitMenuStrings();
+
   if (startloadgame >= 0)
   {
     char *file;
@@ -2729,8 +2933,6 @@ void D_DoomMain(void)
 
   TryRunTics();
 
-  main_loop_started = true;
-
   D_StartGameLoop();
 
   for (;;)
@@ -2743,6 +2945,8 @@ void D_DoomMain(void)
       // Update display, next frame, with current state.
       if (screenvisible)
         D_Display();
+
+      S_UpdateMusic();
     }
 }
 
