@@ -22,7 +22,8 @@
 
 // Game Mode - identify IWAD as shareware, retail etc.
 GameMode_t gamemode = indetermined;
-GameMission_t   gamemission = doom;
+GameMission_t gamemission = doom;
+GameVariant_t gamevariant = vanilla;
 
 // [FG] emulate a specific version of Doom
 GameVersion_t gameversion = exe_doom_1_9;
@@ -119,7 +120,8 @@ boolean hide_weapon;
 // [FG] centered weapon sprite
 int center_weapon;
 
-int cosmetic_bobbing;
+int view_bobbing_pct;
+int weapon_bobbing_pct;
 
 char *MAPNAME(int e, int m)
 {
