@@ -19,24 +19,40 @@
 
 // killough 5/3/98: remove unnecessary headers
 
-#include "doomstat.h"
+#include <stdlib.h>
+#include <string.h>
+
+#include "d_deh.h" /* Ty 03/27/98 - externalization of mapnamesx arrays */
+#include "d_event.h"
+#include "d_items.h"
+#include "d_player.h"
 #include "doomkeys.h"
-#include "hu_stuff.h"
-#include "hu_obituary.h"
-#include "hu_lib.h"
-#include "st_stuff.h" /* jff 2/16/98 need loc of status bar */
-#include "w_wad.h"
-#include "s_sound.h"
+#include "doomstat.h"
 #include "dstrings.h"
-#include "sounds.h"
-#include "d_deh.h"   /* Ty 03/27/98 - externalization of mapnamesx arrays */
-#include "r_draw.h"
+#include "hu_lib.h"
+#include "hu_obituary.h"
+#include "hu_stuff.h"
+#include "i_video.h" // fps
+#include "m_fixed.h"
 #include "m_input.h"
-#include "p_map.h" // crosshair (linetarget)
 #include "m_misc2.h"
 #include "m_swap.h"
+#include "p_map.h" // crosshair (linetarget)
+#include "p_mobj.h"
+#include "r_data.h"
+#include "r_defs.h"
 #include "r_main.h"
+#include "r_state.h"
+#include "r_voxel.h"
+#include "s_sound.h"
+#include "sounds.h"
+#include "st_stuff.h" /* jff 2/16/98 need loc of status bar */
+#include "tables.h"
+#include "u_mapinfo.h"
 #include "u_scanner.h"
+#include "v_video.h"
+#include "w_wad.h"
+#include "z_zone.h"
 
 // global heads up display controls
 
@@ -44,11 +60,10 @@ int hud_active;       //jff 2/17/98 controls heads-up display mode
 int hud_displayed;    //jff 2/23/98 turns heads-up display on/off
 int hud_secret_message; // "A secret is revealed!" message
 int hud_widget_font;
-int hud_draw_bargraphs;
-int hud_threelined_widgets;
+int hud_widget_layout;
 
-int crispy_hud; // Crispy HUD
-static boolean draw_crispy_hud;
+int hud_type; // Crispy HUD or Boom variants
+boolean draw_crispy_hud;
 
 //
 // Locally used constants, shortcuts.
@@ -119,38 +134,41 @@ static hu_multiline_t w_monsec; //jff 2/16/98 new kill/secret widget for hud
 static hu_multiline_t w_sttime; // time above status bar
 static hu_multiline_t w_coord;
 static hu_multiline_t w_fps;
+static hu_multiline_t w_rate;
 
 #define MAX_HUDS 3
-#define MAX_WIDGETS 10
+#define MAX_WIDGETS_D 5
+#define MAX_WIDGETS_B 12
 
-static hu_widget_t doom_widgets[MAX_HUDS][MAX_WIDGETS] = {
+static hu_widget_t doom_widgets[MAX_HUDS][MAX_WIDGETS_D] = {
   {
-    {&w_title,   align_left,   align_bottom},
-    {&w_message, align_left,   align_top},
-    {&w_chat,    align_left,   align_top},
+    {&w_title,   align_direct, align_bottom, 0},
+    {&w_message, align_direct, align_top,    0},
+    {&w_chat,    align_direct, align_top,    0},
     {&w_secret,  align_center, align_direct, 0, 84},
     {NULL}
   }, {
-    {&w_title,   align_left,   align_bottom},
-    {&w_message, align_left,   align_top},
-    {&w_chat,    align_left,   align_top},
+    {&w_title,   align_direct, align_bottom, 0},
+    {&w_message, align_direct, align_top,    0},
+    {&w_chat,    align_direct, align_top,    0},
     {&w_secret,  align_center, align_direct, 0, 84},
     {NULL}
   }, {
-    {&w_title,   align_left,   align_bottom},
-    {&w_message, align_left,   align_top},
-    {&w_chat,    align_left,   align_top},
+    {&w_title,   align_direct, align_bottom, 0},
+    {&w_message, align_direct, align_top,    0},
+    {&w_chat,    align_direct, align_top,    0},
     {&w_secret,  align_center, align_direct, 0, 84},
     {NULL}
   }
 };
 
-static hu_widget_t boom_widgets[MAX_HUDS][MAX_WIDGETS] = {
+static hu_widget_t boom_widgets[MAX_HUDS][MAX_WIDGETS_B] = {
   {
     {&w_monsec, align_left,  align_top},
     {&w_sttime, align_left,  align_top},
     {&w_coord,  align_right, align_top},
     {&w_fps,    align_right, align_top},
+    {&w_rate,   align_left,  align_top},
     {NULL}
   }, {
     {&w_armor,  align_left,  align_bottom},
@@ -163,6 +181,7 @@ static hu_widget_t boom_widgets[MAX_HUDS][MAX_WIDGETS] = {
     {&w_sttime, align_left,  align_bottom},
     {&w_coord,  align_right, align_top},
     {&w_fps,    align_right, align_top},
+    {&w_rate,   align_left,  align_top},
     {NULL}
   }, {
     {&w_health, align_right, align_top},
@@ -175,6 +194,7 @@ static hu_widget_t boom_widgets[MAX_HUDS][MAX_WIDGETS] = {
     {&w_sttime, align_left,  align_bottom},
     {&w_coord , align_right, align_top},
     {&w_fps,    align_right, align_top},
+    {&w_rate,   align_left,  align_top},
     {NULL}
   }
 };
@@ -183,6 +203,7 @@ static hu_widget_t *doom_widget = doom_widgets[0],
                    *boom_widget = boom_widgets[0];
 
 static void HU_ParseHUD (void);
+static void HU_set_centered_message (boolean);
 
 static char       chat_dest[MAXPLAYERS];
 boolean           chat_on;
@@ -358,7 +379,7 @@ void HU_ResetMessageColors(void)
 
 extern boolean st_invul;
 
-static char* ColorByHealth(int health, int maxhealth, boolean invul)
+static byte* ColorByHealth(int health, int maxhealth, boolean invul)
 {
   if (invul)
     return colrngs[CR_GRAY];
@@ -375,16 +396,6 @@ static char* ColorByHealth(int health, int maxhealth, boolean invul)
     return colrngs[CR_BLUE];
 }
 
-static int lightest_color, darkest_color;
-
-static void HU_InitDemoProgressBar (void)
-{
-  byte *playpal = W_CacheLumpName("PLAYPAL", PU_STATIC);
-
-  lightest_color = I_GetPaletteIndex(playpal, 0xFF, 0xFF, 0xFF);
-  darkest_color  = I_GetPaletteIndex(playpal, 0x00, 0x00, 0x00);
-}
-
 //
 // HU_Init()
 //
@@ -400,28 +411,28 @@ void HU_Init(void)
   // load the heads-up font
   for (i = 0, j = HU_FONTSTART; i < HU_FONTSIZE; i++, j++)
   {
-    sprintf(buffer, "STCFN%.3d", j);
+    M_snprintf(buffer, sizeof(buffer), "STCFN%.3d", j);
     if (W_CheckNumForName(buffer) != -1)
       big_font.patches[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
 
     if ('0' <= j && j <= '9')
     {
-      sprintf(buffer, "DIG%.1d", j - 48);
+      M_snprintf(buffer, sizeof(buffer), "DIG%.1d", j - 48);
       sml_font.patches[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
     }
     else if ('A' <= j && j <= 'Z')
     {
-      sprintf(buffer, "DIG%c", j);
+      M_snprintf(buffer, sizeof(buffer), "DIG%c", j);
       sml_font.patches[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
     }
     else if (j > 122)
     {
-      sprintf(buffer, "STBR%.3d", j);
+      M_snprintf(buffer, sizeof(buffer), "STBR%.3d", j);
       sml_font.patches[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
     }
     else
     {
-      sprintf(buffer, "DIG%.2d", j);
+      M_snprintf(buffer, sizeof(buffer), "DIG%.2d", j);
       if (W_CheckNumForName(buffer) != -1)
         sml_font.patches[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
     }
@@ -447,7 +458,7 @@ void HU_Init(void)
   //jff 2/26/98 load patches for keys and double keys
   for (i = HU_FONTSIZE, j = 0; j < 6; i++, j++)
   {
-    sprintf(buffer, "STKEYS%.1d", j);
+    M_snprintf(buffer, sizeof(buffer), "STKEYS%.1d", j);
     sml_font.patches[i] =
     big_font.patches[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
   }
@@ -459,10 +470,10 @@ void HU_Init(void)
   // [FG] support crosshair patches from extras.wad
   HU_InitCrosshair();
 
-  HU_InitDemoProgressBar();
-
   HU_InitObituaries();
+
   HU_ParseHUD();
+  HU_set_centered_message(true);
 
   // [Woof!] prepare player messages for colorization
   for (i = 0; i < arrlen(colorize_strings); i++)
@@ -475,7 +486,7 @@ void HU_Init(void)
 
 // [FG] support centered player messages
 
-static void HU_set_centered_message()
+static void HU_set_centered_message(boolean init)
 {
   int i, j;
 
@@ -487,14 +498,13 @@ static void HU_set_centered_message()
     {
       if (d_w[j].multiline == &w_message)
       {
-        if (d_w[j].h_align == align_direct)
-          continue;
+        // [FG] save original alignment
+        if (init)
+        {
+          d_w[j].h_align_orig = d_w[j].h_align;
+        }
 
-        // [FG] save original alignment in the unused x coordinate
-        if (d_w[j].x == 0)
-          d_w[j].x = d_w[j].h_align;
-
-        d_w[j].h_align = message_centered ? align_center : d_w[j].x;
+        d_w[j].h_align = message_centered ? align_center : d_w[j].h_align_orig;
       }
     }
   }
@@ -502,13 +512,15 @@ static void HU_set_centered_message()
 
 static inline void HU_cond_build_widget (hu_multiline_t *const multiline, boolean cond)
 {
-  if (cond)
+  if (cond && multiline->built == false)
   {
     multiline->builder();
     multiline->built = true;
   }
 }
 
+static boolean hud_pending;
+
 void HU_disable_all_widgets (void)
 {
   hu_widget_t *w = boom_widget;
@@ -518,6 +530,8 @@ void HU_disable_all_widgets (void)
     w->multiline->built = false;
     w++;
   }
+
+  hud_pending = true;
 }
 
 //
@@ -548,6 +562,7 @@ static void HU_widget_build_ammo (void);
 static void HU_widget_build_armor (void);
 static void HU_widget_build_coord (void);
 static void HU_widget_build_fps (void);
+static void HU_widget_build_rate (void);
 static void HU_widget_build_health (void);
 static void HU_widget_build_keys (void);
 static void HU_widget_build_frag (void);
@@ -556,6 +571,8 @@ static void HU_widget_build_sttime(void);
 static void HU_widget_build_title (void);
 static void HU_widget_build_weapon (void);
 
+static hu_multiline_t *w_stats;
+
 void HU_Start(void)
 {
   int i;
@@ -575,9 +592,6 @@ void HU_Start(void)
   message_count = (message_timer  * TICRATE) / 1000 + 1;
   chat_count    = (chat_msg_timer * TICRATE) / 1000 + 1;
 
-  // [crispy] re-calculate WIDESCREENDELTA
-  I_GetScreenDimensions();
-
   // create the message widget
   HUlib_init_multiline(&w_message, message_list ? hud_msg_lines : 1,
                        &doom_font, colrngs[hudcolor_mesg],
@@ -635,16 +649,18 @@ void HU_Start(void)
                        NULL, deathmatch ? HU_widget_build_frag : HU_widget_build_keys);
 
   // create the hud monster/secret widget
-  HUlib_init_multiline(&w_monsec, hud_threelined_widgets ? 3 : 1,
+  HUlib_init_multiline(&w_monsec, hud_widget_layout ? 3 : 1,
                        &boom_font, colrngs[CR_GRAY],
                        NULL, HU_widget_build_monsec);
+  // [FG] in deathmatch: w_keys.builder = HU_widget_build_frag()
+  w_stats = deathmatch ? &w_keys : &w_monsec;
 
   HUlib_init_multiline(&w_sttime, 1,
                        &boom_font, colrngs[CR_GRAY],
                        NULL, HU_widget_build_sttime);
 
   // create the automaps coordinate widget
-  HUlib_init_multiline(&w_coord, hud_threelined_widgets ? 3 : 1,
+  HUlib_init_multiline(&w_coord, hud_widget_layout ? 3 : 1,
                        &boom_font, colrngs[hudcolor_xyco],
                        NULL, HU_widget_build_coord);
 
@@ -652,7 +668,11 @@ void HU_Start(void)
                        &boom_font, colrngs[hudcolor_xyco],
                        NULL, HU_widget_build_fps);
 
-  HU_set_centered_message();
+  HUlib_init_multiline(&w_rate, (voxels_rendering ? 2 : 1),
+                       &boom_font, colrngs[hudcolor_xyco],
+                       NULL, HU_widget_build_rate);
+
+  HU_set_centered_message(false);
 
   HU_disable_all_widgets();
   HUlib_set_margins();
@@ -732,7 +752,7 @@ static void HU_widget_build_ammo (void)
   // special case for weapon with no ammo selected - blank bargraph + N/A
   if (weaponinfo[plr->readyweapon].ammo == am_noammo || fullammo == 0)
   {
-    if (hud_draw_bargraphs)
+    if (hud_type == HUD_TYPE_BOOM)
     {
       strcat(hud_ammostr, "\x7f\x7f\x7f\x7f\x7f\x7f\x7f");
     }
@@ -746,7 +766,7 @@ static void HU_widget_build_ammo (void)
     int ammobars = ammopct / 4;
 
     // build the bargraph string
-    if (hud_draw_bargraphs)
+    if (hud_type == HUD_TYPE_BOOM)
     {
       // full bargraph chars
       for (i = 4; i < 4 + ammobars / 4;)
@@ -775,7 +795,7 @@ static void HU_widget_build_ammo (void)
     }
 
     // build the numeric amount init string
-    sprintf(hud_ammostr + i, "%3d/%3d", ammo, fullammo);
+    M_snprintf(hud_ammostr + i, sizeof(hud_ammostr), "%3d/%3d", ammo, fullammo);
 
     // backpack changes thresholds (ammo widget)
     if (plr->backpack && !hud_backpack_thresholds && fullammo)
@@ -804,7 +824,7 @@ static void HU_widget_build_health (void)
   int healthbars = (st_health > 100) ? 25 : (st_health / 4);
 
   // build the bargraph string
-  if (hud_draw_bargraphs)
+  if (hud_type == HUD_TYPE_BOOM)
   {
     // full bargraph chars
     for (i = 4; i < 4 + healthbars / 4;)
@@ -833,7 +853,7 @@ static void HU_widget_build_health (void)
   }
 
   // build the numeric amount init string
-  sprintf(hud_healthstr + i, "%3d", st_health);
+  M_snprintf(hud_healthstr + i, sizeof(hud_healthstr), "%3d", st_health);
 
   // set the display color from the amount of health posessed
   w_health.cr = ColorByHealth(plr->health, 100, st_invul);
@@ -850,7 +870,7 @@ static void HU_widget_build_armor (void)
   int armorbars = (st_armor > 100) ? 25 : (st_armor / 4);
 
   // build the bargraph string
-  if (hud_draw_bargraphs)
+  if (hud_type == HUD_TYPE_BOOM)
   {
     // full bargraph chars
     for (i = 4; i < 4 + armorbars / 4;)
@@ -879,7 +899,7 @@ static void HU_widget_build_armor (void)
   }
 
   // build the numeric amount init string
-  sprintf(hud_armorstr + i, "%3d", st_armor);
+  M_snprintf(hud_armorstr + i, sizeof(hud_armorstr), "%3d", st_armor);
 
   // color of armor depends on type
   if (hud_armor_type)
@@ -989,6 +1009,37 @@ static void HU_widget_build_keys (void)
     hud_keysstr[i++] = ' ';   // spacing
     hud_keysstr[i++] = ' ';
   }
+
+  // [Alaux] Blink missing keys *after* possessed keys
+  for (k = 0; k < 6; k++)
+  {
+    if (plr->cards[k])
+      continue;
+
+    switch (ST_BlinkKey(plr, k % 3))
+    {
+      case KEYBLINK_CARD:
+        if (k >= 3)
+          continue;
+        break;
+
+      case KEYBLINK_SKULL:
+        if (k < 3)
+          continue;
+        break;
+
+      case KEYBLINK_BOTH:
+        break;
+
+      default:
+        continue;
+    }
+
+    hud_keysstr[i++] = HU_FONTEND + k + 1;
+    hud_keysstr[i++] = ' ';
+    hud_keysstr[i++] = ' ';
+  }
+
   hud_keysstr[i] = '\0';
 
   // transfer the built string (frags or key title) to the widget
@@ -1001,7 +1052,7 @@ static inline int HU_top (char *const fragstr, int i, const int idx1, const int
   {
     char numbuf[32], *s;
 
-    sprintf(numbuf, "%5d", top1);
+    M_snprintf(numbuf, sizeof(numbuf), "%5d", top1);
     // make frag count in player's color via escape code
 
     fragstr[i++] = '\x1b'; //jff 3/26/98 use ESC not '\' for paths
@@ -1090,79 +1141,58 @@ static void HU_widget_build_frag (void)
 static void HU_widget_build_monsec(void)
 {
   char hud_monsecstr[HU_MAXLINELENGTH];
-  int i, playerscount;
-  char kills_str[60];
-  int offset = 0;
+  int i;
+  int fullkillcount, fullitemcount, fullsecretcount;
+  int killcolor, itemcolor, secretcolor;
+  int kill_percent_count;
 
-  int kills = 0, kills_color, kills_percent, kills_percent_color;
-  int items = 0, items_color;
-  int secrets = 0, secrets_color;
+  fullkillcount = 0;
+  fullitemcount = 0;
+  fullsecretcount = 0;
+  kill_percent_count = 0;
 
-  for (i = 0, playerscount = 0; i < MAXPLAYERS; ++i)
+  for (i = 0; i < MAXPLAYERS; ++i)
   {
-    int color = (i == displayplayer) ? '0'+CR_GRAY : '0'+CR_GREEN;
     if (playeringame[i])
     {
-      if (playerscount == 0)
-      {
-        offset = sprintf(kills_str,
-          "\x1b%c%d", color, players[i].killcount);
-      }
-      else
-      {
-        offset += sprintf(kills_str + offset,
-          "\x1b%c+\x1b%c%d", '0'+CR_GREEN, color, players[i].killcount);
-      }
-
-      kills += players[i].killcount;
-      items += players[i].itemcount;
-      secrets += players[i].secretcount;
-      ++playerscount;
+      fullkillcount += players[i].killcount - players[i].maxkilldiscount;
+      fullitemcount += players[i].itemcount;
+      fullsecretcount += players[i].secretcount;
+      kill_percent_count += players[i].killcount;
     }
   }
 
-  kills_color = (kills - extrakills >= totalkills) ? '0'+CR_BLUE : '0'+CR_GRAY;
-  kills_percent_color = (kills >= totalkills) ? '0'+CR_BLUE : '0'+CR_GRAY;
-  kills_percent = (totalkills == 0) ? 100 : (kills * 100 / totalkills);
-  items_color = (items >= totalitems) ? '0'+CR_BLUE : '0'+CR_GRAY;
-  secrets_color = (secrets >= totalsecret) ? '0'+CR_BLUE : '0'+CR_GRAY;
-
-  if (playerscount > 1)
+  if (respawnmonsters)
   {
-    offset = sprintf(hud_monsecstr,
-      "\x1b%cK %s \x1b%c%d/%d",
-      '0'+CR_RED, kills_str, kills_color, kills, totalkills);
-  }
-  else
-  {
-    offset = sprintf(hud_monsecstr,
-      "\x1b%cK \x1b%c%d/%d",
-      '0'+CR_RED, kills_color, plr->killcount, totalkills);
+    fullkillcount = kill_percent_count;
+    max_kill_requirement = totalkills;
   }
 
-  if (extrakills)
-  {
-    offset += sprintf(hud_monsecstr + offset, "+%d", extrakills);
-  }
+  killcolor = (fullkillcount >= max_kill_requirement) ? '0'+CR_BLUE : '0'+CR_GRAY;
+  secretcolor = (fullsecretcount >= totalsecret) ? '0'+CR_BLUE : '0'+CR_GRAY;
+  itemcolor = (fullitemcount >= totalitems) ? '0'+CR_BLUE : '0'+CR_GRAY;
 
-  if (hud_threelined_widgets)
+  if (hud_widget_layout)
   {
-    sprintf(hud_monsecstr + offset, " \x1b%c%d%%", kills_percent_color, kills_percent);
+    M_snprintf(hud_monsecstr, sizeof(hud_monsecstr),
+      "\x1b%cK\t\x1b%c%d/%d", ('0'+CR_RED), killcolor, fullkillcount, max_kill_requirement);
     HUlib_add_string_to_cur_line(&w_monsec, hud_monsecstr);
 
-    sprintf(hud_monsecstr, "\x1b%cI \x1b%c%d/%d", ('0'+CR_RED), items_color, items, totalitems);
+    M_snprintf(hud_monsecstr, sizeof(hud_monsecstr),
+      "\x1b%cI\t\x1b%c%d/%d", ('0'+CR_RED), itemcolor, fullitemcount, totalitems);
     HUlib_add_string_to_cur_line(&w_monsec, hud_monsecstr);
 
-    sprintf(hud_monsecstr, "\x1b%cS \x1b%c%d/%d", ('0'+CR_RED), secrets_color, secrets, totalsecret);
+    M_snprintf(hud_monsecstr, sizeof(hud_monsecstr),
+      "\x1b%cS\t\x1b%c%d/%d", ('0'+CR_RED), secretcolor, fullsecretcount, totalsecret);
     HUlib_add_string_to_cur_line(&w_monsec, hud_monsecstr);
   }
   else
   {
-    sprintf(hud_monsecstr + offset,
-      " \x1b%c%d%% \x1b%cI \x1b%c%d/%d \x1b%cS \x1b%c%d/%d",
-      kills_percent_color, kills_percent,
-      '0'+CR_RED, items_color, items, totalitems,
-      '0'+CR_RED, secrets_color, secrets, totalsecret);
+    M_snprintf(hud_monsecstr, sizeof(hud_monsecstr),
+      "\x1b%cK \x1b%c%d/%d \x1b%cI \x1b%c%d/%d \x1b%cS \x1b%c%d/%d",
+      '0'+CR_RED, killcolor, fullkillcount, max_kill_requirement,
+      '0'+CR_RED, itemcolor, fullitemcount, totalitems,
+      '0'+CR_RED, secretcolor, fullsecretcount, totalsecret);
 
     HUlib_add_string_to_cur_line(&w_monsec, hud_monsecstr);
   }
@@ -1174,25 +1204,47 @@ static void HU_widget_build_sttime(void)
   int offset = 0;
   extern int time_scale;
 
-  if (time_scale != 100)
+  if ((hud_level_time & HUD_WIDGET_HUD     && !automapactive) ||
+      (hud_level_time & HUD_WIDGET_AUTOMAP &&  automapactive))
   {
-    offset += sprintf(hud_timestr, "\x1b%c%d%% ",
-            '0'+CR_BLUE, time_scale);
+    if (time_scale != 100)
+    {
+      offset += M_snprintf(hud_timestr, sizeof(hud_timestr), "\x1b%c%d%% ",
+                           '0'+CR_BLUE, time_scale);
+    }
+
+    if (totalleveltimes)
+    {
+      const int time = (totalleveltimes + leveltime) / TICRATE;
+
+      offset += M_snprintf(hud_timestr + offset, sizeof(hud_timestr),
+                           "\x1b%c%d:%02d ",
+                           '0'+CR_GREEN, time/60, time%60);
+    }
+
+    if (!plr->btuse_tics)
+    {
+      M_snprintf(hud_timestr + offset, sizeof(hud_timestr), "\x1b%c%d:%05.2f\t",
+                 '0'+CR_GRAY, leveltime / TICRATE / 60,
+                 (float)(leveltime % (60 * TICRATE)) / TICRATE);
+    }
   }
 
-  if (totalleveltimes)
+  if (plr->btuse_tics)
   {
-    const int time = (totalleveltimes + leveltime) / TICRATE;
-
-    offset += sprintf(hud_timestr + offset, "\x1b%c%d:%02d ",
-            '0'+CR_GREEN, time/60, time%60);
+    M_snprintf(hud_timestr + offset, sizeof(hud_timestr), "\x1b%cU %d:%05.2f\t",
+               '0'+CR_GOLD, plr->btuse / TICRATE / 60, 
+               (float)(plr->btuse % (60 * TICRATE)) / TICRATE);
   }
-  sprintf(hud_timestr + offset, "\x1b%c%d:%05.2f\t",
-    '0'+CR_GRAY, leveltime/TICRATE/60, (float)(leveltime%(60*TICRATE))/TICRATE);
 
   HUlib_add_string_to_cur_line(&w_sttime, hud_timestr);
 }
 
+void HU_widget_rebuild_sttime(void)
+{
+  HU_widget_build_sttime();
+}
+
 static void HU_widget_build_coord (void)
 {
   char hud_coordstr[HU_MAXLINELENGTH];
@@ -1203,20 +1255,20 @@ static void HU_widget_build_coord (void)
   AM_Coordinates(plr->mo, &x, &y, &z);
 
   //jff 2/16/98 output new coord display
-  if (hud_threelined_widgets)
+  if (hud_widget_layout)
   {
-    sprintf(hud_coordstr, "X \x1b%c%d", '0'+CR_GRAY, x >> FRACBITS);
+    M_snprintf(hud_coordstr, sizeof(hud_coordstr), "X\t\x1b%c%d", '0'+CR_GRAY, x >> FRACBITS);
     HUlib_add_string_to_cur_line(&w_coord, hud_coordstr);
 
-    sprintf(hud_coordstr, "Y \x1b%c%d", '0'+CR_GRAY, y >> FRACBITS);
+    M_snprintf(hud_coordstr, sizeof(hud_coordstr), "Y\t\x1b%c%d", '0'+CR_GRAY, y >> FRACBITS);
     HUlib_add_string_to_cur_line(&w_coord, hud_coordstr);
 
-    sprintf(hud_coordstr, "Z \x1b%c%d", '0'+CR_GRAY, z >> FRACBITS);
+    M_snprintf(hud_coordstr, sizeof(hud_coordstr), "Z\t\x1b%c%d", '0'+CR_GRAY, z >> FRACBITS);
     HUlib_add_string_to_cur_line(&w_coord, hud_coordstr);
   }
   else
   {
-    sprintf(hud_coordstr, "X \x1b%c%d \x1b%cY \x1b%c%d \x1b%cZ \x1b%c%d",
+    M_snprintf(hud_coordstr, sizeof(hud_coordstr), "X \x1b%c%d \x1b%cY \x1b%c%d \x1b%cZ \x1b%c%d",
             '0'+CR_GRAY, x >> FRACBITS, '0'+hudcolor_xyco,
             '0'+CR_GRAY, y >> FRACBITS, '0'+hudcolor_xyco,
             '0'+CR_GRAY, z >> FRACBITS);
@@ -1228,12 +1280,29 @@ static void HU_widget_build_coord (void)
 static void HU_widget_build_fps (void)
 {
   char hud_fpsstr[HU_MAXLINELENGTH/4];
-  extern int fps;
 
-  sprintf(hud_fpsstr,"\x1b%c%d \x1b%cFPS", '0'+CR_GRAY, fps, '0'+CR_ORIG);
+  M_snprintf(hud_fpsstr, sizeof(hud_fpsstr), "\x1b%c%d \x1b%cFPS",
+             '0'+CR_GRAY, fps, '0'+CR_ORIG);
   HUlib_add_string_to_cur_line(&w_fps, hud_fpsstr);
 }
 
+static void HU_widget_build_rate (void)
+{
+  char hud_ratestr[HU_MAXLINELENGTH];
+
+  M_snprintf(hud_ratestr, sizeof(hud_ratestr),
+             "Sprites %4d Segs %4d Visplanes %4d   \x1b%cFPS %3d %dx%d\x1b%c",
+             rendered_vissprites, rendered_segs, rendered_visplanes,
+             '0'+CR_GRAY, fps, video.width, video.height, '0'+CR_ORIG);
+  HUlib_add_string_to_cur_line(&w_rate, hud_ratestr);
+
+  if (voxels_rendering)
+  {
+    M_snprintf(hud_ratestr, sizeof(hud_ratestr), " Voxels %4d", rendered_voxels);
+    HUlib_add_string_to_cur_line(&w_rate, hud_ratestr);
+  }
+}
+
 // Crosshair
 
 boolean hud_crosshair_health;
@@ -1246,21 +1315,26 @@ typedef struct
 {
   patch_t *patch;
   int w, h, x, y;
-  char *cr;
+  byte *cr;
 } crosshair_t;
 
 static crosshair_t crosshair;
 
-const char *crosshair_nam[HU_CROSSHAIRS] =
-  { NULL,
-    "CROSS00", "CROSS01", "CROSS02", "CROSS03",
-    "CROSS04", "CROSS05", "CROSS06", "CROSS07",
-    "CROSS08" };
-const char *crosshair_str[HU_CROSSHAIRS+1] =
-  { "none",
-    "Cross", "Angle", "Dot", "Big Cross",
-    "Circle", "Big Circle", "Chevron", "Chevrons",
-    "Arcs", NULL };
+const char *crosshair_lumps[HU_CROSSHAIRS] =
+{
+  NULL,
+  "CROSS00", "CROSS01", "CROSS02", "CROSS03",
+  "CROSS04", "CROSS05", "CROSS06", "CROSS07",
+  "CROSS08"
+};
+
+const char *crosshair_strings[HU_CROSSHAIRS] =
+{
+  "Off",
+  "Cross", "Angle", "Dot", "Big Cross",
+  "Circle", "Big Circle", "Chevron", "Chevrons",
+  "Arcs"
+};
 
 static void HU_InitCrosshair(void)
 {
@@ -1268,13 +1342,13 @@ static void HU_InitCrosshair(void)
 
   for (i = 1; i < HU_CROSSHAIRS; i++)
   {
-    j = W_CheckNumForName(crosshair_nam[i]);
+    j = W_CheckNumForName(crosshair_lumps[i]);
     if (j >= num_predefined_lumps)
     {
       if (R_IsPatchLump(j))
-        crosshair_str[i] = crosshair_nam[i];
+        crosshair_strings[i] = crosshair_lumps[i];
       else
-        crosshair_nam[i] = NULL;
+        crosshair_lumps[i] = NULL;
     }
   }
 }
@@ -1284,9 +1358,9 @@ static void HU_StartCrosshair(void)
   if (crosshair.patch)
     Z_ChangeTag(crosshair.patch, PU_CACHE);
 
-  if (crosshair_nam[hud_crosshair])
+  if (crosshair_lumps[hud_crosshair])
   {
-    crosshair.patch = W_CacheLumpName(crosshair_nam[hud_crosshair], PU_STATIC);
+    crosshair.patch = W_CacheLumpName(crosshair_lumps[hud_crosshair], PU_STATIC);
 
     crosshair.w = SHORT(crosshair.patch->width)/2;
     crosshair.h = SHORT(crosshair.patch->height)/2;
@@ -1299,8 +1373,8 @@ mobj_t *crosshair_target; // [Alaux] Lock crosshair on target
 
 static void HU_UpdateCrosshair(void)
 {
-  crosshair.x = ORIGWIDTH/2;
-  crosshair.y = (screenblocks <= 10) ? (ORIGHEIGHT-ST_HEIGHT)/2 : ORIGHEIGHT/2;
+  crosshair.x = SCREENWIDTH/2;
+  crosshair.y = (screenblocks <= 10) ? (SCREENHEIGHT-ST_HEIGHT)/2 : SCREENHEIGHT/2;
 
   if (hud_crosshair_health)
     crosshair.cr = ColorByHealth(plr->health, 100, st_invul);
@@ -1317,13 +1391,13 @@ static void HU_UpdateCrosshair(void)
     crosshair_target = linetarget = NULL;
 
     overflow[emu_intercepts].enabled = false;
-    P_AimLineAttack(plr->mo, an, range, 0);
-    if (ammo == am_misl || ammo == am_cell)
+    P_AimLineAttack(plr->mo, an, range, CROSSHAIR_AIM);
+    if (!direct_vertical_aiming && (ammo == am_misl || ammo == am_cell))
     {
       if (!linetarget)
-        P_AimLineAttack(plr->mo, an += 1<<26, range, 0);
+        P_AimLineAttack(plr->mo, an += 1<<26, range, CROSSHAIR_AIM);
       if (!linetarget)
-        P_AimLineAttack(plr->mo, an -= 2<<26, range, 0);
+        P_AimLineAttack(plr->mo, an -= 2<<26, range, CROSSHAIR_AIM);
     }
     overflow[emu_intercepts].enabled = intercepts_overflow_enabled;
 
@@ -1347,14 +1421,14 @@ static void HU_UpdateCrosshair(void)
 
 void HU_UpdateCrosshairLock(int x, int y)
 {
-  int w = (crosshair.w << hires);
-  int h = (crosshair.h << hires);
+  int w = (crosshair.w * video.xscale) >> FRACBITS;
+  int h = (crosshair.h * video.yscale) >> FRACBITS;
 
   x = viewwindowx + BETWEEN(w, viewwidth  - w - 1, x);
   y = viewwindowy + BETWEEN(h, viewheight - h - 1, y);
 
-  crosshair.x = (x >> hires) - WIDESCREENDELTA;
-  crosshair.y = (y >> hires);
+  crosshair.x = (x << FRACBITS) / video.xscale - video.deltaw;
+  crosshair.y = (y << FRACBITS) / video.yscale;
 }
 
 void HU_DrawCrosshair(void)
@@ -1371,13 +1445,13 @@ void HU_DrawCrosshair(void)
   if (crosshair.patch)
     V_DrawPatchTranslated(crosshair.x - crosshair.w,
                           crosshair.y - crosshair.h,
-                          0, crosshair.patch, crosshair.cr);
+                          crosshair.patch, crosshair.cr);
 }
 
 // [crispy] print a bar indicating demo progress at the bottom of the screen
 boolean HU_DemoProgressBar(boolean force)
 {
-  const int progress = SCREENWIDTH * playback_tic / playback_totaltics;
+  const int progress = video.unscaledw * playback_tic / playback_totaltics;
   static int old_progress = 0;
 
   if (old_progress < progress)
@@ -1389,15 +1463,16 @@ boolean HU_DemoProgressBar(boolean force)
     return false;
   }
 
-  V_DrawHorizLine(0, SCREENHEIGHT - 2, 0, progress, darkest_color);
-  V_DrawHorizLine(0, SCREENHEIGHT - 1, 0, progress, lightest_color);
+  V_FillRect(0, SCREENHEIGHT - 2, progress, 1, v_darkest_color);
+  V_FillRect(0, SCREENHEIGHT - 1, progress, 1, v_lightest_color);
 
   return true;
 }
 
 // [FG] level stats and level time widgets
-int map_player_coords, map_level_stats, map_level_time;
-int hud_level_stats, hud_level_time;
+int hud_player_coords, hud_level_stats, hud_level_time;
+
+int hud_time_use;
 
 //
 // HU_Drawer()
@@ -1410,16 +1485,10 @@ void HU_Drawer(void)
 {
   hu_widget_t *w;
 
-  HUlib_reset_align_offsets();
+  if (hud_pending)
+    return;
 
-  // jff 4/24/98 Erase current lines before drawing current
-  // needed when screen not fullsize
-  // killough 11/98: only do it when not fullsize
-  // moved here to properly update the w_sttime and w_monsec widgets
-  if (scaledviewheight < 200)
-  {
-    HU_Erase();
-  }
+  HUlib_reset_align_offsets();
 
   w = doom_widget;
   while (w->multiline)
@@ -1450,9 +1519,9 @@ void HU_Drawer(void)
 // [FG] draw Time widget on intermission screen
 void WI_DrawTimeWidget(void)
 {
-  hu_widget_t w = {&w_sttime, align_left, align_top};
+  const hu_widget_t w = {&w_sttime, align_left, align_top};
 
-  if (hud_level_time)
+  if (hud_level_time & HUD_WIDGET_HUD)
   {
     HUlib_reset_align_offsets();
     // leveltime is already added to totalleveltimes before WI_Start()
@@ -1471,9 +1540,29 @@ void WI_DrawTimeWidget(void)
 
 void HU_Erase(void)
 {
-  // [FG] TODO optimize!
-  if (!automapactive && viewwindowx)
-    R_DrawViewBorder();
+  hu_widget_t *w;
+
+  if (automapactive || !scaledviewx)
+    return;
+
+  HUlib_reset_align_offsets();
+
+  w = doom_widget;
+  while (w->multiline)
+  {
+    HUlib_erase_widget(w);
+    w++;
+  }
+
+  w = boom_widget;
+  while (w->multiline)
+  {
+    if (w->multiline->built)
+    {
+      HUlib_erase_widget(w);
+    }
+    w++;
+  }
 }
 
 //
@@ -1498,7 +1587,9 @@ void HU_Ticker(void)
   HU_disable_all_widgets();
   draw_crispy_hud = false;
 
-  if ((automapactive && hud_widget_font == 1) || hud_widget_font == 2)
+  if ((automapactive && hud_widget_font == 1) || 
+      (!automapactive && hud_widget_font == 2) ||
+      hud_widget_font == 3)
   {
     boom_font = &big_font;
     CR_BLUE = CR_BLUE2;
@@ -1613,22 +1704,25 @@ void HU_Ticker(void)
 
   if (automapactive)
   {
-    HU_cond_build_widget(&w_monsec, map_level_stats);
-    HU_cond_build_widget(&w_sttime, map_level_time);
-    HU_cond_build_widget(&w_coord, STRICTMODE(map_player_coords));
+    HU_cond_build_widget(w_stats, hud_level_stats & HUD_WIDGET_AUTOMAP);
+    HU_cond_build_widget(&w_sttime, hud_level_time & HUD_WIDGET_AUTOMAP || plr->btuse_tics);
+    HU_cond_build_widget(&w_coord, STRICTMODE(hud_player_coords) & HUD_WIDGET_AUTOMAP);
   }
   else
   {
-    HU_cond_build_widget(&w_coord, STRICTMODE(map_player_coords) == 2);
+    HU_cond_build_widget(w_stats, hud_level_stats & HUD_WIDGET_HUD);
+    HU_cond_build_widget(&w_sttime, hud_level_time & HUD_WIDGET_HUD || plr->btuse_tics);
+    HU_cond_build_widget(&w_coord, STRICTMODE(hud_player_coords) & HUD_WIDGET_HUD);
   }
 
   HU_cond_build_widget(&w_fps, plr->cheats & CF_SHOWFPS);
+  HU_cond_build_widget(&w_rate, plr->cheats & CF_RENDERSTATS);
 
   if (hud_displayed &&
       scaledviewheight == SCREENHEIGHT &&
       automap_off)
   {
-    if (crispy_hud)
+    if (hud_type == HUD_TYPE_CRISPY)
     {
       if (hud_active > 0)
         draw_crispy_hud = true;
@@ -1641,21 +1735,16 @@ void HU_Ticker(void)
       HU_cond_build_widget(&w_ammo, true);
       HU_cond_build_widget(&w_keys, true);
     }
+  }
 
-    HU_cond_build_widget(&w_monsec, hud_level_stats);
-    HU_cond_build_widget(&w_sttime, hud_level_time);
-  }
-  else if (scaledviewheight &&
-           scaledviewheight < SCREENHEIGHT &&
-           automap_off)
-  {
-    HU_cond_build_widget(&w_monsec, hud_level_stats);
-    HU_cond_build_widget(&w_sttime, hud_level_time);
-  }
+  if (plr->btuse_tics)
+    plr->btuse_tics--;
 
   // update crosshair properties
   if (hud_crosshair)
     HU_UpdateCrosshair();
+
+  hud_pending = false;
 }
 
 #define QUEUESIZE   128
@@ -1884,7 +1973,8 @@ static const multiline_names_t
     {"monsec", "stats",   &w_monsec},
     {"sttime", "time",    &w_sttime},
     {"coord",  "coords",  &w_coord},
-    {"fps",    "rate",    &w_fps},
+    {"fps",     NULL,     &w_fps},
+    {"rate",    NULL,     &w_rate},
     {NULL},
 };
 
@@ -1897,7 +1987,7 @@ static boolean HU_ReplaceInDoomWidgets (hu_multiline_t *multiline, int hud, alig
     return false;
   }
 
-  for (i = 0; i < MAX_WIDGETS - 1; i++)
+  for (i = 0; i < MAX_WIDGETS_D - 1; i++)
   {
     if (doom_widgets[hud][i].multiline == multiline)
     {
@@ -1922,7 +2012,7 @@ static boolean HU_AddToBoomWidgets (hu_multiline_t *multiline, int hud, align_t
     return false;
   }
 
-  for (i = 0; i < MAX_WIDGETS - 1; i++)
+  for (i = 0; i < MAX_WIDGETS_B - 1; i++)
   {
     if (boom_widgets[hud][i].multiline == NULL)
     {
@@ -1930,7 +2020,7 @@ static boolean HU_AddToBoomWidgets (hu_multiline_t *multiline, int hud, align_t
     }
   }
 
-  if (i + 1 >= MAX_WIDGETS)
+  if (i + 1 >= MAX_WIDGETS_B)
   {
     return false;
   }
@@ -1995,14 +2085,14 @@ static boolean HU_AddHUDCoords (char *name, int hud, int x, int y)
   // [FG] relative alignment to the edges
   if (x < 0)
   {
-    x += ORIGWIDTH;
+    x += SCREENWIDTH;
   }
   if (y < 0)
   {
-    y += ORIGHEIGHT;
+    y += SCREENHEIGHT;
   }
 
-  if (x < 0 || x >= ORIGWIDTH || y < 0 || y >= ORIGHEIGHT)
+  if (x < 0 || x >= SCREENWIDTH || y < 0 || y >= SCREENHEIGHT)
   {
     return false;
   }
