@@ -15,6 +15,12 @@
 //      FluidSynth backend
 
 #include "fluidsynth.h"
+#include "SDL.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "config.h"
 
 #if (FLUIDSYNTH_VERSION_MAJOR < 2 || (FLUIDSYNTH_VERSION_MAJOR == 2 && FLUIDSYNTH_VERSION_MINOR < 2))
   typedef int fluid_int_t;
@@ -23,22 +29,20 @@
   typedef fluid_long_long_t fluid_int_t;
 #endif
 
-#include "SDL.h"
-#include "i_oalmusic.h"
-
+#include "d_iwad.h" // [FG] D_DoomExeDir()
 #include "doomtype.h"
+#include "i_glob.h"
+#include "i_oalmusic.h"
 #include "i_printf.h"
-#include "i_system.h"
 #include "i_sound.h"
+#include "m_array.h"
 #include "m_misc2.h"
 #include "memio.h"
 #include "mus2mid.h"
 #include "w_wad.h"
 #include "z_zone.h"
-#include "i_glob.h"
-#include "d_iwad.h" // [FG] D_DoomExeDir()
 
-char *soundfont_path = "";
+const char *soundfont_path = "";
 char *soundfont_dir = "";
 boolean mus_chorus;
 boolean mus_reverb;
@@ -48,11 +52,9 @@ static fluid_synth_t *synth = NULL;
 static fluid_settings_t *settings = NULL;
 static fluid_player_t *player = NULL;
 
-#define SOUNDFONTS_INITIAL_SIZE 16
-static char **soundfonts;
-static int soundfonts_num;
+static const char **soundfonts = NULL;
 
-static uint32_t FL_Callback(uint8_t *buffer, uint32_t buffer_samples)
+static int FL_Callback(byte *buffer, int buffer_samples)
 {
     int result;
 
@@ -110,19 +112,6 @@ static fluid_long_long_t FL_sftell(void *handle)
     return mem_ftell((MEMFILE *)handle);
 }
 
-static void AddSoundFont(const char *path)
-{
-    static int size;
-
-    if (soundfonts_num >= size)
-    {
-        size = (size ? size * 2 : SOUNDFONTS_INITIAL_SIZE);
-        soundfonts = I_Realloc(soundfonts, size * sizeof(*soundfonts));
-    }
-
-    soundfonts[soundfonts_num++] = M_StringDuplicate(path);
-}
-
 static void ScanDir(const char *dir)
 {
     char *rel = NULL;
@@ -147,7 +136,7 @@ static void ScanDir(const char *dir)
             break;
         }
 
-        AddSoundFont(filename);
+        array_push(soundfonts, M_StringDuplicate(filename));
     }
 
     I_EndGlob(glob);
@@ -162,7 +151,7 @@ static void GetSoundFonts(void)
 {
     char *left, *p, *dup_path;
 
-    if (soundfonts_num)
+    if (array_size(soundfonts))
     {
         return;
     }
@@ -281,11 +270,11 @@ static boolean I_FL_InitMusic(int device)
     {
         if (device == DEFAULT_MIDI_DEVICE)
         {
-            int i;
-
             GetSoundFonts();
 
-            for (i = 0; i < soundfonts_num; ++i)
+            device = 0;
+
+            for (int i = 0; i < array_size(soundfonts); ++i)
             {
                 if (!strcasecmp(soundfonts[i], soundfont_path))
                 {
@@ -293,15 +282,11 @@ static boolean I_FL_InitMusic(int device)
                     break;
                 }
             }
-            if (i == soundfonts_num)
-            {
-                device = 0;
-            }
         }
 
-        if (soundfonts_num)
+        if (array_size(soundfonts))
         {
-            if (device >= soundfonts_num)
+            if (device >= array_size(soundfonts))
             {
                 device = 0;
             }
@@ -452,25 +437,21 @@ static void I_FL_ShutdownMusic(void)
 
 #define NAME_MAX_LENGTH 25
 
-static int I_FL_DeviceList(const char *devices[], int size, int *current_device)
+static const char **I_FL_DeviceList(int *current_device)
 {
-    int i;
+    const char **devices = NULL;
 
     *current_device = 0;
 
     if (W_CheckNumForName("SNDFONT") >= 0)
     {
-        if (size > 0)
-        {
-            devices[0] = "FluidSynth (SNDFONT)";
-            return 1;
-        }
-        return 0;
+        array_push(devices, "FluidSynth (SNDFONT)");
+        return devices;
     }
 
     GetSoundFonts();
 
-    for (i = 0; i < size && i < soundfonts_num; ++i)
+    for (int i = 0; i < array_size(soundfonts); ++i)
     {
         char *name = M_StringDuplicate(M_BaseName(soundfonts[i]));
         if (strlen(name) >= NAME_MAX_LENGTH)
@@ -478,7 +459,7 @@ static int I_FL_DeviceList(const char *devices[], int size, int *current_device)
             name[NAME_MAX_LENGTH] = '\0';
         }
 
-        devices[i] = M_StringJoin("FluidSynth (", name, ")", NULL);
+        array_push(devices, M_StringJoin("FluidSynth (", name, ")", NULL));
 
         if (!strcasecmp(soundfonts[i], soundfont_path))
         {
@@ -486,8 +467,12 @@ static int I_FL_DeviceList(const char *devices[], int size, int *current_device)
         }
         free(name);
     }
+    return devices;
+}
 
-    return i;
+static void I_FL_UpdateMusic(void)
+{
+    ;
 }
 
 music_module_t music_fl_module =
@@ -499,6 +484,7 @@ music_module_t music_fl_module =
     I_FL_ResumeSong,
     I_FL_RegisterSong,
     I_FL_PlaySong,
+    I_FL_UpdateMusic,
     I_FL_StopSong,
     I_FL_UnRegisterSong,
     I_FL_DeviceList,
