@@ -16,15 +16,23 @@
 //      System interface for OpenAL sound.
 //
 
-#include <stdio.h>
+#include "al.h"
+#include "alc.h"
+#include "alext.h"
+#include "efx.h"
+
+#include <math.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include "doomstat.h"
-#include "i_printf.h"
 #include "i_oalsound.h"
+#include "i_printf.h"
 #include "i_sndfile.h"
-#include "r_data.h"
+#include "i_sound.h"
+#include "m_fixed.h"
+#include "sounds.h"
 #include "w_wad.h"
+#include "z_zone.h"
 
 #define OAL_ROLLOFF_FACTOR 1
 #define OAL_SPEED_OF_SOUND 343.3f
@@ -312,13 +320,8 @@ void I_OAL_UpdateListenerParams(const ALfloat *position, const ALfloat *velocity
     alListenerfv(AL_ORIENTATION, orientation);
 }
 
-void I_OAL_UpdateUserSoundSettings(void)
+static void UpdateUserSoundSettings(void)
 {
-    if (!oal)
-    {
-        return;
-    }
-
     SetResampler(oal->sources);
 
     if (snd_module == SND_MODULE_3D)
@@ -370,29 +373,21 @@ static void ResetParams(void)
     alDistanceModel(AL_INVERSE_DISTANCE_CLAMPED); // OpenAL 1.1 Specs, 3.4.2.
     alSpeedOfSound(OAL_SPEED_OF_SOUND * OAL_MAP_UNITS_PER_METER);
 
-    I_OAL_UpdateUserSoundSettings();
+    UpdateUserSoundSettings();
 }
 
-static boolean OpenDevice(ALCdevice **device)
+static void PrintDeviceInfo(ALCdevice *device)
 {
     const ALCchar *name;
     ALCint srate = -1;
 
-    *device = alcOpenDevice(NULL);
-    if (!*device)
-    {
-        return false;
-    }
-
-    if (alcIsExtensionPresent(*device, "ALC_ENUMERATE_ALL_EXT") == ALC_TRUE)
-        name = alcGetString(*device, ALC_ALL_DEVICES_SPECIFIER);
+    if (alcIsExtensionPresent(device, "ALC_ENUMERATE_ALL_EXT") == ALC_TRUE)
+        name = alcGetString(device, ALC_ALL_DEVICES_SPECIFIER);
     else
-        name = alcGetString(*device, ALC_DEVICE_SPECIFIER);
+        name = alcGetString(device, ALC_DEVICE_SPECIFIER);
 
-    alcGetIntegerv(*device, ALC_FREQUENCY, 1, &srate);
+    alcGetIntegerv(device, ALC_FREQUENCY, 1, &srate);
     I_Printf(VB_INFO, " Using '%s' @ %d Hz.", name, srate);
-
-    return true;
 }
 
 static void GetAttribs(ALCint *attribs)
@@ -420,7 +415,6 @@ static void GetAttribs(ALCint *attribs)
 
 boolean I_OAL_InitSound(void)
 {
-    ALCdevice *device;
     ALCint attribs[OAL_NUM_ATTRIBS];
 
     if (oal)
@@ -428,16 +422,17 @@ boolean I_OAL_InitSound(void)
         I_OAL_ShutdownSound();
     }
 
-    if (!OpenDevice(&device))
+    oal = calloc(1, sizeof(*oal));
+    oal->device = alcOpenDevice(NULL);
+    if (!oal->device)
     {
         I_Printf(VB_ERROR, "I_OAL_InitSound: Failed to open device.");
+        free(oal);
+        oal = NULL;
         return false;
     }
 
-    oal = calloc(1, sizeof(*oal));
-    oal->device = device;
     GetAttribs(attribs);
-
     oal->context = alcCreateContext(oal->device, attribs);
     if (!oal->context || !alcMakeContextCurrent(oal->context))
     {
@@ -445,6 +440,7 @@ boolean I_OAL_InitSound(void)
         I_OAL_ShutdownSound();
         return false;
     }
+    PrintDeviceInfo(oal->device);
 
     oal->sources = malloc(sizeof(*oal->sources) * MAX_CHANNELS);
     alGetError();
@@ -546,6 +542,22 @@ static boolean IsPaddedSound(const byte *data, int size)
     return true;
 }
 
+static void FadeInMono8(byte *data, ALsizei size, ALsizei freq)
+{
+    const int fadelen = freq * FADETIME / 1000000;
+    int i;
+
+    if (data[0] == 128 || size < fadelen)
+    {
+        return;
+    }
+
+    for (i = 0; i < fadelen; i++)
+    {
+        data[i] = (data[i] - 128) * i / fadelen + 128;
+    }
+}
+
 boolean I_OAL_CacheSound(sfxinfo_t *sfx)
 {
     int lumpnum;
@@ -604,6 +616,9 @@ boolean I_OAL_CacheSound(sfxinfo_t *sfx)
 
             // All Doom sounds are 8-bit
             format = AL_FORMAT_MONO8;
+
+            // Fade in sounds that start at a non-zero amplitude to prevent clicking.
+            FadeInMono8(sampledata, size, freq);
         }
         else
         {
@@ -611,6 +626,7 @@ boolean I_OAL_CacheSound(sfxinfo_t *sfx)
 
             if (I_SND_LoadFile(lumpdata, &format, &wavdata, &size, &freq) == false)
             {
+                I_Printf(VB_WARNING, " I_OAL_CacheSound: %s", lumpinfo[lumpnum].name);
                 break;
             }
 
