@@ -16,26 +16,23 @@
 //   System interface for music.
 //
 
-
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "mus2mid.h"
-#include "memio.h"
-#include "doomtype.h"
+#include "../opl/opl.h"
 
+#include "doomtype.h"
 #include "i_printf.h"
 #include "i_sound.h"
-#include "m_swap.h"
-#include "m_misc2.h"
+#include "m_array.h"
 #include "m_io.h"
+#include "m_swap.h"
+#include "memio.h"
+#include "midifile.h"
+#include "mus2mid.h"
 #include "w_wad.h"
 #include "z_zone.h"
 
-#include "../opl/opl.h"
-#include "midifile.h"
-
 // #define OPL_MIDI_DEBUG
 
 #define MAXMIDLENGTH (96 * 1024)
@@ -1769,15 +1766,17 @@ static boolean OPL_InitMusic(void)
     return true;
 }
 
-static int I_OPL_DeviceList(const char* devices[], int size, int *current_device)
+static const char **I_OPL_DeviceList(int *current_device)
 {
+    const char **devices = NULL;
     *current_device = 0;
-    if (size > 0)
-    {
-        devices[0] = "OPL3 Emulation";
-        return 1;
-    }
-    return 0;
+    array_push(devices, "OPL3 Emulation");
+    return devices;
+}
+
+static void I_OPL_UpdateMusic(void)
+{
+    ;
 }
 
 music_module_t music_opl_module =
@@ -1789,6 +1788,7 @@ music_module_t music_opl_module =
     I_OPL_ResumeSong,
     I_OPL_RegisterSong,
     I_OPL_PlaySong,
+    I_OPL_UpdateMusic,
     I_OPL_StopSong,
     I_OPL_UnRegisterSong,
     I_OPL_DeviceList,
