@@ -16,15 +16,21 @@
 // DESCRIPTION:
 //      Load sound lumps with libsndfile.
 
-#include "i_sndfile.h"
-#include "i_printf.h"
-
+#include "sndfile.h"
+#include "al.h"
 #include "alext.h"
-#include <sndfile.h>
+
+#include "i_sndfile.h"
+
+#include <math.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "config.h"
 #include "i_oalstream.h"
+#include "i_printf.h"
 #include "m_swap.h"
 #include "memio.h"
 
@@ -394,7 +400,7 @@ static boolean OpenFile(sndfile_t *file, void *data, sf_count_t size)
 
     if (!file->sndfile)
     {
-        I_Printf(VB_WARNING, "SndFile: %s", sf_strerror(file->sndfile));
+        I_Printf(VB_DEBUG, "SndFile: %s", sf_strerror(file->sndfile));
         return false;
     }
 
@@ -483,6 +489,38 @@ static boolean OpenFile(sndfile_t *file, void *data, sf_count_t size)
     return true;
 }
 
+static void FadeInMono16(short *data, ALsizei size, ALsizei freq)
+{
+    const int fadelen = freq * FADETIME / 1000000;
+    int i;
+
+    if (!data[0] || size / sizeof(short) < fadelen)
+    {
+        return;
+    }
+
+    for (i = 0; i < fadelen; i++)
+    {
+        data[i] = data[i] * i / fadelen;
+    }
+}
+
+static void FadeInMonoFloat32(float *data, ALsizei size, ALsizei freq)
+{
+    const int fadelen = freq * FADETIME / 1000000;
+    int i;
+
+    if (fabsf(data[0]) < 0.000001f || size / sizeof(float) < fadelen)
+    {
+        return;
+    }
+
+    for (i = 0; i < fadelen; i++)
+    {
+        data[i] = data[i] * i / fadelen;
+    }
+}
+
 boolean I_SND_LoadFile(void *data, ALenum *format, byte **wavdata,
                        ALsizei *size, ALsizei *freq)
 {
@@ -519,10 +557,21 @@ boolean I_SND_LoadFile(void *data, ALenum *format, byte **wavdata,
         return false;
     }
 
-    *wavdata = local_wavdata;
     *size = num_frames * file.frame_size / file.sfinfo.channels;
     *freq = file.sfinfo.samplerate;
 
+    // Fade in sounds that start at a non-zero amplitude to prevent clicking.
+    if (*format == AL_FORMAT_MONO16)
+    {
+        FadeInMono16(local_wavdata, *size, *freq);
+    }
+    else if (*format == AL_FORMAT_MONO_FLOAT32)
+    {
+        FadeInMonoFloat32(local_wavdata, *size, *freq);
+    }
+
+    *wavdata = local_wavdata;
+
     CloseFile(&file);
 
     return true;
@@ -574,7 +623,7 @@ static void I_SND_PlayStream(boolean looping)
     stream_looping = looping;
 }
 
-static uint32_t I_SND_FillStream(byte *data, uint32_t frames)
+static int I_SND_FillStream(byte *data, int frames)
 {
     sf_count_t filled = 0;
     boolean restart = false;
