@@ -19,10 +19,10 @@
 
 #include "SDL.h"
 
-#include "i_timer.h"
+#include "doomdef.h"
+#include "doomtype.h"
+#include "i_system.h"
 #include "m_fixed.h"
-#include "doomstat.h"
-#include "m_argv.h"
 
 static uint64_t basecounter = 0;
 static uint64_t basefreq = 0;
@@ -116,8 +116,20 @@ static int I_GetFracTime_FastDemo(void)
 
 int (*I_GetFracTime)(void) = I_GetFracTime_Scaled;
 
+void I_ShutdownTimer(void)
+{
+    SDL_QuitSubSystem(SDL_INIT_TIMER);
+}
+
 void I_InitTimer(void)
 {
+    if (SDL_Init(SDL_INIT_TIMER) < 0)
+    {
+        I_Error("I_InitTimer: Failed to initialize timer: %s", SDL_GetError());
+    }
+
+    I_AtExit(I_ShutdownTimer, true);
+
     basefreq = SDL_GetPerformanceFrequency();
 
     I_GetTime = I_GetTime_Scaled;
