@@ -17,25 +17,37 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "am_map.h"
+#include "d_deh.h" // Ty 03/27/98 - externalized strings
+#include "d_event.h"
+#include "d_player.h"
+#include "d_think.h"
+#include "doomdata.h"
+#include "doomdef.h"
 #include "doomstat.h"
-#include "p_tick.h"
 #include "g_game.h"
-#include "r_data.h"
+#include "info.h"
+#include "m_cheat.h"
+#include "m_fixed.h"
+#include "m_input.h"
+#include "m_misc2.h"
 #include "p_inter.h"
 #include "p_map.h"
-#include "m_cheat.h"
-#include "m_argv.h"
+#include "p_mobj.h"
+#include "p_pspr.h"
+#include "p_spec.h" // SPECHITS
+#include "p_tick.h"
+#include "r_defs.h"
+#include "r_state.h"
 #include "s_sound.h"
 #include "sounds.h"
-#include "dstrings.h"
-#include "d_deh.h"  // Ty 03/27/98 - externalized strings
+#include "tables.h"
 #include "u_mapinfo.h"
 #include "w_wad.h"
-#include "m_misc2.h"
-#include "p_spec.h" // SPECHITS
-#include "d_main.h"
-#include "m_input.h"
-#include "am_map.h"
 
 #define plyr (players+consoleplayer)     /* the console player */
 
@@ -419,6 +431,11 @@ static void cheat_god()
     P_SpawnMobj(plyr->mo->x+20*finecosine[an], plyr->mo->y+20*finesine[an], plyr->mo->z, MT_TFOG);
     S_StartSound(plyr->mo, sfx_slop);
     P_MapEnd();
+
+    // Fix reviving as "zombie" if god mode was already enabled
+    if (plyr->mo)
+      plyr->mo->health = god_health;  // Ty 03/09/98 - deh
+    plyr->health = god_health;
   }
 
   plyr->cheats ^= CF_GODMODE;
@@ -660,7 +677,6 @@ static void cheat_clev(char *buf)
 // killough 2/7/98: simplified using dprintf and made output more user-friendly
 static void cheat_mypos()
 {
-  plyr->cheats &= ~CF_RENDERSTATS;
   plyr->cheats ^= CF_MAPCOORDS;
   if ((plyr->cheats & CF_MAPCOORDS) == 0)
     plyr->message = "";
@@ -742,7 +758,6 @@ static void cheat_tran()
 {
   displaymsg(            // Ty 03/27/98 - *not* externalized
     (translucency = !translucency) ? "Translucency enabled" : "Translucency disabled");
-  D_SetPredefinedTranslucency();
 }
 
 static void cheat_massacre()    // jff 2/01/98 kill all monsters
@@ -1157,10 +1172,7 @@ static void cheat_nuke()
 
 static void cheat_rate()
 {
-  plyr->cheats &= ~CF_MAPCOORDS;
   plyr->cheats ^= CF_RENDERSTATS;
-  if ((plyr->cheats & CF_RENDERSTATS) == 0)
-    plyr->message = "";
 }
 
 //-----------------------------------------------------------------------------
@@ -1281,9 +1293,6 @@ boolean M_CheatResponder(event_t *ev)
 {
   int i;
 
-  if (strictmode)
-    return false;
-
   if (ev->type == ev_keydown && M_FindCheats(ev->data1))
     return true;
 
