@@ -14,29 +14,23 @@
 // Network client code
 //
 
-#include <ctype.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "config.h"
+#include "d_ticcmd.h"
 #include "doomtype.h"
 #include "i_printf.h"
-#include "i_system.h"
-#include "i_timer.h" // I_Sleep
-#include "m_argv.h"
+#include "i_timer.h"
 #include "m_fixed.h"
 #include "m_misc2.h"
 #include "net_client.h"
 #include "net_common.h"
-#include "net_defs.h"
 #include "net_io.h"
 #include "net_packet.h"
 #include "net_query.h"
 #include "net_server.h"
 #include "net_structrw.h"
-#include "net_petname.h"
-#include "w_wad.h"
 
 extern void D_ReceiveTic(ticcmd_t *ticcmds, boolean *playeringame);
 
@@ -1205,9 +1199,9 @@ void NET_CL_Disconnect(void)
 
 void NET_CL_Init(void)
 {
-    if (net_player_name == NULL || !strcasecmp("none", net_player_name))
+    if (net_player_name == NULL)
     {
-        net_player_name = NET_GetRandomPetName();
+        net_player_name = M_StringDuplicate(DEFAULT_PLAYER_NAME);
     }
 }
 
