@@ -19,17 +19,17 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 
 #include "config.h"
-
+#include "doomkeys.h"
+#include "doomtype.h"
 #include "i_printf.h"
 #include "i_system.h"
 #include "i_video.h"
 #include "m_argv.h"
 #include "m_misc2.h"
-
 #include "net_client.h"
+#include "net_defs.h"
 #include "net_gui.h"
 #include "net_query.h"
 #include "net_server.h"
