@@ -14,8 +14,6 @@
 // Reading and writing various structures into packets
 //
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "doomtype.h"
