@@ -18,17 +18,25 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <math.h>
+#include <string.h>
+
+#include "doomdata.h"
 #include "i_printf.h"
-#include "r_main.h"
-#include "w_wad.h"
+#include "i_system.h"
 #include "m_argv.h"
+#include "m_fixed.h"
 #include "m_swap.h"
-
-// [FG] support maps with NODES in compressed ZNOD/ZGLN formats
-#include "../miniz/miniz.h"
-
 #include "p_extnodes.h"
 #include "p_setup.h"
+#include "r_defs.h"
+#include "r_main.h"
+#include "r_state.h"
+#include "w_wad.h"
+#include "z_zone.h"
+
+// [FG] support maps with NODES in compressed ZNOD/ZGLN formats
+#include "miniz.h"
 
 // [FG] support maps with NODES in DeePBSP format
 
@@ -95,13 +103,25 @@ typedef PACKED_PREFIX struct
 mapformat_t P_CheckMapFormat(int lumpnum)
 {
     mapformat_t format = MFMT_DOOM;
-    byte *nodes = NULL;
-    int b;
+    byte *lump_data = NULL;
+    int size_subs = 0, size_nodes = 0;
 
-    if ((b = lumpnum + ML_BLOCKMAP + 1) < numlumps &&
-        !strcasecmp(lumpinfo[b].name, "BEHAVIOR"))
+    if (W_LumpExistsWithName(lumpnum + ML_BLOCKMAP + 1, "BEHAVIOR"))
+    {
         I_Error("P_SetupLevel: Hexen map format not supported in %s.\n",
                 lumpinfo[lumpnum].name);
+    }
+
+    //!
+    // @category mod
+    //
+    // Forces rebuilding of nodes.
+    //
+
+    if (M_CheckParm("-bsp"))
+    {
+        return MFMT_UNSUPPORTED;
+    }
 
     //!
     // @category mod
@@ -111,51 +131,68 @@ mapformat_t P_CheckMapFormat(int lumpnum)
 
     if (!M_CheckParm("-force_old_zdoom_nodes"))
     {
-        if ((b = lumpnum + ML_SSECTORS) < numlumps &&
-            (nodes = W_CacheLumpNum(b, PU_STATIC)) && W_LumpLength(b) > 4)
+        size_subs = W_LumpLengthWithName(lumpnum + ML_SSECTORS, "SSECTORS");
+
+        if (size_subs >= sizeof(mapsubsector_t))
         {
-            if (!memcmp(nodes, "XGLN", 4))
+            lump_data = W_CacheLumpNum(lumpnum + ML_SSECTORS, PU_STATIC);
+
+            if (!memcmp(lump_data, "XGLN", 4))
                 format = MFMT_XGLN;
-            else if (!memcmp(nodes, "ZGLN", 4))
+            else if (!memcmp(lump_data, "ZGLN", 4))
                 format = MFMT_ZGLN;
-            else if (!memcmp(nodes, "XGL2", 4))
+            else if (!memcmp(lump_data, "XGL2", 4))
                 format = MFMT_XGL2;
-            else if (!memcmp(nodes, "ZGL2", 4))
+            else if (!memcmp(lump_data, "ZGL2", 4))
                 format = MFMT_ZGL2;
-            else if (!memcmp(nodes, "XGL3", 4))
+            else if (!memcmp(lump_data, "XGL3", 4))
                 format = MFMT_XGL3;
-            else if (!memcmp(nodes, "ZGL3", 4))
+            else if (!memcmp(lump_data, "ZGL3", 4))
                 format = MFMT_ZGL3;
         }
+        else
+        {
+            format = MFMT_UNSUPPORTED;
+        }
     }
 
-    if (nodes)
+    if (lump_data)
     {
-        Z_Free(nodes);
-        nodes = NULL;
+        Z_Free(lump_data);
+        lump_data = NULL;
     }
 
     if (format == MFMT_DOOM || format >= MFMT_UNSUPPORTED)
     {
-        if ((b = lumpnum + ML_NODES) < numlumps &&
-            (nodes = W_CacheLumpNum(b, PU_STATIC)) && W_LumpLength(b) > 8)
+        size_nodes = W_LumpLengthWithName(lumpnum + ML_NODES, "NODES");
+
+        if (size_nodes >= sizeof(mapnode_t))
         {
-            if (!memcmp(nodes, "xNd4\0\0\0\0", 8))
+            lump_data = W_CacheLumpNum(lumpnum + ML_NODES, PU_STATIC);
+
+            if (!memcmp(lump_data, "xNd4\0\0\0\0", 8))
                 format = MFMT_DEEP;
-            else if (!memcmp(nodes, "XNOD", 4))
+            else if (!memcmp(lump_data, "XNOD", 4))
                 format = MFMT_XNOD;
-            else if (!memcmp(nodes, "ZNOD", 4))
+            else if (!memcmp(lump_data, "ZNOD", 4))
                 format = MFMT_ZNOD;
         }
+        else
+        {
+            format = MFMT_UNSUPPORTED;
+        }
     }
 
-    if (nodes)
-        Z_Free(nodes);
-
-    if (format >= MFMT_UNSUPPORTED)
+    // [FG] no nodes for exactly one subsector
+    if (size_subs == sizeof(mapsubsector_t) && size_nodes == 0)
     {
-        I_Error("P_SetupLevel: Nodes format not supported in %s.\n",
-                lumpinfo[lumpnum].name);
+        format = MFMT_DOOM;
+    }
+
+    if (lump_data)
+    {
+        Z_Free(lump_data);
+        lump_data = NULL;
     }
 
     return format;
@@ -163,14 +200,14 @@ mapformat_t P_CheckMapFormat(int lumpnum)
 
 // [FG] recalculate seg offsets
 
-fixed_t P_GetOffset(vertex_t *v1, vertex_t *v2)
+int P_GetOffset(vertex_t *v1, vertex_t *v2)
 {
-    fixed_t dx, dy;
-    fixed_t r;
+    float a, b;
+    int r;
 
-    dx = (v1->x - v2->x) >> FRACBITS;
-    dy = (v1->y - v2->y) >> FRACBITS;
-    r = (fixed_t) (sqrt(dx * dx + dy * dy)) << FRACBITS;
+    a = (float)(v1->x - v2->x) / (float)FRACUNIT;
+    b = (float)(v1->y - v2->y) / (float)FRACUNIT;
+    r = (int)(sqrt(a*a+b*b) * (float)FRACUNIT);
 
     return r;
 }
@@ -536,8 +573,9 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
         if (err != Z_STREAM_END)
             I_Error("P_LoadNodes_XNOD: Error during ZNOD nodes decompression!");
 
-        I_Printf(VB_DEBUG, "P_LoadNodes_XNOD: ZNOD nodes compression ratio %.3f",
-                (float) zstream->total_out / zstream->total_in);
+        I_Printf(VB_DEBUG,
+                 "P_LoadNodes_XNOD: ZNOD nodes compression ratio %.3f",
+                 (float) zstream->total_out / zstream->total_in);
 
         data = output;
 
