@@ -22,6 +22,10 @@
 #ifndef __P_EXTNODES__
 #define __P_EXTNODES__
 
+#include "doomtype.h"
+
+struct vertex_s;
+
 typedef enum
 {
     MFMT_DOOM,
@@ -39,7 +43,7 @@ typedef enum
 } mapformat_t;
 
 extern mapformat_t P_CheckMapFormat(int lumpnum);
-extern fixed_t P_GetOffset(vertex_t *v1, vertex_t *v2);
+extern int P_GetOffset(struct vertex_s *v1, struct vertex_s *v2);
 
 extern void P_LoadSegs_DEEP(int lump);
 extern void P_LoadSubsectors_DEEP(int lump);
