@@ -17,17 +17,27 @@
 //
 //-----------------------------------------------------------------------------
 
-#include "doomstat.h"
-#include "dstrings.h"
-#include "m_random.h"
+#include "p_inter.h"
+
 #include "am_map.h"
+#include "d_deh.h" // Ty 03/22/98 - externalized strings
+#include "d_items.h"
+#include "d_player.h"
+#include "d_think.h"
+#include "doomdef.h"
+#include "doomstat.h"
+#include "i_system.h"
+#include "info.h"
+#include "m_fixed.h"
+#include "m_random.h"
+#include "p_mobj.h"
+#include "p_pspr.h"
+#include "p_tick.h"
+#include "r_defs.h"
 #include "r_main.h"
 #include "s_sound.h"
 #include "sounds.h"
-#include "p_tick.h"
-#include "d_deh.h"  // Ty 03/22/98 - externalized strings
-
-#include "p_inter.h"
+#include "tables.h"
 
 #define BONUSADD        6
 
@@ -39,6 +49,7 @@
 int initial_health = 100;
 int initial_bullets = 50;
 int maxhealth = 100; // was MAXHEALTH as a #define, used only in this module
+int maxhealthbonus = 200;
 int max_armor = 200;
 int green_armor_class = 1;  // these are involved with armortype below
 int blue_armor_class = 2;
@@ -334,8 +345,8 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 	}
 
       player->health++;               // can go over 100%
-      if (player->health > (maxhealth * 2))
-        player->health = (maxhealth * 2);
+      if (player->health > maxhealthbonus)
+        player->health = maxhealthbonus;
       player->mo->health = player->health;
       pickupmsg(player, "%s", s_GOTHTHBONUS); // Ty 03/22/98 - externalized
       break;
@@ -649,6 +660,16 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 //
 // killough 11/98: make static
 
+static void WatchKill(player_t* player, mobj_t* target)
+{
+  player->killcount++;
+
+  if (target->intflags & MIF_SPAWNED_BY_ICON)
+  {
+    player->maxkilldiscount++;
+  }
+}
+
 static void P_KillMobj(mobj_t *source, mobj_t *target, method_t mod)
 {
   mobjtype_t item;
@@ -671,7 +692,7 @@ static void P_KillMobj(mobj_t *source, mobj_t *target, method_t mod)
       // killough 7/20/98: don't count friends
       if (!(target->flags & MF_FRIEND))
 	if (target->flags & MF_COUNTKILL)
-	  source->player->killcount++;
+	  WatchKill(source->player, target);
       if (target->player)
         source->player->frags[target->player-players]++;
     }
@@ -682,7 +703,7 @@ static void P_KillMobj(mobj_t *source, mobj_t *target, method_t mod)
           // even those caused by other monsters
 	  // killough 7/20/98: don't count friends
 	  if (!(target->flags & MF_FRIEND))
-	    players->killcount++;
+	    WatchKill(players, target);
         }
 #ifndef MBF_STRICT
       // For compatibility with PrBoom+ complevel 11 netgame
@@ -692,7 +713,7 @@ static void P_KillMobj(mobj_t *source, mobj_t *target, method_t mod)
           if (target->lastenemy && target->lastenemy->health > 0 &&
               target->lastenemy->player)
           {
-              target->lastenemy->player->killcount++;
+              WatchKill(target->lastenemy->player, target);
           }
           else
           {
@@ -711,7 +732,7 @@ static void P_KillMobj(mobj_t *source, mobj_t *target, method_t mod)
               else
                 i = Woof_Random() % playerscount;
 
-              players[i].killcount++;
+              WatchKill(&players[i], target);
             }
           }
         }
