@@ -19,12 +19,15 @@
 //
 //-----------------------------------------------------------------------------
 
+#include "d_think.h"
 #include "doomstat.h" //jff 5/18/98
-#include "doomdef.h"
+#include "m_fixed.h"
 #include "m_random.h"
-#include "r_main.h"
 #include "p_spec.h"
 #include "p_tick.h"
+#include "r_defs.h"
+#include "r_state.h"
+#include "z_zone.h"
 
 //////////////////////////////////////////////////////////
 //
