@@ -18,22 +18,33 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <stdlib.h>
+
+#include "d_player.h"
+#include "doomdata.h"
+#include "doomdef.h"
 #include "doomstat.h"
+#include "hu_obituary.h"
 #include "i_printf.h"
-#include "r_main.h"
-#include "p_mobj.h"
-#include "p_maputl.h"
+#include "i_system.h"
+#include "info.h"
+#include "m_argv.h"
+#include "m_bbox.h"
+#include "m_misc2.h"
+#include "m_random.h"
+#include "p_inter.h"
 #include "p_map.h"
+#include "p_maputl.h"
+#include "p_mobj.h"
 #include "p_setup.h"
 #include "p_spec.h"
+#include "r_defs.h"
+#include "r_main.h"
+#include "r_state.h"
 #include "s_sound.h"
 #include "sounds.h"
-#include "p_inter.h"
-#include "m_random.h"
-#include "m_bbox.h"
 #include "v_video.h"
-#include "m_argv.h"
-#include "m_misc2.h"
+#include "z_zone.h"
 
 static mobj_t    *tmthing;
 static int       tmflags;
@@ -491,6 +502,13 @@ static boolean P_ProjectileImmune(mobj_t *target, mobj_t *source)
     );
 }
 
+// [FG] mobj or actual sprite height
+static const inline fixed_t thingheight (const mobj_t *const thing, const mobj_t *const cond)
+{
+  return (direct_vertical_aiming && cond && cond->player && thing->actualheight > thing->height) ?
+        thing->actualheight : thing->height;
+}
+
 static boolean PIT_CheckThing(mobj_t *thing) // killough 3/26/98: make static
 {
   fixed_t blockdist;
@@ -565,13 +583,9 @@ static boolean PIT_CheckThing(mobj_t *thing) // killough 3/26/98: make static
   if (tmthing->flags & MF_MISSILE || (tmthing->flags & MF_BOUNCES &&
 				      !(tmthing->flags & MF_SOLID)))
     {
-      // [crispy] mobj or actual sprite height
-      const fixed_t thingheight = (direct_vertical_aiming && tmthing->target && tmthing->target->player) ?
-        thing->info->actualheight : thing->height;
-
       // see if it went over / under
 
-      if (tmthing->z > thing->z + thingheight)
+      if (tmthing->z > thing->z + thingheight(thing, tmthing->target))
 	return true;    // overhead
 
       if (tmthing->z+tmthing->height < thing->z)
@@ -1487,7 +1501,7 @@ static boolean PTR_AimTraverse (intercept_t *in)
   // check angles to see if the thing can be aimed at
 
   dist = FixedMul(attackrange, in->frac);
-  thingtopslope = FixedDiv(th->z+th->height - shootz , dist);
+  thingtopslope = FixedDiv(th->z+thingheight(th, shootthing) - shootz , dist);
 
   if (thingtopslope < bottomslope)
     return true;    // shot over the thing
@@ -1615,12 +1629,7 @@ static boolean PTR_ShootTraverse(intercept_t *in)
   // check angles to see if the thing can be aimed at
 
   dist = FixedMul (attackrange, in->frac);
-  {
-  // [crispy] mobj or actual sprite height
-  const fixed_t thingheight = (direct_vertical_aiming && shootthing->player) ?
-    th->info->actualheight : th->height;
-  thingtopslope = FixedDiv (th->z+thingheight - shootz , dist);
-  }
+  thingtopslope = FixedDiv (th->z+thingheight(th, shootthing) - shootz , dist);
 
   if (thingtopslope < aimslope)
     return true;  // shot over the thing
@@ -1674,21 +1683,22 @@ fixed_t P_AimLineAttack(mobj_t *t1,angle_t angle,fixed_t distance,int mask)
 
   // can't shoot outside view angles
 
-  if (t1->player && direct_vertical_aiming)
+  if (t1->player && direct_vertical_aiming && (mask & CROSSHAIR_AIM))
   {
-    bottomslope = (topslope = t1->player->slope + 1) - 2;
+    topslope = t1->player->slope + 1;
+    bottomslope = t1->player->slope - 1;
   }
   else
   {
-  topslope = 100*FRACUNIT/160;
-  bottomslope = -100*FRACUNIT/160;
+    topslope = 100*FRACUNIT/160;
+    bottomslope = -100*FRACUNIT/160;
   }
 
   attackrange = distance;
   linetarget = NULL;
 
   // killough 8/2/98: prevent friends from aiming at friends
-  aim_flags_mask = mask;
+  aim_flags_mask = mask & MF_FRIEND;
 
   P_PathTraverse(t1->x,t1->y,x2,y2,PT_ADDLINES|PT_ADDTHINGS,PTR_AimTraverse);
 
