@@ -20,8 +20,14 @@
 #ifndef __P_MAP__
 #define __P_MAP__
 
-#include "r_defs.h"
-#include "d_player.h"
+#include "doomtype.h"
+#include "m_fixed.h"
+#include "tables.h"
+
+struct mobj_s;
+struct msecnode_s;
+struct player_s;
+struct sector_s;
 
 #define USERANGE        (64*FRACUNIT)
 #define MELEERANGE      (64*FRACUNIT)
@@ -35,32 +41,35 @@
 // is larger, but we do not have any moving sectors nearby
 #define MAXRADIUS       (32*FRACUNIT)
 
+#define CROSSHAIR_AIM 1
+
 // killough 3/15/98: add fourth argument to P_TryMove
-boolean P_TryMove(mobj_t *thing, fixed_t x, fixed_t y, boolean dropoff);
+boolean P_TryMove(struct mobj_s *thing, fixed_t x, fixed_t y, boolean dropoff);
 
 // killough 8/9/98: extra argument for telefragging
-boolean P_TeleportMove(mobj_t *thing, fixed_t x, fixed_t y,boolean boss);
-void    P_SlideMove(mobj_t *mo);
-boolean P_CheckSight(mobj_t *t1, mobj_t *t2);
-boolean P_CheckFov(mobj_t *t1, mobj_t *t2, angle_t fov);
-void    P_UseLines(player_t *player);
+boolean P_TeleportMove(struct mobj_s *thing, fixed_t x, fixed_t y, boolean boss);
+void    P_SlideMove(struct mobj_s *mo);
+boolean P_CheckSight(struct mobj_s *t1, struct mobj_s *t2);
+boolean P_CheckFov(struct mobj_s *t1, struct mobj_s *t2, angle_t fov);
+void    P_UseLines(struct player_s *player);
 
 // killough 8/2/98: add 'mask' argument to prevent friends autoaiming at others
-fixed_t P_AimLineAttack(mobj_t *t1, angle_t angle, fixed_t distance,int mask);
-void    P_LineAttack(mobj_t *t1, angle_t angle, fixed_t distance,
+fixed_t P_AimLineAttack(struct mobj_s *t1, angle_t angle, fixed_t distance, int mask);
+void    P_LineAttack(struct mobj_s *t1, angle_t angle, fixed_t distance,
                      fixed_t slope, int damage );
-void    P_RadiusAttack(mobj_t *spot, mobj_t *source, int damage, int distance);
-boolean P_CheckPosition(mobj_t *thing, fixed_t x, fixed_t y);
+void    P_RadiusAttack(struct mobj_s *spot, struct mobj_s *source,
+                       int damage, int distance);
+boolean P_CheckPosition(struct mobj_s *thing, fixed_t x, fixed_t y);
 
 //jff 3/19/98 P_CheckSector(): new routine to replace P_ChangeSector()
-boolean P_CheckSector(sector_t *sector, boolean crunch);
-void    P_DelSeclist(msecnode_t*);                          // phares 3/16/98
-void    P_CreateSecNodeList(mobj_t*,fixed_t,fixed_t);       // phares 3/14/98
-boolean Check_Sides(mobj_t *, int, int);                    // phares
+boolean P_CheckSector(struct sector_s *sector, boolean crunch);
+void    P_DelSeclist(struct msecnode_s *);                          // phares 3/16/98
+void    P_CreateSecNodeList(struct mobj_s *, fixed_t, fixed_t);       // phares 3/14/98
+boolean Check_Sides(struct mobj_s *, int, int);                    // phares
 
-int     P_GetMoveFactor(const mobj_t *mo, int *friction);   // killough 8/28/98
-int     P_GetFriction(const mobj_t *mo, int *factor);       // killough 8/28/98
-void    P_ApplyTorque(mobj_t *mo);                          // killough 9/12/98
+int     P_GetMoveFactor(const struct mobj_s *mo, int *friction);   // killough 8/28/98
+int     P_GetFriction(const struct mobj_s *mo, int *factor);       // killough 8/28/98
+void    P_ApplyTorque(struct mobj_s *mo);                          // killough 9/12/98
 
 void    P_MapStart(void);
 void    P_MapEnd(void);
@@ -70,12 +79,12 @@ extern boolean floatok;
 extern boolean felldown;   // killough 11/98: indicates object pushed off ledge
 extern fixed_t tmfloorz;
 extern fixed_t tmceilingz;
-extern line_t *ceilingline;
-extern line_t *floorline;      // killough 8/23/98
-extern mobj_t *linetarget;     // who got hit (or NULL)
-extern msecnode_t *sector_list;                             // phares 3/16/98
+extern struct line_s *ceilingline;
+extern struct line_s *floorline;      // killough 8/23/98
+extern struct mobj_s *linetarget;     // who got hit (or NULL)
+extern struct msecnode_s *sector_list;                             // phares 3/16/98
 extern fixed_t tmbbox[4];         // phares 3/20/98
-extern line_t *blockline;   // killough 8/11/98
+extern struct line_s *blockline;   // killough 8/11/98
 
 #endif // __P_MAP__
 
