@@ -17,27 +17,35 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <stdlib.h>
+#include <string.h>
+
+#include "d_player.h"
 #include "doomdef.h"
 #include "doomstat.h"
+#include "g_game.h"
+#include "hu_stuff.h"
 #include "i_printf.h"
+#include "info.h"
 #include "m_random.h"
-#include "r_main.h"
-#include "r_things.h"
-#include "p_maputl.h"
+#include "p_inter.h"
 #include "p_map.h"
-#include "p_tick.h"
+#include "p_maputl.h"
+#include "p_mobj.h"
+#include "p_pspr.h"
 #include "p_spec.h"
+#include "p_tick.h"
+#include "r_defs.h"
+#include "r_main.h"
+#include "r_state.h"
+#include "r_things.h"
+#include "s_musinfo.h" // [crispy] S_ParseMusInfo()
+#include "s_sound.h"
 #include "sounds.h"
 #include "st_stuff.h"
-#include "hu_stuff.h"
-#include "s_sound.h"
-#include "s_musinfo.h" // [crispy] S_ParseMusInfo()
-#include "info.h"
-#include "g_game.h"
-#include "p_inter.h"
+#include "tables.h"
 #include "v_video.h"
-#include "m_swap.h"
-#include "w_wad.h"
+#include "z_zone.h"
 
 // [FG] colored blood and gibs
 boolean colored_blood;
@@ -108,6 +116,12 @@ boolean P_SetMobjState(mobj_t* mobj,statenum_t state)
   if (tempstate)
     Z_Free(tempstate);
 
+  // [FG] update object's actual height
+  if (ret)
+  {
+    mobj->actualheight = spritetopoffset[sprites[mobj->sprite].spriteframes[mobj->frame & FF_FRAMEMASK].lump[0]];
+  }
+
   return ret;
 }
 
@@ -673,10 +687,6 @@ void P_NightmareRespawn(mobj_t* mobj)
   // killough 11/98: transfer friendliness from deceased
   mo->flags = (mo->flags & ~MF_FRIEND) | (mobj->flags & MF_FRIEND);
 
-  // [crispy] count respawned monsters
-  if (!(mo->flags & MF_FRIEND))
-    extrakills++;
-
   mo->reactiontime = 18;
 
   // remove the old monster,
@@ -883,31 +893,16 @@ mobj_t *P_SpawnMobj(fixed_t x, fixed_t y, fixed_t z, mobjtype_t type)
       mobj->intflags &= ~MIF_FLIP;
   }
 
-  // [crispy] height of the spawnstate's first sprite in pixels
-  if (!info->actualheight)
-  {
-    const spritedef_t *const sprdef = &sprites[mobj->sprite];
-
-    if (!sprdef->numframes || !(mobj->flags & (MF_SOLID|MF_SHOOTABLE)))
-    {
-      info->actualheight = info->height;
-    }
-    else
-    {
-      spriteframe_t *sprframe;
-      int lump;
-      patch_t *patch;
-
-      sprframe = &sprdef->spriteframes[mobj->frame & FF_FRAMEMASK];
-      lump = sprframe->lump[0];
-      patch = W_CacheLumpNum(lump + firstspritelump, PU_CACHE);
-
-      info->actualheight = SHORT(patch->height) << FRACBITS;
-    }
-  }
+  // [FG] initialize object's actual height
+  mobj->actualheight = spritetopoffset[sprites[st->sprite].spriteframes[st->frame & FF_FRAMEMASK].lump[0]];
 
   P_AddThinker(&mobj->thinker);
 
+  if (!((mobj->flags ^ MF_COUNTKILL) & (MF_FRIEND | MF_COUNTKILL)))
+  {
+    ++max_kill_requirement;
+  }
+
   return mobj;
 }
 
@@ -1321,6 +1316,37 @@ spawnit:
   {
       mobj->health = 1000 + musid;
   }
+
+  // [crispy] blinking key or skull in the status bar
+  switch (mobj->sprite)
+  {
+    case SPR_BKEY:
+      st_keyorskull[it_bluecard] |= KEYBLINK_CARD;
+      break;
+
+    case SPR_BSKU:
+      st_keyorskull[it_bluecard] |= KEYBLINK_SKULL;
+      break;
+
+    case SPR_RKEY:
+      st_keyorskull[it_redcard] |= KEYBLINK_CARD;
+      break;
+
+    case SPR_RSKU:
+      st_keyorskull[it_redcard] |= KEYBLINK_SKULL;
+      break;
+
+    case SPR_YKEY:
+      st_keyorskull[it_yellowcard] |= KEYBLINK_CARD;
+      break;
+
+    case SPR_YSKU:
+      st_keyorskull[it_yellowcard] |= KEYBLINK_SKULL;
+      break;
+
+    default:
+      break;
+  }
 }
 
 //
