@@ -20,9 +20,11 @@
 #ifndef __P_MOBJ__
 #define __P_MOBJ__
 
+#include <limits.h>
+
 // Basics.
-#include "tables.h"
 #include "m_fixed.h"
+#include "tables.h"
 
 // We need the thinker_t stuff.
 #include "d_think.h"
@@ -233,6 +235,7 @@ enum {
   MIF_SCROLLING = 8,    // Object is affected by scroller / pusher / puller
   // cosmetic
   MIF_FLIP = 16,
+  MIF_SPAWNED_BY_ICON = 32,
 };
 
 // Map Object definition.
@@ -379,6 +382,9 @@ typedef struct mobj_s
 
     // [FG] colored blood and gibs
     int bloodcolor;
+
+    // [FG] height of the sprite in pixels
+    int actualheight;
 } mobj_t;
 
 // External declarations (fomerly in p_local.h) -- killough 5/2/98
@@ -389,8 +395,8 @@ typedef struct mobj_s
 #define GRAVITY         FRACUNIT
 #define MAXMOVE         (30*FRACUNIT)
 
-#define ONFLOORZ        D_MININT
-#define ONCEILINGZ      D_MAXINT
+#define ONFLOORZ        INT_MIN
+#define ONCEILINGZ      INT_MAX
 
 // Time interval for item respawning.
 #define ITEMQUESIZE     128
@@ -416,13 +422,6 @@ extern int iquetail;
 // [FG] colored blood and gibs
 extern boolean colored_blood;
 
-// The value `160` is used for the denominator because in Vanilla Doom
-// the horizontal FOV is 90° and spans 320 px, so the height of the
-// resulting triangle is 160.
-// Heretic/Hexen used value `173` presumably because Raven assumed an
-// equilateral triangle (i.e. 60°) for the vertical FOV which spans
-// 200 px, so the height of that triangle would be 100*sqrt(3) = 173.
-#define PLAYER_SLOPE(a)	((((a)->lookdir / MLOOKUNIT) << FRACBITS) / 160)
 extern boolean direct_vertical_aiming, default_direct_vertical_aiming;
 void P_UpdateDirectVerticalAiming(void);
 
