@@ -18,20 +18,24 @@
 //
 //-----------------------------------------------------------------------------
 
+#include "d_event.h"
+#include "d_items.h"
+#include "d_player.h"
 #include "doomstat.h"
-#include "r_main.h"
-#include "p_map.h"
-#include "p_inter.h"
-#include "p_pspr.h"
-#include "p_enemy.h"
+#include "i_printf.h"
+#include "i_video.h" // uncapped
 #include "m_random.h"
+#include "p_action.h"
+#include "p_enemy.h"
+#include "p_inter.h"
+#include "p_map.h"
+#include "p_mobj.h"
+#include "p_pspr.h"
+#include "p_tick.h"
+#include "r_main.h"
 #include "s_sound.h"
 #include "sounds.h"
-#include "d_event.h"
-#include "p_tick.h"
-#include "i_video.h" // uncapped
-
-#include "p_action.h"
+#include "tables.h"
 
 #define LOWERSPEED   (FRACUNIT*6)
 #define RAISESPEED   (FRACUNIT*6)
@@ -49,14 +53,14 @@ static struct
   int pitch;
 } recoil_values[] = {    // phares
   { 10, 0 },   // wp_fist
-  { 10, 4 },   // wp_pistol
-  { 30, 8 },   // wp_shotgun
-  { 10, 4 },   // wp_chaingun
-  { 100, 16 }, // wp_missile
-  { 20, 4 },   // wp_plasma
-  { 100, 20 }, // wp_bfg
-  { 0, -2 },   // wp_chainsaw
-  { 80, 16 }   // wp_supershotgun
+  { 10, 2 },   // wp_pistol
+  { 30, 4 },   // wp_shotgun
+  { 10, 2 },   // wp_chaingun
+  { 100, 7 }, // wp_missile
+  { 20, 2 },   // wp_plasma
+  { 100, 9 }, // wp_bfg
+  { 0, -1 },   // wp_chainsaw
+  { 80, 7 }   // wp_supershotgun
 };
 
 // [crispy] add weapon recoil pitch
@@ -66,7 +70,7 @@ void A_Recoil(player_t* player)
 {
     if (player && weapon_recoilpitch)
     {
-        player->recoilpitch = recoil_values[player->readyweapon].pitch;
+        player->recoilpitch = recoil_values[player->readyweapon].pitch * ANG1;
     }
 }
 
@@ -154,6 +158,12 @@ static void P_BringUpWeapon(player_t *player)
   if (player->pendingweapon == wp_chainsaw)
     S_StartSound(player->mo, sfx_sawup);
 
+  if (player->pendingweapon >= NUMWEAPONS)
+  {
+    player->pendingweapon = NUMWEAPONS;
+    I_Printf(VB_WARNING, "P_BringUpWeapon: weaponinfo overrun has occurred.");
+  }
+
   newstate = weaponinfo[player->pendingweapon].upstate;
 
   player->pendingweapon = wp_nochange;
@@ -773,7 +783,7 @@ void A_FireOldBFG(player_t *player, pspdef_t *psp)
 
   if (weapon_recoilpitch && (leveltime & 2))
   {
-    player->recoilpitch = recoil_values[wp_plasma].pitch;
+    player->recoilpitch = recoil_values[wp_plasma].pitch * ANG1;
   }
 
   P_SubtractAmmo(player, 1);
@@ -1079,8 +1089,6 @@ void P_SetupPsprites(player_t *player)
 // Called every tic by player thinking routine.
 //
 
-#define BOBBING_75 2
-
 #define WEAPON_CENTERED 1
 #define WEAPON_BOBBING 2
 
@@ -1108,7 +1116,7 @@ void P_MovePsprites(player_t *player)
 
   if (psp->state)
   {
-    if (!cosmetic_bobbing)
+    if (!weapon_bobbing_pct)
     {
       static fixed_t last_sy = WEAPONTOP;
 
@@ -1125,7 +1133,7 @@ void P_MovePsprites(player_t *player)
         psp->sy2 -= (last_sy - WEAPONTOP);
       }
     }
-    else if (cosmetic_bobbing == BOBBING_75 || center_weapon_strict || uncapped)
+    else if (center_weapon_strict || uncapped)
     {
       // [FG] don't center during lowering and raising states
       if (psp->state->misc1 || player->switching)
@@ -1134,7 +1142,8 @@ void P_MovePsprites(player_t *player)
       // [FG] not attacking means idle
       else if (!player->attackdown || center_weapon_strict == WEAPON_BOBBING)
       {
-        P_ApplyBobbing(&psp->sx2, &psp->sy2, player->bob2);
+        fixed_t bob = player->bob * weapon_bobbing_pct / 4;
+        P_ApplyBobbing(&psp->sx2, &psp->sy2, bob);
       }
       // [FG] center the weapon sprite horizontally and push up vertically
       else if (center_weapon_strict == WEAPON_CENTERED)
