@@ -55,7 +55,8 @@ typedef enum saveg_compat_e
 {
   saveg_mbf,
   saveg_woof510,
-  saveg_current,
+  saveg_woof600,
+  saveg_current, // saveg_woof1300
 } saveg_compat_t;
 
 extern saveg_compat_t saveg_compat;
