@@ -17,15 +17,27 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <stdlib.h>
+
+#include "d_player.h"
+#include "d_think.h"
+#include "doomdef.h"
 #include "doomstat.h"
-#include "p_spec.h"
-#include "p_maputl.h"
+#include "doomtype.h"
+#include "info.h"
+#include "m_fixed.h"
 #include "p_map.h"
-#include "r_main.h"
+#include "p_maputl.h"
+#include "p_mobj.h"
+#include "p_spec.h"
 #include "p_tick.h"
+#include "p_user.h"
+#include "r_defs.h"
+#include "r_main.h"
+#include "r_state.h"
 #include "s_sound.h"
 #include "sounds.h"
-#include "p_user.h"
+#include "tables.h"
 
 //
 // TELEPORTATION
