@@ -19,13 +19,41 @@
 //
 //-----------------------------------------------------------------------------
 
-#include "doomstat.h"
+#include <stdlib.h>
+
 #include "d_event.h"
-#include "r_main.h"
+#include "d_player.h"
+#include "d_ticcmd.h"
+#include "doomdef.h"
+#include "doomstat.h"
+#include "doomtype.h"
+#include "g_game.h"
+#include "hu_stuff.h"
+#include "info.h"
+#include "m_input.h"
 #include "p_map.h"
+#include "p_mobj.h"
+#include "p_pspr.h"
 #include "p_spec.h"
 #include "p_user.h"
-#include "g_game.h"
+#include "r_defs.h"
+#include "r_main.h"
+#include "r_state.h"
+
+static fixed_t PlayerSlope(player_t *player)
+{
+  const fixed_t pitch = player->pitch;
+
+  if (pitch)
+  {
+    const fixed_t slope = -finetangent[(ANG90 - pitch) >> ANGLETOFINESHIFT];
+    return (fixed_t)((int64_t)slope * SCREENHEIGHT / ACTUALHEIGHT);
+  }
+  else
+  {
+    return 0;
+  }
+}
 
 // Index of the special effects (INVUL inverse) map.
 
@@ -77,9 +105,6 @@ void P_Bob(player_t *player, angle_t angle, fixed_t move)
 // Calculate the walking / running height adjustment
 //
 
-// [crispy] variable player view bob
-static const int bobfactors[3] = {0, 4, 3};
-
 void P_CalcHeight (player_t* player)
 {
   int     angle;
@@ -119,9 +144,6 @@ void P_CalcHeight (player_t* player)
     player->bob = MAXBOB;
   }
 
-  // [crispy] variable player view bob
-  player->bob2 = bobfactors[cosmetic_bobbing] * player->bob / 4;
-
   if (!onground || player->cheats & CF_NOMOMENTUM)
     {
       player->viewz = player->mo->z + VIEWHEIGHT;
@@ -140,7 +162,9 @@ void P_CalcHeight (player_t* player)
     }
 
   angle = (FINEANGLES/20*leveltime)&FINEMASK;
-  bob = FixedMul(player->bob2/2,finesine[angle]);
+
+  bob = player->bob * view_bobbing_pct / 4;
+  bob = FixedMul(bob/2,finesine[angle]);
 
   // move viewheight
 
@@ -190,6 +214,12 @@ void P_MovePlayer (player_t* player)
   mo->angle += cmd->angleturn << 16;
   onground = mo->z <= mo->floorz;
 
+  if (player == &players[consoleplayer])
+  {
+    localview.ticangle += localview.ticangleturn << 16;
+    localview.ticangleturn = 0;
+  }
+
   // killough 10/98:
   //
   // We must apply thrust to the player and bobbing separately, to avoid
@@ -232,10 +262,9 @@ void P_MovePlayer (player_t* player)
 
   if (!menuactive && !demoplayback)
   {
-    player->lookdir = BETWEEN(-LOOKDIRMAX * MLOOKUNIT,
-                               LOOKDIRMAX * MLOOKUNIT,
-                               player->lookdir + cmd->lookdir);
-    player->slope = PLAYER_SLOPE(player);
+    player->pitch += cmd->pitch;
+    player->pitch = BETWEEN(-MAX_PITCH_ANGLE, MAX_PITCH_ANGLE, player->pitch);
+    player->slope = PlayerSlope(player);
   }
 }
 
@@ -319,6 +348,8 @@ void P_DeathThink (player_t* player)
           char *file = G_SaveGameName(savegameslot);
           G_LoadGame(file, savegameslot, false);
           free(file);
+          // [Woof!] prevent on-death-action reloads from activating specials
+          M_InputGameDeactivate(input_use);
         }
         else
           player->playerstate = PST_REBORN;
@@ -350,9 +381,14 @@ void P_PlayerThink (player_t* player)
   player->mo->oldz = player->mo->z;
   player->mo->oldangle = player->mo->angle;
   player->oldviewz = player->viewz;
-  player->oldlookdir = player->lookdir;
+  player->oldpitch = player->pitch;
   player->oldrecoilpitch = player->recoilpitch;
 
+  if (player == &players[consoleplayer])
+  {
+    localview.oldticangle = localview.ticangle;
+  }
+
   // killough 2/8/98, 3/21/98:
   // (this code is necessary despite questions raised elsewhere in a comment)
 
@@ -373,21 +409,29 @@ void P_PlayerThink (player_t* player)
     }
 
   // [crispy] center view
+  #define CENTERING_VIEW_ANGLE (4 * ANG1)
+
   if (player->centering)
   {
-    if (player->lookdir > 0)
+    if (player->pitch > 0)
     {
-      player->lookdir -= 8 * MLOOKUNIT;
+      player->pitch -= CENTERING_VIEW_ANGLE;
     }
-    else if (player->lookdir < 0)
+    else if (player->pitch < 0)
     {
-      player->lookdir += 8 * MLOOKUNIT;
+      player->pitch += CENTERING_VIEW_ANGLE;
     }
-    if (abs(player->lookdir) < 8 * MLOOKUNIT)
+    if (abs(player->pitch) < CENTERING_VIEW_ANGLE)
     {
-      player->lookdir = 0;
-      player->centering = false;
+      player->pitch = 0;
+
+      if (player->oldpitch == 0)
+      {
+        player->centering = false;
+      }
     }
+
+    player->slope = PlayerSlope(player);
   }
 
   // [crispy] weapon recoil pitch
@@ -395,17 +439,16 @@ void P_PlayerThink (player_t* player)
   {
     if (player->recoilpitch > 0)
     {
-      player->recoilpitch -= 1;
+      player->recoilpitch -= ANG1;
     }
     else if (player->recoilpitch < 0)
     {
-      player->recoilpitch += 1;
+      player->recoilpitch += ANG1;
     }
   }
 
   if (player->playerstate == PST_DEAD)
     {
-      player->slope = PLAYER_SLOPE(player); // For 3D audio pitch angle.
       P_DeathThink (player);
       return;
     }
@@ -493,6 +536,14 @@ void P_PlayerThink (player_t* player)
 	{
 	  P_UseLines (player);
 	  player->usedown = true;
+
+	  // [crispy] "use" button timer
+	  if (hud_time_use)
+	  {
+	    // to match the timer, we use the leveltime value at the end of the frame
+	    player->btuse = leveltime + 1;
+	    player->btuse_tics = 5*TICRATE/2; // [crispy] 2.5 seconds
+	  }
 	}
     }
   else
@@ -524,12 +575,6 @@ void P_PlayerThink (player_t* player)
   if (player->powers[pw_ironfeet] > 0)        // killough
     player->powers[pw_ironfeet]--;
 
-  if (player->cheats & CF_RENDERSTATS)
-  {
-    extern void R_ShowRenderingStats();
-    R_ShowRenderingStats();
-  }
-
   if (player->cheats & CF_MAPCOORDS)
   {
     extern void cheat_mypos_print();
