@@ -19,17 +19,20 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <string.h>
+
+#include "doomdef.h"
 #include "doomstat.h"
-#include "w_wad.h"
-#include "r_draw.h" // [FG]
+#include "i_system.h"
+#include "i_video.h"
+#include "r_bsp.h"
+#include "r_defs.h"
+#include "r_draw.h"
 #include "r_main.h"
+#include "r_state.h"
 #include "v_video.h"
-#include "m_menu.h"
-
-#define MAXWIDTH  MAX_SCREENWIDTH          /* kilough 2/8/98 */
-#define MAXHEIGHT MAX_SCREENHEIGHT
-
-#define SBARHEIGHT 32             /* status bar height at bottom of screen */
+#include "w_wad.h"
+#include "z_zone.h"
 
 //
 // All drawing to the view buffer is accomplished in this file.
@@ -44,12 +47,14 @@ byte *viewimage;
 int  viewwidth;
 int  scaledviewwidth;
 int  scaledviewheight;        // killough 11/98
+int  scaledviewx;
+int  scaledviewy;
 int  viewheight;
 int  viewwindowx;
 int  viewwindowy; 
-byte *ylookup[MAXHEIGHT]; 
-int  columnofs[MAXWIDTH]; 
-int  linesize = ORIGWIDTH;  // killough 11/98
+static byte **ylookup = NULL;
+static int  *columnofs = NULL;
+static int  linesize;  // killough 11/98
 
 // Color tables for different players,
 //  translate a limited part to another
@@ -61,6 +66,11 @@ byte translations[3][256];
 byte *tranmap;          // translucency filter maps 256x256   // phares 
 byte *main_tranmap;     // killough 4/11/98
 
+// Backing buffer containing the bezel drawn around the screen and surrounding
+// background.
+
+static pixel_t *background_buffer = NULL;
+
 //
 // R_DrawColumn
 // Source is the top of the column to scale.
@@ -97,9 +107,9 @@ void R_DrawColumn (void)
     return; 
                                  
 #ifdef RANGECHECK 
-  if ((unsigned)dc_x >= MAX_SCREENWIDTH
+  if ((unsigned)dc_x >= video.width
       || dc_yl < 0
-      || dc_yh >= MAX_SCREENHEIGHT) 
+      || dc_yh >= video.height) 
     I_Error ("R_DrawColumn: %i to %i at %i", dc_yl, dc_yh, dc_x); 
 #endif 
 
@@ -200,9 +210,9 @@ void R_DrawTLColumn (void)
     return; 
                                  
 #ifdef RANGECHECK 
-  if ((unsigned)dc_x >= MAX_SCREENWIDTH
+  if ((unsigned)dc_x >= video.width
       || dc_yl < 0
-      || dc_yh >= MAX_SCREENHEIGHT) 
+      || dc_yh >= video.height) 
     I_Error ("R_DrawColumn: %i to %i at %i", dc_yl, dc_yh, dc_x); 
 #endif 
 
@@ -226,8 +236,9 @@ void R_DrawTLColumn (void)
   
   {
     register const byte *source = dc_source;            
-    register const lighttable_t *colormap = dc_colormap[0];
+    register lighttable_t *const *colormap = dc_colormap;
     register int heightmask = dc_texheight-1;
+    register const byte *brightmap = dc_brightmap;
     if (dc_texheight & heightmask)   // not a power of 2 -- killough
       {
         heightmask++;
@@ -246,7 +257,9 @@ void R_DrawTLColumn (void)
             
             // heightmask is the Tutti-Frutti fix -- killough
               
-            *dest = tranmap[(*dest<<8)+colormap[source[frac>>FRACBITS]]]; // phares
+            // [crispy] brightmaps
+            byte src = source[frac>>FRACBITS];
+            *dest = tranmap[(*dest<<8)+colormap[brightmap[src]][src]]; // phares
             dest += linesize;          // killough 11/98
             if ((frac += fracstep) >= heightmask)
               frac -= heightmask;
@@ -257,15 +270,20 @@ void R_DrawTLColumn (void)
       {
         while ((count-=2)>=0)   // texture height is a power of 2 -- killough
           {
-            *dest = tranmap[(*dest<<8)+colormap[source[(frac>>FRACBITS) & heightmask]]]; // phares
+            byte src = source[(frac>>FRACBITS) & heightmask];
+            *dest = tranmap[(*dest<<8)+colormap[brightmap[src]][src]]; // phares
             dest += linesize;   // killough 11/98
             frac += fracstep;
-            *dest = tranmap[(*dest<<8)+colormap[source[(frac>>FRACBITS) & heightmask]]]; // phares
+            src = source[(frac>>FRACBITS) & heightmask];
+            *dest = tranmap[(*dest<<8)+colormap[brightmap[src]][src]]; // phares
             dest += linesize;   // killough 11/98
             frac += fracstep;
           }
         if (count & 1)
-          *dest = tranmap[(*dest<<8)+colormap[source[(frac>>FRACBITS) & heightmask]]]; // phares
+        {
+          byte src = source[(frac>>FRACBITS) & heightmask];
+          *dest = tranmap[(*dest<<8)+colormap[brightmap[src]][src]]; // phares
+        }
       }
   }
 } 
@@ -287,9 +305,9 @@ void R_DrawSkyColumn(void)
     return;
 
 #ifdef RANGECHECK
-  if ((unsigned)dc_x >= MAX_SCREENWIDTH
+  if ((unsigned)dc_x >= video.width
     || dc_yl < 0
-    || dc_yh >= MAX_SCREENHEIGHT)
+    || dc_yh >= video.height)
     I_Error ("R_DrawSkyColumn: %i to %i at %i", dc_yl, dc_yh, dc_x);
 #endif
 
@@ -310,7 +328,7 @@ void R_DrawSkyColumn(void)
 
     if (frac < -2 * FRACUNIT)
       {
-        n = (-frac - 2 * FRACUNIT) / fracstep;
+        n = (-frac - 2 * FRACUNIT + fracstep - 1) / fracstep;
         if (n > count)
           n = count;
 
@@ -327,7 +345,7 @@ void R_DrawSkyColumn(void)
 
     if (frac < -FRACUNIT)
       {
-        n = (-frac - FRACUNIT) / fracstep;
+        n = (-frac - FRACUNIT + fracstep - 1) / fracstep;
         if (n > count)
           n = count;
 
@@ -349,7 +367,7 @@ void R_DrawSkyColumn(void)
     // Now it's on the edge
     if (frac < 0)
       {
-        n = (-frac) / fracstep;
+        n = (-frac + fracstep - 1) / fracstep;
         if (n > count)
           n = count;
 
@@ -468,9 +486,9 @@ static void R_DrawFuzzColumn_orig(void)
     return; 
     
 #ifdef RANGECHECK 
-  if ((unsigned) dc_x >= MAX_SCREENWIDTH
+  if ((unsigned) dc_x >= video.width
       || dc_yl < 0 
-      || dc_yh >= MAX_SCREENHEIGHT)
+      || dc_yh >= video.height)
     I_Error ("R_DrawFuzzColumn: %i to %i at %i",
              dc_yl, dc_yh, dc_x);
 #endif
@@ -498,7 +516,7 @@ static void R_DrawFuzzColumn_orig(void)
       // fraggle 1/8/2000: fix with the bugfix from lees
       // why_i_left_doom.html
 
-      *dest = fullcolormap[6*256+dest[fuzzoffset[fuzzpos++] ? linesize : -linesize]];
+      *dest = fullcolormap[6*256+dest[fuzzoffset[fuzzpos++] ? -linesize : linesize]];
       dest += linesize;             // killough 11/98
 
       // Clamp table lookup index.
@@ -514,31 +532,29 @@ static void R_DrawFuzzColumn_orig(void)
   }
 }
 
-// [FG] "blocky" spectre drawing for hires mode:
-//      draw only even columns, in each column
-//      draw only even pixels as 2x2 squares
-//      using the same fuzzoffset value
+// [FG] "blocky" spectre drawing for hires mode
 
 static void R_DrawFuzzColumn_block(void)
 {
   int count;
   byte *dest;
   boolean cutoff = false;
+  const int nx = video.xscale >> FRACBITS;
+  const int ny = video.yscale >> FRACBITS;
 
-  // [FG] draw only even columns
-  if (dc_x & 1)
+  if (dc_x % nx)
     return;
 
-  // [FG] draw only even pixels
-  dc_yl &= (int)~1;
-  dc_yh &= (int)~1;
+  dc_yl += ny;
+  dc_yl -= dc_yl % ny;
+  dc_yh -= dc_yh % ny;
 
   if (!dc_yl)
-    dc_yl = 2;
+    dc_yl = ny;
 
-  if (dc_yh == viewheight-2)
+  if (dc_yh >= viewheight - ny)
   {
-    dc_yh = viewheight - 4;
+    dc_yh = viewheight - 2 * ny;
     cutoff = true;
   }
 
@@ -548,46 +564,47 @@ static void R_DrawFuzzColumn_block(void)
     return;
 
 #ifdef RANGECHECK
-  if ((unsigned) dc_x >= MAX_SCREENWIDTH
+  if ((unsigned) dc_x >= video.width
       || dc_yl < 0
-      || dc_yh >= MAX_SCREENHEIGHT)
+      || dc_yh >= video.height)
     I_Error ("R_DrawFuzzColumn: %i to %i at %i",
              dc_yl, dc_yh, dc_x);
 #endif
 
   dest = ylookup[dc_yl] + columnofs[dc_x];
 
-  count+=2;
+  count += ny;
 
   do
     {
-      // [FG] draw only even pixels as 2x2 squares
+      // [FG] draw only even pixels as (nx * ny) squares
       //      using the same fuzzoffset value
-      const byte fuzz = fullcolormap[6*256+dest[fuzzoffset[fuzzpos] ? 2*linesize : -2*linesize]];
+      const int offset = fuzzoffset[fuzzpos] ? -ny * linesize : ny * linesize;
+      const byte fuzz = fullcolormap[6 * 256 + dest[offset]];
+      int i;
 
-      dest[0] = fuzz;
-      dest[1] = fuzz;
-      dest += linesize;
-
-      dest[0] = fuzz;
-      dest[1] = fuzz;
-      dest += linesize;
+      for (i = 0; i < ny && count - i > 0; i++)
+      {
+        memset(dest, fuzz, nx);
+        dest += linesize;
+      }
 
       fuzzpos++;
-      fuzzpos &= (fuzzpos - FUZZTABLE) >> (8*sizeof fuzzpos-1);
+      fuzzpos &= (fuzzpos - FUZZTABLE) >> (8 * sizeof(fuzzpos) - 1);
     }
-  while (count -= 2);
+  while ((count -= ny) > 0);
 
   if (cutoff)
     {
-      const byte fuzz = fullcolormap[6*256+dest[2*linesize*fuzzoffset[fuzzpos]]];
+      const int offset = ny * linesize * fuzzoffset[fuzzpos];
+      const byte fuzz = fullcolormap[6 * 256 + dest[offset]];
+      int i;
 
-      dest[0] = fuzz;
-      dest[1] = fuzz;
-      dest += linesize;
-
-      dest[0] = fuzz;
-      dest[1] = fuzz;
+      for (i = 0; i < ny; i++)
+      {
+        memset(dest, fuzz, nx);
+        dest += linesize;
+      }
     }
 }
 
@@ -597,7 +614,7 @@ int fuzzcolumn_mode;
 void (*R_DrawFuzzColumn) (void) = R_DrawFuzzColumn_orig;
 void R_SetFuzzColumnMode (void)
 {
-  if (fuzzcolumn_mode && hires)
+  if (fuzzcolumn_mode && current_video_height > SCREENHEIGHT)
     R_DrawFuzzColumn = R_DrawFuzzColumn_block;
   else
     R_DrawFuzzColumn = R_DrawFuzzColumn_orig;
@@ -627,9 +644,9 @@ void R_DrawTranslatedColumn (void)
     return; 
                                  
 #ifdef RANGECHECK 
-  if ((unsigned)dc_x >= MAX_SCREENWIDTH
+  if ((unsigned)dc_x >= video.width
       || dc_yl < 0
-      || dc_yh >= MAX_SCREENHEIGHT)
+      || dc_yh >= video.height)
     I_Error ( "R_DrawColumn: %i to %i at %i",
               dc_yl, dc_yh, dc_x);
 #endif 
@@ -652,7 +669,9 @@ void R_DrawTranslatedColumn (void)
       // Thus the "green" ramp of the player 0 sprite
       //  is mapped to gray, red, black/indigo. 
       
-      *dest = dc_colormap[0][dc_translation[dc_source[frac>>FRACBITS]]];
+      // [crispy] brightmaps
+      byte src = dc_source[frac>>FRACBITS];
+      *dest = dc_colormap[dc_brightmap[src]][dc_translation[src]];
       dest += linesize;      // killough 11/98
         
       frac += fracstep; 
@@ -789,6 +808,18 @@ void R_DrawSpan (void)
     } 
 } 
 
+void R_InitBufferRes(void)
+{
+  if (solidcol) Z_Free(solidcol);
+  if (columnofs) Z_Free(columnofs);
+  if (ylookup) Z_Free(ylookup);
+
+  columnofs = Z_Malloc(video.width * sizeof(*columnofs), PU_STATIC, NULL);
+  ylookup = Z_Malloc(video.height * sizeof(*ylookup), PU_STATIC, NULL);
+
+  solidcol = Z_Calloc(1, video.width * sizeof(*solidcol), PU_STATIC, NULL);
+}
+
 //
 // R_InitBuffer 
 // Creats lookup tables that avoid
@@ -797,132 +828,106 @@ void R_DrawSpan (void)
 //  of a pixel to draw.
 //
 
-void R_InitBuffer(int width, int height)
-{ 
-  int i; 
+void R_InitBuffer(void)
+{
+  int i;
 
-  linesize = SCREENWIDTH << hires;    // killough 11/98
+  linesize = video.pitch;    // killough 11/98
 
   // Handle resize,
   //  e.g. smaller view windows
   //  with border and/or status bar.
 
-  viewwindowx = (SCREENWIDTH-width) >> !hires;  // killough 11/98
-
   // Column offset. For windows.
 
-  for (i = width << hires ; i--; )   // killough 11/98
+  for (i = viewwidth; i--; )   // killough 11/98
     columnofs[i] = viewwindowx + i;
-    
+
   // Same with base row offset.
 
-  viewwindowy = width==SCREENWIDTH ? 0 : (SCREENHEIGHT-SBARHEIGHT-height)>>1; 
-
-  viewwindowy <<= hires;   // killough 11/98
-
   // Preclaculate all row offsets.
 
-  for (i = height << hires; i--; )
-    ylookup[i] = screens[0] + (i+viewwindowy)*linesize; // killough 11/98
-} 
+  for (i = viewheight; i--; )
+    ylookup[i] = I_VideoBuffer + (i + viewwindowy) * linesize; // killough 11/98
 
-//
-// R_FillBackScreen
-// Fills the back screen with a pattern
-//  for variable screen sizes
-// Also draws a beveled edge.
-//
-
-void R_DrawBackground(char *patchname, byte *back_dest)
-{
-  int x, y;
-  byte *src = W_CacheLumpNum(firstflat + R_FlatNumForName(patchname), PU_CACHE);
-
-  if (hires)       // killough 11/98: hires support
+  if (background_buffer != NULL)
   {
-    for (y = 0; y < SCREENHEIGHT<<1; y++)
-      for (x = 0; x < SCREENWIDTH<<1; x += 2)
-      {
-        const byte dot = src[(((y>>1)&63)<<6) + ((x>>1)&63)];
-
-        *back_dest++ = dot;
-        *back_dest++ = dot;
-      }
-  }
-  else
-  {
-    for (y = 0; y < SCREENHEIGHT; y++)
-      for (x = 0; x < SCREENWIDTH; x++)
-      {
-        *back_dest++ = src[((y&63)<<6) + (x&63)];
-      }
+    Z_Free(background_buffer);
+    background_buffer = NULL;
   }
 }
 
-void R_DrawBorder (int x, int y, int w, int h, int s)
+void R_DrawBorder (int x, int y, int w, int h)
 {
   int i, j;
   patch_t *patch;
 
   patch = W_CacheLumpName("brdr_t", PU_CACHE);
   for (i = 0; i < w; i += 8)
-    V_DrawPatch(x + i - WIDESCREENDELTA, y - 8, s, patch);
+    V_DrawPatch(x + i - video.deltaw, y - 8, patch);
 
   patch = W_CacheLumpName("brdr_b", PU_CACHE);
   for (i = 0; i < w; i += 8)
-    V_DrawPatch(x + i - WIDESCREENDELTA, y + h, s, patch);
+    V_DrawPatch(x + i - video.deltaw, y + h, patch);
 
   patch = W_CacheLumpName("brdr_l", PU_CACHE);
   for (j = 0; j < h; j += 8)
-    V_DrawPatch(x - 8 - WIDESCREENDELTA, y + j, s, patch);
+    V_DrawPatch(x - 8 - video.deltaw, y + j, patch);
 
   patch = W_CacheLumpName("brdr_r", PU_CACHE);
   for (j = 0; j < h; j += 8)
-    V_DrawPatch(x + w - WIDESCREENDELTA, y + j, s, patch);
+    V_DrawPatch(x + w - video.deltaw, y + j, patch);
 
   // Draw beveled edge. 
-  V_DrawPatch(x - 8 - WIDESCREENDELTA,
-              y - 8, s,
+  V_DrawPatch(x - 8 - video.deltaw,
+              y - 8,
               W_CacheLumpName("brdr_tl", PU_CACHE));
     
-  V_DrawPatch(x + w - WIDESCREENDELTA,
-              y - 8, s,
+  V_DrawPatch(x + w - video.deltaw,
+              y - 8,
               W_CacheLumpName("brdr_tr", PU_CACHE));
     
-  V_DrawPatch(x - 8 - WIDESCREENDELTA,
-              y + h, s,
+  V_DrawPatch(x - 8 - video.deltaw,
+              y + h,
               W_CacheLumpName("brdr_bl", PU_CACHE));
     
-  V_DrawPatch(x + w - WIDESCREENDELTA,
-              y + h, s,
+  V_DrawPatch(x + w - video.deltaw,
+              y + h,
               W_CacheLumpName("brdr_br", PU_CACHE));
 }
 
 void R_FillBackScreen (void)
 {
-  if (scaledviewwidth == SCREENWIDTH)
+  if (scaledviewwidth == video.unscaledw)
     return;
 
-  // killough 11/98: use the function in m_menu.c
-  R_DrawBackground(gamemode==commercial ? "GRNROCK" : "FLOOR7_2", screens[1]);
+  // Allocate the background buffer if necessary
+  if (background_buffer == NULL)
+  {
+    int size = video.pitch * video.height;
+    background_buffer = Z_Malloc(size * sizeof(*background_buffer), PU_STATIC, NULL);
+  }
 
-  R_DrawBorder(viewwindowx >> hires, viewwindowy >> hires, scaledviewwidth, scaledviewheight, 1);
+  V_UseBuffer(background_buffer);
+
+  V_DrawBackground(gamemode == commercial ? "GRNROCK" : "FLOOR7_2");
+
+  R_DrawBorder(scaledviewx, scaledviewy, scaledviewwidth, scaledviewheight);
+
+  V_RestoreBuffer();
 }
 
 //
 // Copy a screen buffer.
 //
 
-void R_VideoErase(unsigned ofs, int count)
-{ 
-  if (hires)     // killough 11/98: hires support
-    {
-      ofs = ofs*4 - (ofs % SCREENWIDTH)*2;   // recompose offset
-      memcpy(screens[0]+ofs, screens[1]+ofs, count*=2);   // LFB copy.
-      ofs += SCREENWIDTH*2;
-    }
-  memcpy(screens[0]+ofs, screens[1]+ofs, count);   // LFB copy.
-} 
+void R_VideoErase(int x, int y, int w, int h)
+{
+  if (background_buffer == NULL)
+    return;
+
+  V_CopyRect(x, y, background_buffer, w, h, x, y);
+}
 
 //
 // R_DrawViewBorder
@@ -934,29 +939,24 @@ void R_VideoErase(unsigned ofs, int count)
 // can scale to hires automatically in R_VideoErase().
 //
 
-void R_DrawViewBorder(void) 
-{ 
-  int side, ofs, i;
- 
-  if (scaledviewwidth == SCREENWIDTH) 
+void R_DrawViewBorder(void)
+{
+  int side;
+
+  if (scaledviewwidth == video.unscaledw || background_buffer == NULL)
     return;
 
   // copy top
-  for (ofs = 0, i = viewwindowy >> hires; i--; ofs += SCREENWIDTH)
-    R_VideoErase(ofs, SCREENWIDTH); 
+  R_VideoErase(0, 0, video.unscaledw, scaledviewy);
 
   // copy sides
-  for (side = viewwindowx >> hires, i = scaledviewheight; i--;)
-    { 
-      R_VideoErase(ofs, side); 
-      ofs += SCREENWIDTH;
-      R_VideoErase(ofs - side, side); 
-    } 
+  side = scaledviewx;
+  R_VideoErase(0, scaledviewy, side, scaledviewheight);
+  R_VideoErase(video.unscaledw - side, scaledviewy, side, scaledviewheight);
 
-  // copy bottom 
-  for (i = viewwindowy >> hires; i--; ofs += SCREENWIDTH)
-    R_VideoErase(ofs, SCREENWIDTH); 
-} 
+  // copy bottom
+  R_VideoErase(0, scaledviewy + scaledviewheight, video.unscaledw, scaledviewy);
+}
 
 //----------------------------------------------------------------------------
 //
