@@ -19,20 +19,37 @@
 //
 //-----------------------------------------------------------------------------
 
+#define _USE_MATH_DEFINES
+#include <limits.h>
+#include <math.h>
+#include <stdint.h>
+#include <string.h>
+
+#include "d_loop.h"
+#include "d_player.h"
+#include "doomdata.h"
+#include "doomdef.h"
 #include "doomstat.h"
-#include "r_main.h"
-#include "r_things.h"
-#include "r_plane.h"
+#include "i_video.h"
+#include "p_mobj.h"
+#include "p_setup.h" // P_SegLengths
 #include "r_bsp.h"
+#include "r_data.h"
+#include "r_defs.h"
 #include "r_draw.h"
-#include "m_bbox.h"
+#include "r_main.h"
+#include "r_plane.h"
 #include "r_sky.h"
-#include "v_video.h"
+#include "r_state.h"
+#include "r_things.h"
+#include "r_voxel.h"
 #include "st_stuff.h"
-#include "hu_stuff.h"
+#include "v_flextran.h"
+#include "v_video.h"
+#include "z_zone.h"
 
 // Fineangles in the SCREENWIDTH wide window.
-#define FIELDOFVIEW 2048    
+#define FIELDOFVIEW 2048
 
 // killough: viewangleoffset is a legacy from the pre-v1.2 days, when Doom
 // had Left/Mid/Right viewing. +/-ANG90 offsets were placed here on each
@@ -44,18 +61,25 @@ lighttable_t *fixedcolormap;
 int      centerx, centery;
 fixed_t  centerxfrac, centeryfrac;
 fixed_t  projection;
+fixed_t  skyiscale;
 fixed_t  viewx, viewy, viewz;
 angle_t  viewangle;
+localview_t localview;
+double deltatics;
+boolean raw_input;
 fixed_t  viewcos, viewsin;
 player_t *viewplayer;
 extern lighttable_t **walllights;
 fixed_t  viewheightfrac; // [FG] sprite clipping optimizations
 
+static fixed_t focallength, lightfocallength;
+
 //
 // precalculated math tables
 //
 
 angle_t clipangle;
+angle_t vx_clipangle;
 
 // The viewangletox[viewangle + FINEANGLES/4] lookup
 // maps the visible view angles to screen X coordinates,
@@ -68,10 +92,10 @@ int viewangletox[FINEANGLES/2];
 // to the lowest viewangle that maps back to x ranges
 // from clipangle to -clipangle.
 
-angle_t xtoviewangle[MAX_SCREENWIDTH+1];   // killough 2/8/98
+angle_t *xtoviewangle = NULL;   // killough 2/8/98
 
 // [FG] linear horizontal sky scrolling
-angle_t linearskyangle[MAX_SCREENWIDTH+1];
+angle_t *linearskyangle = NULL;
 
 int LIGHTLEVELS;
 int LIGHTSEGSHIFT;
@@ -247,11 +271,13 @@ static fixed_t centerxfrac_nonwide;
 //
 // killough 5/2/98: reformatted
 
-static void R_InitTextureMapping (void)
+static void R_InitTextureMapping(void)
 {
   register int i,x;
-  fixed_t focallength;
-    
+  fixed_t slopefrac;
+  angle_t fov;
+  double linearskyfactor;
+
   // Use tangent table to generate viewangletox:
   //  viewangletox will give the next greatest x
   //  after the view angle.
@@ -259,15 +285,37 @@ static void R_InitTextureMapping (void)
   // Calc focallength
   //  so FIELDOFVIEW angles covers SCREENWIDTH.
 
-  focallength = FixedDiv(centerxfrac_nonwide, finetangent[FINEANGLES/4+FIELDOFVIEW/2]);
-        
+  if (custom_fov == FOV_DEFAULT && centerxfrac == centerxfrac_nonwide)
+  {
+    fov = FIELDOFVIEW;
+    slopefrac = finetangent[FINEANGLES / 4 + fov / 2];
+    focallength = FixedDiv(centerxfrac_nonwide, slopefrac);
+    lightfocallength = centerxfrac_nonwide;
+    projection = centerxfrac_nonwide;
+  }
+  else
+  {
+    const double slope = (tan(custom_fov * M_PI / 360.0) *
+                          centerxfrac / centerxfrac_nonwide);
+
+    // For correct light across FOV range. Calculated like R_InitTables().
+    const double lightangle = atan(slope) + M_PI / FINEANGLES;
+    const double lightslopefrac = tan(lightangle) * FRACUNIT;
+    lightfocallength = FixedDiv(centerxfrac, lightslopefrac);
+
+    fov = atan(slope) * FINEANGLES / M_PI;
+    slopefrac = finetangent[FINEANGLES / 4 + fov / 2];
+    focallength = FixedDiv(centerxfrac, slopefrac);
+    projection = centerxfrac / slope;
+  }
+
   for (i=0 ; i<FINEANGLES/2 ; i++)
     {
       int t;
-      if (finetangent[i] > FRACUNIT*2)
+      if (finetangent[i] > slopefrac)
         t = -1;
       else
-        if (finetangent[i] < -FRACUNIT*2)
+        if (finetangent[i] < -slopefrac)
           t = viewwidth+1;
       else
         {
@@ -286,13 +334,15 @@ static void R_InitTextureMapping (void)
   //  xtoviewangle will give the smallest view angle
   //  that maps to x.
 
+  linearskyfactor = FIXED2DOUBLE(slopefrac) * ANG90;
+
   for (x=0; x<=viewwidth; x++)
     {
       for (i=0; viewangletox[i] > x; i++)
         ;
       xtoviewangle[x] = (i<<ANGLETOFINESHIFT)-ANG90;
       // [FG] linear horizontal sky scrolling
-      linearskyangle[x] = ((viewwidth/2-x)*((SCREENWIDTH<<6)/viewwidth))*(ANG90/(NONWIDEWIDTH<<6));
+      linearskyangle[x] = (0.5 - x / (double)viewwidth) * linearskyfactor;
     }
     
   // Take out the fencepost cases from viewangletox.
@@ -304,6 +354,8 @@ static void R_InitTextureMapping (void)
         viewangletox[i] = viewwidth;
         
   clipangle = xtoviewangle[0];
+
+  vx_clipangle = clipangle - ((fov << ANGLETOFINESHIFT) - ANG90);
 }
 
 //
@@ -389,11 +441,14 @@ void R_InitLightTables (void)
       int j, startmap = ((LIGHTLEVELS-LIGHTBRIGHT-i)*2)*NUMCOLORMAPS/LIGHTLEVELS;
 
       for (cm = 0; cm < numcolormaps; ++cm)
+      {
+        c_scalelight[cm][i] = Z_Malloc(MAXLIGHTSCALE * sizeof(***c_scalelight), PU_STATIC, 0);
         c_zlight[cm][i] = Z_Malloc(MAXLIGHTZ * sizeof(***c_zlight), PU_STATIC, 0);
+      }
 
       for (j=0; j<MAXLIGHTZ; j++)
         {
-          int scale = FixedDiv ((ORIGWIDTH/2*FRACUNIT), (j+1)<<LIGHTZSHIFT);
+          int scale = FixedDiv ((SCREENWIDTH/2*FRACUNIT), (j+1)<<LIGHTZSHIFT);
           int t, level = startmap - (scale >>= LIGHTSCALESHIFT)/DISTMAP;
 
           if (level < 0)
@@ -410,6 +465,53 @@ void R_InitLightTables (void)
     }
 }
 
+boolean setsmoothlight;
+
+void R_SmoothLight(void)
+{
+  setsmoothlight = false;
+  // [crispy] re-calculate the zlight[][] array
+  R_InitLightTables();
+  // [crispy] re-calculate the scalelight[][] array
+  // R_ExecuteSetViewSize();
+  // [crispy] re-calculate fake contrast
+  P_SegLengths(true);
+}
+
+int R_GetLightIndex(fixed_t scale)
+{
+  const int index = FixedDiv(scale * 160, lightfocallength) >> LIGHTSCALESHIFT;
+  return BETWEEN(0, MAXLIGHTSCALE - 1, index);
+}
+
+static fixed_t viewpitch;
+
+static void R_SetupFreelook(void)
+{
+  fixed_t dy;
+  int i;
+
+  if (viewpitch)
+  {
+    dy = FixedMul(projection, -finetangent[(ANG90 - viewpitch) >> ANGLETOFINESHIFT]);
+    dy = (fixed_t)((int64_t)dy * SCREENHEIGHT / ACTUALHEIGHT);
+  }
+  else
+  {
+    dy = 0;
+  }
+
+  centeryfrac = (viewheight << FRACBITS) / 2 + dy;
+  centery = centeryfrac >> FRACBITS;
+
+  for (i = 0; i < viewheight; i++)
+  {
+    dy = abs(((i - centery) << FRACBITS) + FRACUNIT / 2);
+    yslope[i] = FixedDiv(projection, dy);
+  }
+}
+
+
 //
 // R_SetViewSize
 // Do not really change anything here,
@@ -420,8 +522,6 @@ void R_InitLightTables (void)
 boolean setsizeneeded;
 int     setblocks;
 
-static int viewblocks;
-
 void R_SetViewSize(int blocks)
 {
   setsizeneeded = true;
@@ -434,101 +534,100 @@ void R_SetViewSize(int blocks)
 
 void R_ExecuteSetViewSize (void)
 {
-  int i, j;
-  extern void AM_Start(void);
+  int i;
+  vrect_t view;
 
   setsizeneeded = false;
 
   if (setblocks == 11)
     {
       scaledviewwidth_nonwide = NONWIDEWIDTH;
-      scaledviewwidth = SCREENWIDTH;
+      scaledviewwidth = video.unscaledw;
       scaledviewheight = SCREENHEIGHT;                    // killough 11/98
     }
   // [crispy] hard-code to SCREENWIDTH and SCREENHEIGHT minus status bar height
   else if (setblocks == 10)
     {
       scaledviewwidth_nonwide = NONWIDEWIDTH;
-      scaledviewwidth = SCREENWIDTH;
-      scaledviewheight = SCREENHEIGHT-ST_HEIGHT;
+      scaledviewwidth = video.unscaledw;
+      scaledviewheight = SCREENHEIGHT - ST_HEIGHT;
     }
   else
     {
-      scaledviewwidth_nonwide = setblocks*32;
-      scaledviewheight = (setblocks*168/10) & ~7;        // killough 11/98
+      const int st_screen = SCREENHEIGHT - ST_HEIGHT;
+
+      scaledviewwidth_nonwide = setblocks * 32;
+      scaledviewheight = (setblocks * st_screen / 10) & ~7; // killough 11/98
+
       if (widescreen)
-      {
-        const int widescreen_edge_aligner = (8 << hires) - 1;
-
-        scaledviewwidth = scaledviewheight*SCREENWIDTH/(SCREENHEIGHT-ST_HEIGHT);
-        // [crispy] make sure scaledviewwidth is an integer multiple of the bezel patch width
-        scaledviewwidth = (scaledviewwidth + widescreen_edge_aligner) & (int)~widescreen_edge_aligner;
-        scaledviewwidth = MIN(scaledviewwidth, SCREENWIDTH);
-      }
+        scaledviewwidth = (scaledviewheight * video.unscaledw / st_screen) & ~7;
       else
-      {
         scaledviewwidth = scaledviewwidth_nonwide;
-      }
     }
 
-  viewwidth = scaledviewwidth << hires;                  // killough 11/98
-  viewheight = scaledviewheight << hires;                // killough 11/98
-  viewwidth_nonwide = scaledviewwidth_nonwide << hires;
+  scaledviewx = (video.unscaledw - scaledviewwidth) / 2;
 
-  viewblocks = MIN(setblocks, 10) << hires;
+  if (scaledviewwidth == video.unscaledw)
+    scaledviewy = 0;
+  else
+    scaledviewy = (SCREENHEIGHT - ST_HEIGHT - scaledviewheight) / 2;
 
-  centery = viewheight/2;
-  centerx = viewwidth/2;
-  centerxfrac = centerx<<FRACBITS;
-  centeryfrac = centery<<FRACBITS;
-  centerxfrac_nonwide = (viewwidth_nonwide/2)<<FRACBITS;
-  projection = centerxfrac_nonwide;
-  viewheightfrac = viewheight<<(FRACBITS+1); // [FG] sprite clipping optimizations
+  view.x = scaledviewx;
+  view.y = scaledviewy;
+  view.w = scaledviewwidth;
+  view.h = scaledviewheight;
+
+  V_ScaleRect(&view);
+
+  viewwindowx = view.sx;
+  viewwindowy = view.sy;
+  viewwidth   = view.sw;
+  viewheight  = view.sh;
+
+  viewwidth_nonwide = V_ScaleX(scaledviewwidth_nonwide);
+
+  centerxfrac = (viewwidth << FRACBITS) / 2;
+  centerx = centerxfrac >> FRACBITS;
+  centerxfrac_nonwide = (viewwidth_nonwide << FRACBITS) / 2;
+
+  viewheightfrac = viewheight << (FRACBITS + 1); // [FG] sprite clipping optimizations
+
+  R_InitBuffer();       // killough 11/98
 
-  R_InitBuffer(scaledviewwidth, scaledviewheight);       // killough 11/98
-        
   R_InitTextureMapping();
-    
-  // psprite scales
-  pspritescale = FixedDiv(viewwidth_nonwide, ORIGWIDTH);       // killough 11/98
-  pspriteiscale= FixedDiv(ORIGWIDTH, viewwidth_nonwide);       // killough 11/98
 
-  // thing clipping
-  for (i=0 ; i<viewwidth ; i++)
-    screenheightarray[i] = viewheight;
-    
-  // planes
-  for (i=0 ; i<viewheight ; i++)
-    {   // killough 5/2/98: reformatted
-      for (j = 0; j < LOOKDIRS; j++)
-      {
-        // [crispy] re-generate lookup-table for yslope[] whenever "viewheight" or "hires" change
-        fixed_t dy = abs(((i-viewheight/2-(j-LOOKDIRMAX)*viewblocks/10)<<FRACBITS)+FRACUNIT/2);
-        yslopes[j][i] = FixedDiv(viewwidth_nonwide*(FRACUNIT/2), dy);
-      }
-    }
-  yslope = yslopes[LOOKDIRMAX];
-        
+  R_SetupFreelook();
+
+  // psprite scales
+  pspritescale = FixedDiv(viewwidth_nonwide, SCREENWIDTH);       // killough 11/98
+  pspriteiscale = FixedDiv(SCREENWIDTH, viewwidth_nonwide);      // killough 11/98
+
+  // [FG] make sure that the product of the weapon sprite scale factor
+  //      and its reciprocal is always at least FRACUNIT to
+  //      fix garbage lines at the top of weapon sprites
+  while (FixedMul(pspriteiscale, pspritescale) < FRACUNIT)
+    pspriteiscale++;
+
+  skyiscale = FixedDiv(160 << FRACBITS, focallength);
+
   for (i=0 ; i<viewwidth ; i++)
     {
       fixed_t cosadj = abs(finecosine[xtoviewangle[i]>>ANGLETOFINESHIFT]);
       distscale[i] = FixedDiv(FRACUNIT,cosadj);
+      // thing clipping
+      screenheightarray[i] = viewheight;
     }
-    
+
   // Calculate the light levels to use
   //  for each level / scale combination.
   for (i=0; i<LIGHTLEVELS; i++)
     {
       int j, startmap = ((LIGHTLEVELS-LIGHTBRIGHT-i)*2)*NUMCOLORMAPS/LIGHTLEVELS;
-      int cm;
-
-      for (cm = 0; cm < numcolormaps; ++cm)
-        c_scalelight[cm][i] = Z_Malloc(MAXLIGHTSCALE * sizeof(***c_scalelight), PU_STATIC, 0);
 
       for (j=0 ; j<MAXLIGHTSCALE ; j++)
         {                                       // killough 11/98:
           int t, level = startmap - j*NONWIDEWIDTH/scaledviewwidth_nonwide/DISTMAP;
-            
+
           if (level < 0)
             level = 0;
 
@@ -543,19 +642,10 @@ void R_ExecuteSetViewSize (void)
         }
     }
 
-    HU_disable_all_widgets();
+  // [crispy] forcefully initialize the status bar backing screen
+  ST_refreshBackground(true);
 
-    // [crispy] forcefully initialize the status bar backing screen
-    ST_refreshBackground(true);
-
-    // [FG] reinitialize Automap
-    if (automapactive)
-        AM_Start();
-
-    // [FG] spectre drawing mode
-    R_SetFuzzColumnMode();
-
-    pspr_interp = false;
+  pspr_interp = false;
 }
 
 //
@@ -570,6 +660,10 @@ void R_Init (void)
   R_InitLightTables();
   R_InitSkyMap();
   R_InitTranslationTables();
+  V_InitFlexTranTable();
+
+  // [FG] spectre drawing mode
+  R_SetFuzzColumnMode();
 }
 
 //
@@ -592,25 +686,22 @@ subsector_t *R_PointInSubsector(fixed_t x, fixed_t y)
   return &subsectors[nodenum & ~NF_SUBSECTOR];
 }
 
-// [AM] Interpolate between two angles.
-angle_t R_InterpolateAngle(angle_t oangle, angle_t nangle, fixed_t scale)
+static inline boolean CheckLocalView(const player_t *player)
 {
-    if (nangle == oangle)
-        return nangle;
-    else if (nangle > oangle)
-    {
-        if (nangle - oangle < ANG270)
-            return oangle + (angle_t)((nangle - oangle) * FIXED2DOUBLE(scale));
-        else // Wrapped around
-            return oangle - (angle_t)((oangle - nangle) * FIXED2DOUBLE(scale));
-    }
-    else // nangle < oangle
-    {
-        if (oangle - nangle < ANG270)
-            return oangle - (angle_t)((oangle - nangle) * FIXED2DOUBLE(scale));
-        else // Wrapped around
-            return oangle + (angle_t)((nangle - oangle) * FIXED2DOUBLE(scale));
-    }
+  return (
+    // Don't use localview when interpolation is preferred.
+    raw_input &&
+    // Don't use localview if the player is spying.
+    player == &players[consoleplayer] &&
+    // Don't use localview if the player is dead.
+    player->playerstate != PST_DEAD &&
+    // Don't use localview if the player just teleported.
+    !player->mo->reactiontime &&
+    // Don't use localview if a demo is playing.
+    !demoplayback &&
+    // Don't use localview during a netgame (single-player or solo-net only).
+    (!netgame || solonet)
+  );
 }
 
 //
@@ -618,10 +709,10 @@ angle_t R_InterpolateAngle(angle_t oangle, angle_t nangle, fixed_t scale)
 //
 
 void R_SetupFrame (player_t *player)
-{               
+{
   int i, cm;
-  int tempCentery, pitch;
-    
+  fixed_t pitch;
+
   viewplayer = player;
   // [AM] Interpolate the player camera if the feature is enabled.
   if (uncapped &&
@@ -634,40 +725,59 @@ void R_SetupFrame (player_t *player)
       // Don't interpolate during a paused state
       leveltime > oldleveltime)
   {
+    // Use localview unless the player or game is in an invalid state, in which
+    // case fall back to interpolation.
+    const boolean use_localview = CheckLocalView(player);
+
     // Interpolate player camera from their old position to their current one.
-    viewx = player->mo->oldx + FixedMul(player->mo->x - player->mo->oldx, fractionaltic);
-    viewy = player->mo->oldy + FixedMul(player->mo->y - player->mo->oldy, fractionaltic);
-    viewz = player->oldviewz + FixedMul(player->viewz - player->oldviewz, fractionaltic);
-    viewangle = R_InterpolateAngle(player->mo->oldangle, player->mo->angle, fractionaltic) + viewangleoffset;
+    viewx = LerpFixed(player->mo->oldx, player->mo->x);
+    viewy = LerpFixed(player->mo->oldy, player->mo->y);
+    viewz = LerpFixed(player->oldviewz, player->viewz);
+
+    if (use_localview)
+    {
+      viewangle = (player->mo->angle + localview.angle - localview.ticangle +
+                   LerpAngle(localview.oldticangle, localview.ticangle));
+    }
+    else
+    {
+      viewangle = LerpAngle(player->mo->oldangle, player->mo->angle);
+    }
+
+    if (use_localview && !player->centering)
+    {
+      pitch = player->pitch + localview.pitch;
+      pitch = BETWEEN(-MAX_PITCH_ANGLE, MAX_PITCH_ANGLE, pitch);
+    }
+    else
+    {
+      pitch = LerpFixed(player->oldpitch, player->pitch);
+    }
+
     // [crispy] pitch is actual lookdir and weapon pitch
-    pitch = (player->oldlookdir + (player->lookdir - player->oldlookdir) * FIXED2DOUBLE(fractionaltic)) / MLOOKUNIT
-                + (player->oldrecoilpitch + FixedMul(player->recoilpitch - player->oldrecoilpitch, fractionaltic));
+    pitch += LerpFixed(player->oldrecoilpitch, player->recoilpitch);
   }
   else
   {
-  viewx = player->mo->x;
-  viewy = player->mo->y;
-  viewz = player->viewz; // [FG] moved here
-  viewangle = player->mo->angle + viewangleoffset;
-  // [crispy] pitch is actual lookdir and weapon pitch
-  pitch = player->lookdir / MLOOKUNIT + player->recoilpitch;
+    viewx = player->mo->x;
+    viewy = player->mo->y;
+    viewz = player->viewz; // [FG] moved here
+    viewangle = player->mo->angle;
+    // [crispy] pitch is actual lookdir and weapon pitch
+    pitch = player->pitch + player->recoilpitch;
   }
+
+  if (pitch != viewpitch)
+  {
+    viewpitch = pitch;
+    R_SetupFreelook();
+  }
+
+  // 3-screen display mode.
+  viewangle += viewangleoffset;
+
   extralight = player->extralight;
   extralight += STRICTMODE(LIGHTBRIGHT * extra_level_brightness);
-    
-  if (pitch > LOOKDIRMAX)
-    pitch = LOOKDIRMAX;
-  else if (pitch < -LOOKDIRMAX)
-    pitch = -LOOKDIRMAX;
-
-  // apply new yslope[] whenever "lookdir", "viewheight" or "hires" change
-  tempCentery = viewheight/2 + pitch * viewblocks / 10;
-  if (centery != tempCentery)
-  {
-      centery = tempCentery;
-      centeryfrac = centery << FRACBITS;
-      yslope = yslopes[LOOKDIRMAX + pitch];
-  }
 
   viewsin = finesine[viewangle>>ANGLETOFINESHIFT];
   viewcos = finecosine[viewangle>>ANGLETOFINESHIFT];
@@ -709,20 +819,14 @@ void R_SetupFrame (player_t *player)
 // R_ShowStats
 //
 
-int rendered_visplanes, rendered_segs, rendered_vissprites;
-
-void R_ShowRenderingStats(void)
-{
-  extern int fps;
-  displaymsg("Segs %d, Visplanes %d, Sprites %d, FPS %d",
-          rendered_segs, rendered_visplanes, rendered_vissprites, fps);
-}
+int rendered_visplanes, rendered_segs, rendered_vissprites, rendered_voxels;
 
 static void R_ClearStats(void)
 {
   rendered_visplanes = 0;
   rendered_segs = 0;
   rendered_vissprites = 0;
+  rendered_voxels = 0;
 }
 
 int autodetect_hom = 0;       // killough 2/7/98: HOM autodetection flag
@@ -741,13 +845,14 @@ void R_RenderPlayerView (player_t* player)
   R_ClearDrawSegs ();
   R_ClearPlanes ();
   R_ClearSprites ();
-    
+  VX_ClearVoxels ();
+
   if (autodetect_hom)
     { // killough 2/10/98: add flashing red HOM indicators
-      byte c[47*47];
+      pixel_t c[47*47];
       extern int lastshottic;
       int i , color = !flashing_hom || (gametic % 20) < 9 ? 0xb0 : 0;
-      memset(*screens+viewwindowy*linesize,color,viewheight*linesize);
+      V_FillRect(scaledviewx, scaledviewy, scaledviewwidth, scaledviewheight, color);
       for (i=0;i<47*47;i++)
         {
           char t =
@@ -808,8 +913,8 @@ void R_RenderPlayerView (player_t* player)
           c[i] = t=='/' ? color : t;
         }
       if (gametic-lastshottic < TICRATE*2 && gametic-lastshottic > TICRATE/8)
-        V_DrawBlock((viewwindowx +  viewwidth/2 - 24)>>hires,
-                    (viewwindowy + viewheight/2 - 24)>>hires, 0, 47, 47, c);
+        V_DrawBlock(scaledviewx +  scaledviewwidth/2 - 24,
+                    scaledviewy + scaledviewheight/2 - 24, 47, 47, c);
       R_DrawViewBorder();
     }
 
@@ -818,7 +923,9 @@ void R_RenderPlayerView (player_t* player)
 
   // The head node is the last node output.
   R_RenderBSPNode (numnodes-1);
-    
+
+  VX_NearbySprites ();
+
   // [FG] update automap while playing
   if (automap_on)
     return;
@@ -839,6 +946,13 @@ void R_RenderPlayerView (player_t* player)
   NetUpdate ();
 }
 
+void R_InitAnyRes(void)
+{
+  R_InitSpritesRes();
+  R_InitBufferRes();
+  R_InitPlanesRes();
+}
+
 //----------------------------------------------------------------------------
 //
 // $Log: r_main.c,v $
