@@ -20,15 +20,15 @@
 #ifndef __R_THINGS__
 #define __R_THINGS__
 
-#include "doomdef.h"
+#include "doomtype.h"
 #include "m_fixed.h"
 #include "r_defs.h"
 
 // Constant arrays used for psprite clipping and initializing clipping.
 
 // [FG] 32-bit integer math
-extern int negonearray[MAX_SCREENWIDTH];         // killough 2/8/98:
-extern int screenheightarray[MAX_SCREENWIDTH];   // change to MAX_*
+extern int *negonearray;         // killough 2/8/98:
+extern int *screenheightarray;   // change to MAX_*
 
 // Vars for R_DrawMaskedColumn
 
@@ -41,6 +41,8 @@ extern fixed_t pspriteiscale;
 
 extern boolean pspr_interp; // weapon bobbing interpolation
 
+extern lighttable_t **spritelights;
+
 void R_DrawMaskedColumn(column_t *column);
 void R_SortVisSprites(void);
 void R_AddSprites(sector_t *sec,int); // killough 9/18/98
@@ -52,6 +54,8 @@ void R_DrawMasked(void);
 
 void R_ClipVisSprite(vissprite_t *vis, int xl, int xh);
 
+void R_InitSpritesRes(void);
+
 #endif
 
 //----------------------------------------------------------------------------
