@@ -20,17 +20,22 @@
 // killough 3/7/98: modified to allow arbitrary listeners in spy mode
 // killough 5/2/98: reindented, removed useless code, beautified
 
-#include <stdlib.h> // [FG] qsort()
+#include <string.h>
 
+#include "doomdef.h"
 #include "doomstat.h"
 #include "i_printf.h"
-#include "s_sound.h"
-#include "s_musinfo.h" // [crispy] struct musinfo
 #include "i_sound.h"
-#include "r_main.h"
-#include "m_random.h"
+#include "i_system.h"
 #include "m_misc2.h"
+#include "m_random.h"
+#include "p_mobj.h"
+#include "s_musinfo.h" // [crispy] struct musinfo
+#include "s_sound.h"
+#include "sounds.h"
+#include "u_mapinfo.h"
 #include "w_wad.h"
+#include "z_zone.h"
 
 //jff end sound enabling variables readable here
 
@@ -443,6 +448,14 @@ void S_UpdateSounds(const mobj_t *listener)
    I_ProcessSoundUpdates();
 }
 
+void S_UpdateMusic(void)
+{
+    if (nomusicparm)
+        return;
+
+    I_UpdateMusic();
+}
+
 void S_SetMusicVolume(int volume)
 {
    //jff 1/22/98 return if music is not enabled
@@ -503,7 +516,7 @@ void S_ChangeMusic(int musicnum, int looping)
    if(!music->lumpnum)
    {
       char namebuf[9];
-      sprintf(namebuf, "d_%s", music->name);
+      M_snprintf(namebuf, sizeof(namebuf), "d_%s", music->name);
       music->lumpnum = W_GetNumForName(namebuf);
    }
    
@@ -536,15 +549,14 @@ void S_ChangeMusInfoMusic (int lumpnum, int looping)
 {
    musicinfo_t *music;
 
-   if (nomusicparm)
+   if (PLAYBACK_SKIP)
    {
+      musinfo.current_item = lumpnum;
       return;
    }
 
-   // [crispy] play no music if this is not the right map
-   if (nodrawers && singletics)
+   if (nomusicparm)
    {
-      musinfo.current_item = lumpnum;
       return;
    }
 
@@ -709,7 +721,7 @@ static void InitE4Music (void)
     musicinfo_t *music = &S_music[i];
     char namebuf[9];
 
-    sprintf(namebuf, "d_%s", music->name);
+    M_snprintf(namebuf, sizeof(namebuf), "d_%s", music->name);
 
     if (W_CheckNumForName(namebuf) == -1)
     {
