@@ -20,22 +20,35 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <stdlib.h>
+
+#include "am_map.h"
+#include "d_event.h"
+#include "d_items.h"
+#include "d_player.h"
 #include "doomdef.h"
 #include "doomstat.h"
-#include "m_random.h"
-#include "i_video.h"
-#include "w_wad.h"
-#include "st_stuff.h"
 #include "hu_stuff.h" // [FG] hud_displayed
-#include "st_lib.h"
-#include "r_main.h"
-#include "am_map.h"
+#include "i_printf.h"
+#include "i_video.h"
+#include "info.h"
 #include "m_cheat.h"
+#include "m_misc2.h"
+#include "m_random.h"
+#include "m_swap.h"
+#include "p_mobj.h"
+#include "r_data.h"
+#include "r_defs.h"
+#include "r_main.h"
+#include "r_state.h"
 #include "s_sound.h"
 #include "sounds.h"
-#include "dstrings.h"
-#include "m_misc2.h"
-#include "m_swap.h"
+#include "st_lib.h"
+#include "st_stuff.h"
+#include "tables.h"
+#include "v_video.h"
+#include "w_wad.h"
+#include "z_zone.h"
 
 // [crispy] immediately redraw status bar after help screens have been shown
 extern boolean inhelpscreens;
@@ -60,10 +73,6 @@ extern boolean inhelpscreens;
 #define ST_FX                   143
 #define ST_FY                   169
 
-// Should be set to patch width
-//  for tall numbers later on
-#define ST_TALLNUMWIDTH         (tallnum[0]->width)
-
 // Number of status faces.
 #define ST_NUMPAINFACES         5
 #define ST_NUMSTRAIGHTFACES     3
@@ -180,6 +189,9 @@ extern boolean inhelpscreens;
 
 // killough 2/8/98: weapon info position macros UNUSED, removed here
 
+// graphics are drawn to a backing screen and blitted to the real screen
+static pixel_t *st_backing_screen = NULL;
+
 // main player in game
 static player_t *plyr;
 
@@ -241,7 +253,7 @@ static patch_t *arms[6][2];
 static st_number_t w_ready;
 
 // [Alaux]
-int smooth_counts;
+int hud_animated_counts;
 int st_health = 100;
 int st_armor = 0;
 
@@ -299,6 +311,8 @@ static int      st_faceindex = 0;
 
 // holds key-type for each key box on bar
 static int      keyboxes[3];
+// [crispy] blinking key or skull in the status bar
+int             st_keyorskull[3];
 
 // a random number per tick
 static int      st_randomnumber;
@@ -313,149 +327,142 @@ void ST_Stop(void);
 
 int st_solidbackground;
 
+static void ST_DrawSolidBackground(int st_x)
+{
+  // [FG] calculate average color of the 16px left and right of the status bar
+  const int vstep[][2] = {{0, 1}, {1, 2}, {2, ST_HEIGHT}};
+
+  byte *pal = W_CacheLumpName("PLAYPAL", PU_STATIC);
+
+  // [FG] temporarily draw status bar to background buffer
+  V_DrawPatch(st_x, 0, sbar);
+
+  const int offset = MAX(st_x + video.deltaw - SHORT(sbar->leftoffset), 0);
+  const int width  = MIN(SHORT(sbar->width), video.unscaledw);
+  const int depth  = 16;
+  int v;
+
+  // [FG] separate colors for the top rows
+  for (v = 0; v < arrlen(vstep); v++)
+  {
+    int x, y;
+    const int v0 = vstep[v][0], v1 = vstep[v][1];
+    unsigned r = 0, g = 0, b = 0;
+    byte col;
+
+    for (y = v0; y < v1; y++)
+    {
+      for (x = 0; x < depth; x++)
+      {
+        byte *c = st_backing_screen + V_ScaleY(y) * video.pitch + V_ScaleX(x + offset);
+        r += pal[3 * c[0] + 0];
+        g += pal[3 * c[0] + 1];
+        b += pal[3 * c[0] + 2];
+
+        c += V_ScaleX(width - 2 * x - 1);
+        r += pal[3 * c[0] + 0];
+        g += pal[3 * c[0] + 1];
+        b += pal[3 * c[0] + 2];
+      }
+    }
+
+    r /= 2 * depth * (v1 - v0);
+    g /= 2 * depth * (v1 - v0);
+    b /= 2 * depth * (v1 - v0);
+
+    // [FG] tune down to half saturation (for empiric reasons)
+    col = I_GetPaletteIndex(pal, r/2, g/2, b/2);
+
+    V_FillRect(0, v0, video.unscaledw, v1 - v0, col);
+  }
+
+  Z_ChangeTag (pal, PU_CACHE);
+}
+
 void ST_refreshBackground(boolean force)
 {
-  if (st_classicstatusbar)
+    int st_x;
+
+    if (!st_classicstatusbar)
     {
-      const int st_x = (SHORT(sbar->width) > ORIGWIDTH && SHORT(sbar->leftoffset) == 0) ?
-                       ST_X + (ORIGWIDTH - SHORT(sbar->width)) / 2 :
-                       ST_X;
+        return;
+    }
 
-      if (SCREENWIDTH != ST_WIDTH)
-      {
-        int x, y;
-        byte *dest = screens[BG];
+    st_x = ST_X + (SCREENWIDTH - SHORT(sbar->width)) / 2 + SHORT(sbar->leftoffset);
 
+    V_UseBuffer(st_backing_screen);
+
+    if (video.unscaledw != ST_WIDTH)
+    {
         if (st_solidbackground)
         {
-          // [FG] calculate average color of the 16px left and right of the status bar
-          const int vstep[][2] = {{0, 1}, {1, 2}, {2, ST_HEIGHT}};
-          const int hstep = hires ? (4 * SCREENWIDTH) : SCREENWIDTH;
-          const int lo = MAX(st_x + WIDESCREENDELTA - SHORT(sbar->leftoffset), 0);
-          const int w = MIN(SHORT(sbar->width), SCREENWIDTH);
-          const int depth = 16;
-          byte *pal = W_CacheLumpName("PLAYPAL", PU_STATIC);
-          int v;
-
-          // [FG] temporarily draw status bar to background buffer
-          V_DrawPatch(st_x, 0, BG, sbar);
-
-          // [FG] separate colors for the top rows
-          for (v = 0; v < arrlen(vstep); v++)
-          {
-            const int v0 = vstep[v][0], v1 = vstep[v][1];
-            unsigned r = 0, g = 0, b = 0;
-            byte col;
-
-            for (y = v0; y < v1; y++)
-            {
-              for (x = 0; x < depth; x++)
-              {
-                byte *c = dest + y * hstep + ((x + lo) << hires);
-                r += pal[3 * c[0] + 0];
-                g += pal[3 * c[0] + 1];
-                b += pal[3 * c[0] + 2];
-
-                c += (w - 2 * x - 1) << hires;
-                r += pal[3 * c[0] + 0];
-                g += pal[3 * c[0] + 1];
-                b += pal[3 * c[0] + 2];
-              }
-            }
-
-            r /= 2 * depth * (v1 - v0);
-            g /= 2 * depth * (v1 - v0);
-            b /= 2 * depth * (v1 - v0);
-
-            // [FG] tune down to half saturation (for empiric reasons)
-            col = I_GetPaletteIndex(pal, r/2, g/2, b/2);
-
-            // [FG] fill background buffer with average status bar color
-            for (y = (v0 << hires); y < (v1 << hires); y++)
-            {
-              memset(dest + y * (SCREENWIDTH << hires), col, (SCREENWIDTH << hires));
-            }
-          }
-
-          Z_ChangeTag (pal, PU_CACHE);
+            ST_DrawSolidBackground(st_x);
         }
         else
         {
-          // [crispy] this is our own local copy of R_FillBackScreen() to
-          // fill the entire background of st_backing_screen with the bezel pattern,
-          // so it appears to the left and right of the status bar in widescreen mode
-          byte *src;
-          const char *name = (gamemode == commercial) ? "GRNROCK" : "FLOOR7_2";
+            // [crispy] this is our own local copy of R_FillBackScreen() to fill
+            // the entire background of st_backing_screen with the bezel
+            // pattern, so it appears to the left and right of the status bar
+            // in widescreen mode
+            const char *name = (gamemode == commercial) ? "GRNROCK" : "FLOOR7_2";
 
-          src = W_CacheLumpNum(firstflat + R_FlatNumForName(name), PU_CACHE);
+            const byte *src = W_CacheLumpNum(firstflat + R_FlatNumForName(name), PU_CACHE);
 
-          if (hires)
-          {
-            for (y = (SCREENHEIGHT-ST_HEIGHT)<<1; y < SCREENHEIGHT<<1; y++)
-                for (x = 0; x < SCREENWIDTH<<1; x += 2)
-                {
-                    const byte dot = src[(((y>>1)&63)<<6) + ((x>>1)&63)];
+            V_TileBlock64(SCREENHEIGHT - ST_HEIGHT, video.unscaledw, ST_HEIGHT, src);
 
-                    *dest++ = dot;
-                    *dest++ = dot;
-                }
-          }
-          else
-          {
-            for (y = SCREENHEIGHT-ST_HEIGHT; y < SCREENHEIGHT; y++)
-              for (x = 0; x < SCREENWIDTH; x++)
-              {
-                *dest++ = src[((y&63)<<6) + (x&63)];
-              }
-          }
-
-          // [crispy] preserve bezel bottom edge
-          if (scaledviewwidth == SCREENWIDTH)
-          {
-            patch_t *const patch = W_CacheLumpName("brdr_b", PU_CACHE);
-
-            for (x = 0; x < WIDESCREENDELTA; x += 8)
+            // [crispy] preserve bezel bottom edge
+            if (scaledviewwidth == video.unscaledw)
             {
-              V_DrawPatch(x - WIDESCREENDELTA, 0, BG, patch);
-              V_DrawPatch(ORIGWIDTH + WIDESCREENDELTA - x - 8, 0, BG, patch);
+                int x;
+                patch_t *patch = W_CacheLumpName("brdr_b", PU_CACHE);
+
+                for (x = 0; x < video.deltaw; x += 8)
+                {
+                    V_DrawPatch(x - video.deltaw, 0, patch);
+                    V_DrawPatch(SCREENWIDTH + video.deltaw - x - 8, 0, patch);
+                }
             }
-          }
         }
-      }
+    }
 
-      // [crispy] center unity rerelease wide status bar
-      V_DrawPatch(st_x, 0, BG, sbar);
+    // [crispy] center unity rerelease wide status bar
+    V_DrawPatch(st_x, 0, sbar);
 
-      // draw right side of bar if needed (Doom 1.0)
-      if (sbarr)
-        V_DrawPatch(ST_ARMSBGX, 0, BG, sbarr);
+    // draw right side of bar if needed (Doom 1.0)
+    if (sbarr)
+        V_DrawPatch(ST_ARMSBGX, 0, sbarr);
 
-      if (st_notdeathmatch)
-        V_DrawPatch(ST_ARMSBGX, 0, BG, armsbg);
+    if (st_notdeathmatch)
+        V_DrawPatch(ST_ARMSBGX, 0, armsbg);
 
-      // killough 3/7/98: make face background change with displayplayer
-      if (netgame)
-        V_DrawPatch(ST_FX, 0, BG, faceback[displayplayer]);
+    // killough 3/7/98: make face background change with displayplayer
+    if (netgame)
+        V_DrawPatch(ST_FX, 0, faceback[displayplayer]);
 
-      // [crispy] copy entire SCREENWIDTH, to preserve the pattern
-      // to the left and right of the status bar in widescren mode
-      if (!force)
-      {
-        V_CopyRect(ST_X, 0, BG, SCREENWIDTH, ST_HEIGHT, ST_X, ST_Y, FG);
-      }
-      else
-      {
-        if (WIDESCREENDELTA > 0 && !st_firsttime)
+    V_RestoreBuffer();
+
+    // [crispy] copy entire video.unscaledw, to preserve the pattern to the left
+    // and right of the status bar in widescren mode
+    if (!force)
+    {
+        V_CopyRect(ST_X, 0, st_backing_screen,
+                   video.unscaledw, ST_HEIGHT,
+                   ST_X, ST_Y);
+    }
+    else
+    {
+        if (video.deltaw > 0 && !st_firsttime)
         {
-          V_CopyRect(0, 0, BG, WIDESCREENDELTA, ST_HEIGHT, 0, ST_Y, FG);
-          V_CopyRect(ORIGWIDTH + WIDESCREENDELTA, 0, BG, WIDESCREENDELTA, ST_HEIGHT, ORIGWIDTH + WIDESCREENDELTA, ST_Y, FG);
+            V_CopyRect(0, 0, st_backing_screen,
+                       video.deltaw, ST_HEIGHT,
+                       0, ST_Y);
+            V_CopyRect(SCREENWIDTH + video.deltaw, 0, st_backing_screen,
+                       video.deltaw, ST_HEIGHT,
+                       SCREENWIDTH + video.deltaw, ST_Y);
         }
-      }
-
     }
 }
 
-
 // Respond to keyboard input events,
 //  intercept cheats.
 boolean ST_Responder(event_t *ev)
@@ -682,6 +689,65 @@ void ST_updateFaceWidget(void)
 }
 
 int sts_traditional_keys; // killough 2/28/98: traditional status bar keys
+int hud_blink_keys; // [crispy] blinking key or skull in the status bar
+
+void ST_SetKeyBlink(player_t* player, int blue, int yellow, int red)
+{
+  int i;
+  // Init array with args to iterate through
+  const int keys[3] = { blue, yellow, red };
+
+  player->keyblinktics = KEYBLINKTICS;
+
+  for (i = 0; i < 3; i++)
+  {
+    if (   ((keys[i] == KEYBLINK_EITHER) && !(player->cards[i] || player->cards[i+3]))
+        || ((keys[i] == KEYBLINK_CARD)   && !(player->cards[i]))
+        || ((keys[i] == KEYBLINK_SKULL)  && !(player->cards[i+3]))
+        || ((keys[i] == KEYBLINK_BOTH)   && !(player->cards[i] && player->cards[i+3])))
+    {
+      player->keyblinkkeys[i] = keys[i];
+    }
+    else
+    {
+      player->keyblinkkeys[i] = KEYBLINK_NONE;
+    }
+  }
+}
+
+int ST_BlinkKey(player_t* player, int index)
+{
+  const keyblink_t keyblink = player->keyblinkkeys[index];
+
+  if (!keyblink)
+    return KEYBLINK_NONE;
+
+  if (player->keyblinktics & KEYBLINKMASK)
+  {
+    if (keyblink == KEYBLINK_EITHER)
+    {
+      if (st_keyorskull[index] && st_keyorskull[index] != KEYBLINK_BOTH)
+      {
+        return st_keyorskull[index];
+      }
+      else if ( (player->keyblinktics & (2*KEYBLINKMASK)) &&
+               !(player->keyblinktics & (4*KEYBLINKMASK)))
+      {
+        return KEYBLINK_SKULL;
+      }
+      else
+      {
+        return KEYBLINK_CARD;
+      }
+    }
+    else
+    {
+      return keyblink;
+    }
+  }
+
+  return -1;
+}
 
 void ST_updateWidgets(void)
 {
@@ -723,6 +789,48 @@ void ST_updateWidgets(void)
         keyboxes[i] = keyboxes[i]==-1 || sts_traditional_keys ? i+3 : i+6;
     }
 
+  // [crispy] blinking key or skull in the status bar
+  if (plyr->keyblinktics)
+  {
+    if (!hud_blink_keys ||
+        !(st_classicstatusbar || (hud_displayed && hud_active > 0)))
+    {
+      plyr->keyblinktics = 0;
+    }
+    else
+    {
+      if (!(plyr->keyblinktics & (2*KEYBLINKMASK - 1)))
+        S_StartSound(NULL, sfx_itemup);
+
+      plyr->keyblinktics--;
+
+      for (i = 0; i < 3; i++)
+      {
+        switch (ST_BlinkKey(plyr, i))
+        {
+          case KEYBLINK_NONE:
+            continue;
+
+          case KEYBLINK_CARD:
+            keyboxes[i] = i;
+            break;
+
+          case KEYBLINK_SKULL:
+            keyboxes[i] = i + 3;
+            break;
+
+          case KEYBLINK_BOTH:
+            keyboxes[i] = i + 6;
+            break;
+
+          default:
+            keyboxes[i] = -1;
+            break;
+        }
+      }
+    }
+  }
+
   // refresh everything if this is him coming back to life
   ST_updateFaceWidget();
 
@@ -751,7 +859,7 @@ static int SmoothCount(int shownval, int realval)
 {
   int step = realval - shownval;
 
-  if (!smooth_counts || !step)
+  if (!hud_animated_counts || !step)
   {
     return realval;
   }
@@ -772,6 +880,7 @@ static int SmoothCount(int shownval, int realval)
 }
 
 boolean st_invul;
+static void ST_doPaletteStuff(void);
 
 void ST_Ticker(void)
 {
@@ -785,12 +894,15 @@ void ST_Ticker(void)
   st_invul = (plyr->powers[pw_invulnerability] > 4*32 ||
               plyr->powers[pw_invulnerability] & 8) ||
               plyr->cheats & CF_GODMODE;
+
+  if (!nodrawers)
+    ST_doPaletteStuff();  // Do red-/gold-shifts from damage/items
 }
 
 static int st_palette = 0;
 boolean palette_changes = true;
 
-void ST_doPaletteStuff(void)
+static void ST_doPaletteStuff(void)
 {
   int         palette;
   byte*       pal;
@@ -872,7 +984,8 @@ void ST_drawWidgets(void)
   // clear area
   if (!st_crispyhud && st_statusbaron)
   {
-    V_CopyRect(WIDESCREENDELTA, 0, BG, ST_WIDTH, ST_HEIGHT, WIDESCREENDELTA, ST_Y, FG);
+    V_CopyRect(video.deltaw, 0, st_backing_screen, ST_WIDTH, ST_HEIGHT,
+               video.deltaw, ST_Y);
   }
 
   // used by w_arms[] widgets
@@ -966,13 +1079,11 @@ void ST_Drawer(boolean fullscreen, boolean refresh)
   st_firsttime = st_firsttime || refresh || inhelpscreens;
 
   // [crispy] distinguish classic status bar with background and player face from Crispy HUD
-  st_crispyhud = crispy_hud && hud_displayed && automap_off;
+  st_crispyhud = (hud_type == HUD_TYPE_CRISPY) && hud_displayed && automap_off;
   st_classicstatusbar = st_statusbaron && !st_crispyhud;
 
   ST_MoveHud();
 
-  ST_doPaletteStuff();  // Do red-/gold-shifts from damage/items
-
   if (st_firsttime)     // If just after ST_Start(), refresh all
   {
     st_firsttime = false;
@@ -992,7 +1103,7 @@ void ST_loadGraphics(void)
   // Load the numbers, tall and short
   for (i=0;i<10;i++)
     {
-      sprintf(namebuf, "STTNUM%d", i);
+      M_snprintf(namebuf, sizeof(namebuf), "STTNUM%d", i);
       tallnum[i] = (patch_t *) W_CacheLumpName(namebuf, PU_STATIC);
       M_snprintf(namebuf, sizeof(namebuf), "STYSNUM%d", i);
       shortnum[i] = (patch_t *) W_CacheLumpName(namebuf, PU_STATIC);
@@ -1005,7 +1116,7 @@ void ST_loadGraphics(void)
   // key cards
   for (i=0;i<NUMCARDS+3;i++)  //jff 2/23/98 show both keys too
     {
-      sprintf(namebuf, "STKEYS%d", i);
+      M_snprintf(namebuf, sizeof(namebuf), "STKEYS%d", i);
       keys[i] = (patch_t *) W_CacheLumpName(namebuf, PU_STATIC);
     }
 
@@ -1015,7 +1126,7 @@ void ST_loadGraphics(void)
   // arms ownership widgets
   for (i=0;i<6;i++)
     {
-      sprintf(namebuf, "STGNUM%d", i+2);
+      M_snprintf(namebuf, sizeof(namebuf), "STGNUM%d", i+2);
 
       // gray #
       arms[i][0] = (patch_t *) W_CacheLumpName(namebuf, PU_STATIC);
@@ -1029,7 +1140,7 @@ void ST_loadGraphics(void)
   // player face backgrounds and using displayplayer to choose them:
   for (i=0; i<MAXPLAYERS; i++)
     {
-      sprintf(namebuf, "STFB%d", i);
+      M_snprintf(namebuf, sizeof(namebuf), "STFB%d", i);
       faceback[i] = (patch_t *) W_CacheLumpName(namebuf, PU_STATIC);
     }
 
@@ -1052,18 +1163,18 @@ void ST_loadGraphics(void)
       int j;
       for (j=0;j<ST_NUMSTRAIGHTFACES;j++)
         {
-          sprintf(namebuf, "STFST%d%d", i, j);
+          M_snprintf(namebuf, sizeof(namebuf), "STFST%d%d", i, j);
           faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
         }
-      sprintf(namebuf, "STFTR%d0", i);        // turn right
+      M_snprintf(namebuf, sizeof(namebuf), "STFTR%d0", i);        // turn right
       faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
-      sprintf(namebuf, "STFTL%d0", i);        // turn left
+      M_snprintf(namebuf, sizeof(namebuf), "STFTL%d0", i);        // turn left
       faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
-      sprintf(namebuf, "STFOUCH%d", i);       // ouch!
+      M_snprintf(namebuf, sizeof(namebuf), "STFOUCH%d", i);       // ouch!
       faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
-      sprintf(namebuf, "STFEVL%d", i);        // evil grin ;)
+      M_snprintf(namebuf, sizeof(namebuf), "STFEVL%d", i);        // evil grin ;)
       faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
-      sprintf(namebuf, "STFKILL%d", i);       // pissed off
+      M_snprintf(namebuf, sizeof(namebuf), "STFKILL%d", i);       // pissed off
       faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
     }
   faces[facenum++] = W_CacheLumpName("STFGOD0", PU_STATIC);
@@ -1072,7 +1183,7 @@ void ST_loadGraphics(void)
   // [FG] support face gib animations as in the 3DO/Jaguar/PSX ports
   for (i = 0; i < ST_NUMXDTHFACES; i++)
   {
-    sprintf(namebuf, "STFXDTH%d", i);
+    M_snprintf(namebuf, sizeof(namebuf), "STFXDTH%d", i);
 
     if (W_CheckNumForName(namebuf) != -1)
       faces[facenum++] = W_CacheLumpName(namebuf, PU_STATIC);
@@ -1267,7 +1378,7 @@ static void ST_MoveHud (void)
     static int odelta = 0;
 
     if (st_crispyhud && hud_active == 2)
-        distributed_delta = WIDESCREENDELTA;
+        distributed_delta = video.deltaw;
     else
         distributed_delta = 0;
 
@@ -1293,15 +1404,84 @@ void ST_Stop(void)
 {
   if (st_stopped)
     return;
-  I_SetPalette (W_CacheLumpNum (lu_palette, PU_CACHE));
+  if (!nodrawers)
+    I_SetPalette (W_CacheLumpNum (lu_palette, PU_CACHE));
   st_stopped = true;
 }
 
+static int StatusBarBufferHeight(void)
+{
+  int i;
+  int st_height = ST_HEIGHT;
+  patch_t *const patch = W_CacheLumpName("brdr_b", PU_CACHE);
+
+  if (patch && SHORT(patch->height) > st_height)
+    st_height = SHORT(patch->height);
+
+  if (sbar && SHORT(sbar->height) > st_height)
+    st_height = SHORT(sbar->height);
+
+  if (armsbg && SHORT(armsbg->height) > st_height)
+    st_height = SHORT(armsbg->height);
+
+  for (i = 0; i < MAXPLAYERS; i++)
+  {
+    if (faceback[i] && SHORT(faceback[i]->height) > st_height)
+      st_height = SHORT(faceback[i]->height);
+  }
+
+  return st_height;
+}
+
 void ST_Init(void)
 {
   ST_loadData();
+}
+
+void ST_InitRes(void)
+{
+  int height = V_ScaleY(StatusBarBufferHeight());
+
+  if (st_backing_screen)
+  {
+    Z_Free(st_backing_screen);
+  }
+
   // killough 11/98: allocate enough for hires
-  screens[4] = Z_Malloc(MAX_SCREENWIDTH*ST_HEIGHT*4, PU_STATIC, 0);
+  st_backing_screen = Z_Malloc(video.pitch * height * sizeof(*st_backing_screen), PU_STATIC, 0);
+}
+
+void ST_Warnings(void)
+{
+  int i;
+  patch_t *const patch = W_CacheLumpName("brdr_b", PU_CACHE);
+
+  if (patch && SHORT(patch->height) > ST_HEIGHT)
+  {
+    I_Printf(VB_WARNING, "ST_Init: Non-standard BRDR_B height of %d. "
+                         "Expected <= %d.", SHORT(patch->height), ST_HEIGHT);
+  }
+
+  if (sbar && SHORT(sbar->height) != ST_HEIGHT)
+  {
+    I_Printf(VB_WARNING, "ST_Init: Non-standard STBAR height of %d. "
+                         "Expected %d.", SHORT(sbar->height), ST_HEIGHT);
+  }
+
+  if (armsbg && SHORT(armsbg->height) > ST_HEIGHT)
+  {
+    I_Printf(VB_WARNING, "ST_Init: Non-standard STARMS height of %d. "
+                         "Expected <= %d.", SHORT(armsbg->height), ST_HEIGHT);
+  }
+
+  for (i = 0; i < MAXPLAYERS; i++)
+  {
+    if (faceback[i] && SHORT(faceback[i]->height) > ST_HEIGHT)
+    {
+      I_Printf(VB_WARNING, "ST_Init: Non-standard STFB%d height of %d. "
+                           "Expected <= %d.", i, SHORT(faceback[i]->height), ST_HEIGHT);
+    }
+  }
 }
 
 void ST_ResetPalette(void)
