@@ -18,18 +18,18 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
+#include "doomdef.h"
 #include "doomstat.h"
-#include "info.h"
 #include "i_system.h"
 #include "m_misc2.h"
+#include "u_mapinfo.h"
 #include "u_scanner.h"
 #include "w_wad.h"
-
-#include "u_mapinfo.h"
+#include "z_zone.h"
 
 void M_AddEpisode(const char *map, const char *gfx, const char *txt, const char *alpha);
 void M_ClearEpisodes(void);
@@ -348,6 +348,10 @@ static void UpdateMapEntry(mapentry_t *mape, mapentry_t *newe)
   {
     strcpy(mape->nextmap, newe->nextmap);
   }
+  if (newe->nextsecret[0])
+  {
+    strcpy(mape->nextsecret, newe->nextsecret);
+  }
   if (newe->music[0])
   {
     strcpy(mape->music, newe->music);
