@@ -17,16 +17,20 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <ctype.h>
 #include <fcntl.h>
+#include <stdlib.h>
+#include <string.h>
 
-#include "doomstat.h"
-#include "i_printf.h"
-#include "m_io.h"
-
-#include "w_wad.h"
-#include "m_misc2.h" // [FG] M_BaseName()
-#include "m_swap.h"
 #include "d_main.h" // [FG] wadfiles
+#include "i_printf.h"
+#include "i_system.h"
+#include "m_array.h"
+#include "m_io.h"
+#include "m_misc2.h"
+#include "m_swap.h"
+#include "w_wad.h"
+#include "z_zone.h"
 
 //
 // GLOBALS
@@ -83,8 +87,7 @@ void ExtractFileBase(const char *path, char *dest)
 // Reload hack removed by Lee Killough
 //
 
-static int *handles;
-static int num_handles;
+static int *handles = NULL;
 
 static void W_AddFile(const char *name) // killough 1/31/98: static, const
 {
@@ -150,8 +153,7 @@ static void W_AddFile(const char *name) // killough 1/31/98: static, const
       numlumps += header.numlumps;
     }
 
-    handles = I_Realloc(handles, (num_handles + 1) * sizeof(*handles));
-    handles[num_handles++] = handle;
+    array_push(handles, handle);
 
     free(filename);           // killough 11/98
 
@@ -360,8 +362,10 @@ int W_GetNumForName (const char* name)     // killough -- const added
 //  does override all earlier ones.
 //
 
-void W_InitMultipleFiles(char *const *filenames)
+void W_InitMultipleFiles(void)
 {
+  int i;
+
   // killough 1/31/98: add predefined lumps first
 
   numlumps = num_predefined_lumps;
@@ -372,8 +376,8 @@ void W_InitMultipleFiles(char *const *filenames)
   memcpy(lumpinfo, predefined_lumps, numlumps*sizeof(*lumpinfo));
 
   // open all the files, load headers, and count lumps
-  while (*filenames)
-    W_AddFile(*filenames++);
+  for (i = 0; i < array_size(wadfiles); ++i)
+    W_AddFile(wadfiles[i]);
 
   if (!numlumps)
     I_Error ("W_InitFiles: no files found");
@@ -563,6 +567,25 @@ boolean W_IsWADLump (const int lump)
 	return lump >= 0 && lump < numlumps && lumpinfo[lump].wad_file;
 }
 
+boolean W_LumpExistsWithName(int lump, char *name)
+{
+  if (lump < 0 || lump >= numlumps)
+    return false;
+
+  if (name && strncasecmp(lumpinfo[lump].name, name, 8))
+    return false;
+
+  return true;
+}
+
+int W_LumpLengthWithName(int lump, char *name)
+{
+  if (!W_LumpExistsWithName(lump, name))
+    return 0;
+
+  return W_LumpLength(lump);
+}
+
 // [FG] avoid demo lump name collisions
 void W_DemoLumpNameCollision(char **name)
 {
@@ -608,7 +631,7 @@ void W_CloseFileDescriptors(void)
 {
   int i;
 
-  for (i = 0; i < num_handles; ++i)
+  for (i = 0; i < array_size(handles); ++i)
   {
      close(handles[i]);
   }
